/**********************************************************************
 * Defines for the Tachyon Fibre Channel Controller and the Interphase
 * (i)chip TPI. All the Tachyon specific defines can be obtained 
 * through anonymous FTP from rosegarden.external.hp.com. 
 *********************************************************************/
/* ******************************************************************
 *
 *   Hewlett-Packard Company Proprietary Information
 *
 *   This software is supplied under the terms of a license agreement
 *   or nondisclosure agreement with Hewlett-Packard Company and may
 *   not be copied or disclosed except in accordance with the terms
 *   of that agreement.
 *
 * ******************************************************************/

#ifndef _TACH_H
#define _TACH_H

#define MY_PAGE_SIZE 4096
#define REPLICATE	0xFF
#define MAX_NODES	127
#define BROADCAST	0xFFFFFF
#define LOGIN_COMPLETED	2
#define LOGIN_ATTEMPTED	1
#define LOGIN_NOT_ATTEMPTED	0
#define TRUE 		1
#define FALSE		0
#define TACHYON_LIMIT		0x01EF
#define TACHYON_OFFSET		0x200
#define ICHIP_HW_CONTROL_REG_OFF	(0x080 - TACHYON_OFFSET)
#define ICHIP_HW_STATUS_REG_OFF		(0x084 - TACHYON_OFFSET)
#define ICHIP_HW_ADDR_MASK_REG_OFF		(0x090 - TACHYON_OFFSET)

/* (i)chip Hardware Control Register defines */
#define ICHIP_HCR_RESET			0x01
#define ICHIP_HCR_DERESET		0x0
#define ICHIP_HCR_ENABLE_INTA	0x0000003E 
#define ICHIP_HCR_ENABLE_INTB	0x003E0000
#define ICHIP_HCR_IWDATA_FIFO 	0x800000

/* (i)chip Hardware Status Register defines */
#define ICHIP_HSR_INT_LATCH		0x02
//#define ICHIP_HCR_SINGLE_WR_FIFO 0x40

/* (i)chip Hardware Address Mask Register defines */
#define ICHIP_HAMR_BYTE_SWAP_ADDR_TR	0x08
#define ICHIP_HAMR_BYTE_SWAP_NO_ADDR_TR	0x04

/* NOVRAM defines */
#define IPH5526_NOVRAM_SIZE	64

/* defines for the various Qs */
#define FRAME_SIZE           2048
#define BUFF_ADDRS           8
#define OCQ_LEN              (MY_PAGE_SIZE/32)
#define IMQ_LEN              (MY_PAGE_SIZE/32)
#define MFSBQ_LEN            8
#define MFS_BUF_LEN          FRAME_SIZE
#define SFSBQ_LEN            8
#define SFS_BUF_LEN          (FRAME_SIZE + HEADER_LEN)
#define SEST_LEN             MY_PAGE_SIZE
#define SEST_BUF_LEN         1024
#define TACH_HEADER_SIZE	 64
#define NO_OF_TACH_HEADERS	 (MY_PAGE_SIZE/TACH_HEADER_SIZE)

 /* offsets for the registers that correspond to the Qs on the Tachyon */
#define OCQ_BASE_OFF         0x000  /* OCQ base register            */
#define OCQ_LEN_OFF          0x004  /* OCQ length register          */
#define OCQ_PROD_INDX_OFF    0x008  /* OCQ producer index register  */
#define OCQ_CONS_ADDR_OFF    0x00C  /* OCQ consumer index address register  */
#define HPCQ_BASE_OFF        0x040  /* HPCQ base register           */
#define HPCQ_LEN_OFF         0x044  /* HPCQ length register         */
#define HPCQ_PROD_INDX_OFF   0x048  /* HPCQ producer index register */
#define HPCQ_CONS_ADDR_OFF   0x04C  /* HPCQ consumer index address register */
#define IMQ_BASE_OFF         0x080  /* IMQ base register            */
#define IMQ_LEN_OFF          0x084  /* IMQ length register          */
#define IMQ_CONS_INDX_OFF    0x088  /* IMQ consumer index register  */
#define IMQ_PROD_ADDR_OFF    0x08C  /* IMQ producer index address register  */
#define MFSBQ_BASE_OFF       0x0C0  /* MFSBQ base register offset */
#define MFSBQ_LEN_OFF        0x0C4  /* MFSBQ length register offset */
#define MFSBQ_PROD_INDX_OFF  0x0C8  /* MFSBQ producer index register */
#define MFSBQ_CONS_INDX_OFF  0x0CC  /* MFSBQ consumer index register */  
#define MFS_BUFF_LEN_OFF     0x0D0  /* MFS Buffer length */
#define SFSBQ_BASE_OFF       0x100  /* SFSBQ base register           */
#define SFSBQ_LEN_OFF        0x104  /* SFSBQ length register         */
#define SFSBQ_PROD_INDX_OFF  0x108  /* SFSBQ producer index register */
#define SFSBQ_CONS_INDX_OFF  0x10C  /* SFSBQ consumer index register */
#define SFS_BUFF_LEN_OFF     0x110  /* SFS Buffer length */
#define SEST_BASE_OFF        0x140  /* SEST base register           */
#define SEST_LEN_OFF         0x144  /* SEST length register         */
#define SEST_BUFF_LEN_OFF    0x148  /* SEST Buffer length */

/* offsets to the other Tachyon registers */
#define TACH_CONFIG_OFF      0x184 /* TACH_CONFIG register      */
#define TACH_CONTROL_OFF     0x188 /* TACH_CONTROL register     */
#define TACH_STATUS_OFF      0x18C /* TACH_STATUS register      */
#define TACH_FLUSH_OXID_OFF  0x190 /* TACH_FLUSH OXID register  */
#define TACH_EE_CR_0_TMR_OFF 0x194 /* TACH_EE_CR_0_TMR register */
#define TACH_BB_CR_0_TMR_OFF 0x198 /* TACH_BB_CR_0_TMR register */
#define TACH_RX_FRM_ERR_OFF  0x19C /* TACH_RX_FRM_ERR register  */

/* offsets to the Frame Manager registers */
#define FM_CONFIG_OFF        0x1C0  /* FM_CONFIG register       */
#define FM_CONTROL_OFF       0x1C4  /* FM_CONTROL register      */
#define FM_STATUS_OFF        0x1C8  /* FM_STATUS register       */
#define FM_TOV_OFF           0x1CC  /* FM_TOV register          */
#define FM_LINK_STAT_1_OFF   0x1D0  /* FM_LINK_STAT_1        */
#define FM_LINK_STAT_2_OFF   0x1D4  /* FM_LINK_STAT_2        */
#define FM_WWN_HI_OFF        0x1E0  /* FM_WWN_HI register           */
#define FM_WWN_LO_OFF        0x1E4  /* FM_WWN_LO register           */
#define FM_RX_AL_PA_OFF      0x1E8  /* FM_RX_AL_PA register     */
#define FM_PRIM_REG_OFF      0x1EC

/* Completion Message Types */
#define OUTBOUND_COMPLETION      0x000
#define OUTBOUND_COMPLETION_I    0x100
#define OUT_HI_PRI_COMPLETION    0x001
#define OUT_HI_PRI_COMPLETION_I  0x101
#define INBOUND_MFS_COMPLETION   0x102
#define INBOUND_OOO_COMPLETION   0x003
#define INBOUND_SFS_COMPLETION   0x104
#define INBOUND_C1_TIMEOUT       0x105
#define INBOUND_UNKNOWN_FRAME    0x106
#define INBOUND_BUSIED_FRAME     0x006
#define SFS_BUF_WARN             0x107
#define MFS_BUF_WARN             0x108
#define IMQ_BUF_WARN             0x109
#define FRAME_MGR_INTERRUPT      0x10A
#define READ_STATUS              0x10b
#define INBOUND_SCSI_DATA        0x10c
#define INBOUND_SCSI_COMMAND     0x10d
#define BAD_SCSI_FRAME           0x10e
#define INB_SCSI_STATUS_COMPL    0x10f

/* OUTBOUND COMPLETION STATUS TYPES */

#define OCM_C1_ERROR             0x8000
#define OCM_LINK_DOWN            0x4000
#define OCM_ABORT_REQ            0x2000
#define OCM_ACK_TIMEOUT          0x1000
#define OCM_TO_OR_BAD_ALPA       0x0800
#define OCM_REJECTED             0x0400
#define OCM_RETRIEES_EXCEEDED    0x0200
#define OCM_HOST_PROG_ERR        0x0100
#define OCM_C1_OPEN              0x0080

/* TACHYON DEFINES
 */

/* Tachyon configuration register defines of masks
 */

#define NETWORK_ASSISTS      0x80000000     /* 0x80000000 for assist.   */
#define NO_NETWORK_ASSISTS   0x00000000     /* 0x0 for no assist        */
#define SCSI_ASSISTS         0x40000000     /* 0x40000000 for assist.   */
#define NO_SCSI_ASSISTS      0x00000000     /* 0x0 for no assist        */
#define RESERVED             0x00000000     
#define ENABLE_AUTO_PBSY     0x0000000      /* auto PBSY on incoming MFS*/ 
#define DISABLE_AUTO_PBSY    0x1000000      /* no auto PBSY on added MFS*/

#define BAD_SCSI_ACK         0x2000
#define NO_BAD_SCSI_ACK      0x0000
#define SCSI_COMMAND_ACK     0x1000
#define NO_SCSI_COMMAND_ACK  0x0000

#define W_STREAM1            0x000          /* Set up for 1 write       */
#define W_STREAM4            0x400          /* Set up for 4 writes      */
#define W_STREAM16           0x800          /* Set up for 16 writes     */
#define W_STREAM64           0xc00          /* Set up for 64 writes     */
#define R_STREAM1            0x000          /* Set up for 1 read        */
#define R_STREAM4            0x100          /* Set up for 4 reads       */
#define R_STREAM16           0x200          /* Set up for 16 reads      */
#define R_STREAM64           0x300          /* Set up for 64 reads      */

#define ACK_TYPE1            0x00           /* ACK 1 is 0x0             */
#define ACK_TYPE0            0x80           /* ACK 0 is 0x80.           */
#define ENABLE_OOO           0x00           /* 0x00 enables OOO reassembly*/
#define DISABLE_OOO          0x40           /* 0x40 disables reassembly */
#define ENABLE_ACKS          0x00           /* 0x00 enables auto ACKS   */
#define DISABLE_ACKS         0x20           /* 0x20 disables automatic  */
                                            /*      Debug ONLY          */
#define RETRY_ENABLE         0x00           /* 0x00 enables retries.    */
#define RETRY_DISABLE        0x10           /* 0x10 disables automatic  */ 
                                            /* retries if there's a busy*/
#define POINT_TO_POINT       0x8            /* POINT to POINT is 0x08   */
#define NOT_POINT2POINT      0x0            /* Loop or fabric is 0x00   */
#define PARITY_ENABLE        0x4            /* PARITY ENABLED is 0x04   */
#define PARITY_DISABLE       0x0            /* Disabled is 0x0.         */
#define PARITY_EVEN          0x2            /* EVEN is 0x02             */
#define PARITY_ODD           0x0            /* ODD is 0x0.              */
#define STACKED_CONNECTS     0x1            /* 0x1 fabric supports      */
#define NO_STACKED_CONNECTS  0x0            /* 0x0 should be set if not */


/* Tachyon Control Register defines
 */
#define SOFTWARE_RESET       0x80000000
#define CLR_DEF_P_BSY        0x10
#define SCSI_FREEZE          0x8
#define SCSI_UNFREEZE        0xFFFFFFF7
#define OCQ_RESET            0x4
#define ERROR_RELEASE        0x2
#define STATUS_REQUEST       0x1

/* Tachyon Status Register defines
 */
#define HI_PRI_RESET_STAT    0x40
#define OUTB_FIFO_EMPTY      0x1000
#define FE_STATUS_MASK       0xf80
#define SCSI_FREEZE_STAT     0x40
#define OCQ_RESET_STAT       0x20
#define INB_FIFO_EMPTY       0x10
#define HPFC_300K            0x000
#define HPFC_500P            0x001
#define HPFC_500K            0x010
#define HPFC_5000            0x010
#define OSM_FROZEN           0x1


/* FRAME MANAGER DEFINES
 */

/* Frame control register defines
 */

/* FRAME MANAGER CONFIG Register defines */

#define FM_HPMC_BB_CREDIT    0x20000
#define FM_FABRIC_BB_CREDIT  0x20000
#define FM_PT2PT_BB_CR       0x10000
#define FM_LOOP_BB_CR        0x00000
#define FM_NPORT_CONNECTION  0x8000 /* 0x8000 = force NPORT config        */
                                    /* loop otherwise                     */
#define FM_ENABLE_EXT_LB     0x2000 /* Enable external loopback           */
#define FM_ENABLE_INT_LB     0x4000 /* Enable internal loopback           */
#define FM_TIMER_DISABLE     0x800  /* Value to disable timer             */
#define FM_FABRIC_ADDRESS    0x400  /* ALPA contains a previously         */
                                    /* assigned FABRIC address            */
#define FM_PREV_ADDRESS      0x200  /* ALPA contains a previous           */
                                    /* address assigned in loop init      */
#define FM_HARD_ADDRESS      0x100  /* ALPA contains a preferred address  */
#define FM_SOFT_ADDRESS      0x80   /* participate in ALPA assignment     */
#define FM_RESPOND_FABRIC    0x20   /* Respond to the fabric address of 0 */
#define FM_INIT_FABRIC       0x10   /* Initialize as a fabric             */
#define FM_FABRIC_LOGIN      0x8    /* Login required                     */

#define NON_PARTICIPATING    0xfffff87f
#define FM_NPORT_MASK        0xffff7fff

/* Frame Manager Control register defines */

#define FMCNTL_DC              0x10   /* Don't close loop request     */
#define FMCNTL_CL              0x08   /* Force a loop close           */
#define FMCNTL_CMD_NOP         0x00   /* NOP                          */
#define FMCNTL_CMD_HOST_CTRL   0x02   /* Host Control                 */
#define FMCNTL_CMD_EXIT_HOST   0x03   /* Exit Host Control            */
#define FMCNTL_CMD_LINK_RESET  0x04   /* Force link reset             */
#define FMCNTL_CMD_OFFLINE     0x05   /* Force frame manager offline  */
#define FMCNTL_CMD_INITIALIZE  0x06   /* Force frame manger to init   */
#define FMCNTL_CMD_CLEAR_LF    0x07   /* Transition out of failure state  */
#define FMCNTL_CMD_SQ          0x40   /* Send Primitive Sequence      */
#define FMCNTL_CMD_SP          0x20   /* Send Primitive               */

/* Tachyon Frame manager status register defines
 */
#define FM_STATUS_LP         0x80000000
#define FM_STATUS_TP         0x40000000
#define FM_STATUS_NP         0x20000000
#define FM_STATUS_PAR_ID     0x18000000
#define FM_STATUS_LFLT       0x04000000
#define FM_STATUS_OS         0x02000000
#define FM_STATUS_LS         0x01000000
#define FM_STATUS_NOS_OLS    0x00080000
#define FM_STATUS_LST        0x00040000
#define FM_STATUS_LIPF       0x00020000
#define FM_STATUS_BA         0x00010000
#define FM_STATUS_PR_RXD     0x00008000
#define FM_STATUS_PR_SENT    0x00004000
#define FM_STATUS_LG         0x00002000
#define FM_STATUS_LF         0x00001000
#define FM_STATUS_CE         0x00000800
#define FM_STATUS_EW         0x00000400
#define FM_STATUS_LUP        0x00000200
#define FM_STATUS_LDWN       0x00000100
#define FM_STATUS_MONITORING 0x80000000
#define FM_STATUS_ARBITRATING 0x00000010
#define FM_STATUS_ARB_WON	 0x00000020
#define FM_STATUS_OPEN		 0x00000030
#define FM_STATUS_OPENED	 0x00000040
#define FM_STATUS_TX_CLS	 0x00000050
#define FM_STATUS_RX_CLS	 0x00000060
#define FM_STATUS_TRANSFER	 0x00000070
#define FM_STATUS_INITIALIZING 0x00000080
#define FM_STATUS_LOOP_FAIL	 0x000000d0
#define FM_STATUS_OLD_PORT	 0x000000F0
#define PORT_STATE_ACTIVE    0x0000000F
#define PORT_STATE_OFFLINE	 0x00000000
#define PORT_STATE_LF1		 0x00000009
#define PORT_STATE_LF2		 0x0000000A

/* General Frame Manager defines
 */

#define LOOP_STATUS_MASK     0x00000080

/* Frame Manager World Wide name registers defines.
 * These should be unique.  They should also be stored on the card 
 * in non-volatile memory and read off of it, so the WWN will remain
 * with the card and not the software.
 */

#define WORLD_WIDE_NAME_LOW  fi->g.my_port_name_low
#define WORLD_WIDE_NAME_HIGH fi->g.my_port_name_high


/* Tachyon ODB CNTL defines
 */
#define ODB_CLASS_ONE        0x4000
#define ODB_CLASS_TWO        0x8000
#define ODB_CLASS_THREE      0xC000
#define ODB_LCK              0x2000
#define ODB_SOFC1            0x1000
#define ODB_E_C              0x800
#define ODB_NO_COMP          0x400
#define ODB_NO_INT           0x200
#define ODB_ACK0             0x100
#define ODB_FILL_ONE         0x40
#define ODB_FILL_TWO         0x80
#define ODB_FILL_THREE       0xC0
#define ODB_CONT_SEQ         0x10
#define ODB_SEQID            seq_id<<24

/* Tachyon ODB Misc defines
 */
#define ODB_C                0x8000
#define ODB_U                0x1000
#define ODB_T                0x80
#define ODB_E                0x8000
#define ODB_F                0x2000
#define ODB_EE_CR_2          0xF

/* HP Descriptor block defines
 */
#define HP_END_BIT           0x80

/* Tachyon EDB defines
 */
#define EDB_LEN				 ((8*MY_PAGE_SIZE)/8) 
#define EDB_END_BIT          0x8000
#define EDB_HEADER_BIT       0x4000
#define EDB_FRAME_BOUNDRY    0x2000
#define MAX_EDB_BUFFER_LEN   0x8000
#define LEFT_OVER_MASK       0x3

/* Tachyon Header Structure defines
 */
#define SOFC1                0x30
#define SOFI1                0x50
#define SOFN1                0x90
#define SOFI3_BROADCAST      0x40
#define SOFI2                0x60
#define SOFN2                0xA0
#define SOFI3                0x70
#define SOFF                 0x80
#define SOFN3                0xB0
#define EOFN                 0x5
#define EOFA                 0x4
#define EOFDT                0x1
#define EOFT                 0x6
#define DISABLE_CRC          0x80
#define UNFAIR               0x40
#define LOOP_CLOSE           0x20
#define TIMESTAMP_VALID      0x1
#define LOOP_CR_1            4
#define LOOP_CR_2            8
#define LOOP_CR_0            0
#define FC4_DEVICE_DATA      0
#define EXTENDED_LINK_DATA   0x20000000
#define FC4_LINK_DATA        0x30000000
#define VIDEO_DATA           0x40000000
#define BASIC_LINK_DATA      0x80000000
#define LINK_CONTROL         0xC0000000
#define UNCATEGORIZED        0x0
#define SOLICITED_DATA       0x1000000
#define UNSOLICITED_CONTROL  0x2000000
#define SOLICITED_CONTROL    0x3000000
#define UNSOLICITED_DATA     0x4000000
#define DATA_DESCRIPTOR      0x5000000
#define UNSOLICITED_COMMAND  0x6000000
#define COMMAND_STATUS       0x7000000
#define RX_ID_FIRST_SEQUENCE 0xffff
#define OX_ID_FIRST_SEQUENCE 0xffff
//#define ID_MASK              0x00ffffff
#define HEADER_LEN           32
#define NW_HEADER_LEN		 16
#define EOF_LEN              4

/* Rounting Control Defines
 */
#define RCTL_FCP_SCTL          0x01000000
#define RCTL_ELS_UCTL          0x22000000
#define RCTL_ELS_SCTL          0x23000000
#define RCTL_BASIC_NOP         0x80000000
#define RCTL_BASIC_ABTS        0x81000000
#define RCTL_BASIC_RMC         0x82000000
#define RCTL_BASIC_ACC         0x84000000
#define RCTL_BASIC_RJT         0x85000000
#define RCTL_UDATA             0x04000000
#define RCTL_RFC_UCTL          0x02000000
#define RCTL_RFC_SCTL          0x03000000
#define RCTL_FCP_CMD           0x06000000
#define RCTL_FCP_DATA          0x01000000
#define RCTL_SW		       0x02000000
#define RCTL_SW_RESP	       0x03000000

/* ELS DEFINES
 */
#define ELS_LS_RJT			0x01000000
#define ELS_ACC				0x02000000
#define ELS_PLOGI			0x03000000
#define ELS_NS_PLOGI		0x03FFFFFC /* ;) */
#define ELS_FLOGI			0x04000000
#define ELS_LOGO			0x05000000
#define ELS_GPRLO			0x24000000
#define ELS_ADISC			0x52000000
#define ELS_PDISC			0x50000000
#define ELS_PRLI			0x20100000 
#define ELS_PRLO			0x21000000
#define ELS_SCR				0x62000000
#define ELS_RSCN			0x61000000
#define ELS_FARP_REQ		0x54000000
#define ELS_ABTX			0x06000000
#define ELS_ADVC			0x0D000000
#define ELS_ECHO			0x10000000
#define ELS_ESTC			0x0C000000
#define ELS_ESTS			0x0B000000
#define ELS_RCS				0x07000000
#define ELS_RES				0x08000000
#define ELS_RLS				0x0F000000
#define ELS_RRQ				0x12000000
#define ELS_RSS				0x09000000
#define ELS_RTV				0x0E000000
#define ELS_RSI				0x0A000000
#define ELS_TEST			0x11000000
#define ELS_RNC				0x53000000
#define ELS_RVCS			0x41000000
#define ELS_TPLS			0x23000000
#define ELS_GAID			0x30000000
#define ELS_FACT			0x31000000
#define ELS_FAN				0x60000000
#define ELS_FDACT			0x32000000
#define ELS_NACT			0x33000000
#define ELS_NDACT			0x34000000
#define ELS_QoSR			0x40000000
#define ELS_FDISC			0x51000000

/* Type field Defines
 */
#define TYPE_ELS               0x01000000
#define TYPE_BLS               0x00000000
#define TYPE_FC_SERVICES       0x20000000
#define TYPE_SW		       0x22000000

/* FC-4 Device Data and Link Data */
#define TYPE_LLC               0x04000000
#define TYPE_LLC_SNAP          0x05000000
#define TYPE_FCP               0x08000000
#define TYPE_FC_AL             0x23000000

/* fcntl defines in the header
 */
#define EXCHANGE_RESPONDER     0x800000
#define SEQUENCE_RESPONDER     0x400000
#define FIRST_SEQUENCE         0x200000
#define LAST_SEQUENCE          0x100000
#define END_SEQUENCE           0x80000
#define END_CONNECTION         0x40000
#define CHAINED_SEQUENCE       0x20000
#define SEQUENCE_INITIATIVE    0x10000
#define X_ID_REASSIGNED        0x8000
#define INVALIDATE_X_ID        0x4000
#define RETRANSMITTED          0x200
#define UNIDIRECTIONAL         0x100
#define RELATIVE_OFF_PRESENT   0x8

/* LS_RJT reason codes */
#define INV_LS_CMND_CODE		0x0001
#define LOGICAL_ERR			0x0003
#define LOGICAL_BUSY			0x0005
#define PROTOCOL_ERR			0x0007
#define UNABLE_TO_PERFORM		0x0009
#define CMND_NOT_SUPP			0x000B

/* LS_RJT explanation codes */
#define NO_EXPLN				0x0000
#define RECV_FIELD_SIZE			0x0700
#define CONC_SEQ				0x0900
#define REQ_NOT_SUPPORTED		0x2C00
#define INV_PAYLOAD_LEN			0x2D00

/* Payload Length defines */
#define PLOGI_LEN				116

/* Generic defines
*/

#define CONCURRENT_SEQ       0x01
#define RO_INFO_CAT	     	 0xFE
#define E_D_TOV              0x07D0 /* 2 Secs */
#define R_A_TOV		     0x2710 /* 10 Secs */
#define AL_TIME	             0x0010 /* ~15 msec */
#define TOV_VALUES           (AL_TIME << 16) | E_D_TOV
#define RT_TOV               0x64   /* 100 msec */
#define PTP_TOV_VALUES       (RT_TOV << 16) | E_D_TOV
#define SERVICE_VALID        0x8000
#define SEQUENCE_DELIVERY	 0x0800
#define C3_CONCURRENT_SEQ    0x01
#define C3_OPEN_SEQ          0x01

#define MAX_SEQ_ID			0xFF

/* LOGIN defines
 */
#define N_PORT_NAME_HIGH         fi->g.my_port_name_high
#define N_PORT_NAME_LOW          fi->g.my_port_name_low

#define NODE_NAME_HIGH           fi->g.my_node_name_high
#define NODE_NAME_LOW            fi->g.my_node_name_low
#define FC_PH_VERSION            0x2009
#define BB_CREDIT                0x00
#define PT2PT_BB_CREDIT          0x01
#define FLOGI_C_F		 0xD800 /* Alternate BB_Credit Mgmnt */ 
#define PLOGI_C_F		 0x8800 /* Continuously Increasing + Alternate BB_Credit Management */

/* Fabric defines */
#define DIRECTORY_SERVER		0xFFFFFC
#define FABRIC_CONTROLLER		0xFFFFFD
#define F_PORT				0xFFFFFE

#define PORT_NAME_LEN			8
#define NODE_NAME_LEN			8
#define FLOGI_DID			0xFFFE
#define NS_PLOGI_DID			0xFFFC

/* Fibre Channel Services defines */
#define FCS_RFC_4			0x02170000
#define FCS_GP_ID4			0x01A10000
#define FCS_ACC				0x8002
#define FCS_REJECT			0x8001


// name server Get request
#define GA_NXT                      0x0100    // Get all next
#define GI_A                        0x0101    // Get identifiers for scope
#define GPN_ID                      0x0112    // get port name
#define GNN_ID                      0x0113    // get node name
#define GCS_ID                      0x0114    // get class of service
#define GFT_ID                      0x0117    // get FC-4 types
#define GSPN_ID                     0x0118    // get symbolic port name
#define GPT_ID                      0x011A    // get Port Type
#define GIPP_ID                     0x011B    // Get IP address (port)
#define GFPN_ID                     0x011C    // Get Fabric port name
#define GHA_ID                      0x011D    // Get IP address (port)
#define GID_PN                      0x0121    // get ID
#define GIPP_PN                     0x012B    // Get IP address (port)
#define GID_NN                      0x0131    // get ID by node name
#define GIP_NN                      0x0135    // get IP
#define GIPA_NN                     0x0136    // get Initial Process Associator
#define GSNN_NN                     0x0139    // get Symbolic node name
#define GNN_IP                      0x0153    // get Node name by IP
#define GIPA_IP                     0x0156    // get get IPA
#define GID_FT                      0x0171    // get ID
#define GNN_FT                      0x0173    // Get Node Name for certain FT
#define GID_PT                      0x01A1    // get ID
#define GID_IPP                     0x01B1    // Get ID by IP address (port)
#define GPN_IPP                     0x01B2    // Get Port Name address (port)


// name server Register request
#define FCS_RPN_ID                      0x0212    // Register Port Name
#define FCS_RNN_ID                      0x0213    // Register Node Name
#define FCS_RCS_ID                      0x0214    // Register Class of Service
#define FCS_RFT_ID                      0x0217    // register FC-4 types
#define FCS_RSPN_ID                     0x0218    // register symbolic port
#define FCS_RPT_ID                      0x021A    // register port type
#define FCS_RIPP_ID                     0x021B    // register port IP address
#define FCS_RFPN_ID                     0x021C    // register fabric port name
#define FCS_RHA_ID                      0x021D    // register hard address
#define FCS_RIP_NN                      0x0235    // register IP address
#define FCS_RIPA_NN                     0x0236    // register initial process associator
#define FCS_RSNN_NN                     0x0239    // register symbolic node name

#define DA_ID                       0x0300    // de-register all

// the mask to indicate which was set in the name_server
#define FCS_IP_SET			0x01  // IP address
#define FCS_IPA_SET                     0x02  // initial process associator
#define FCS_FC4_SET                     0x04  // fc_4 types
#define FCS_SPN_SET                     0x08  // symbolic port name
#define FCS_SNN_SET                     0x10  // symbolic node name
#define FCS_PT_SET                      0x20  // port type
#define FCS_IPP_SET                     0x40  // IP address (Port)
#define FCS_FPN_SET                     0x80  // Fabric Port Name
#define FCS_HA_SET                      0x100  // hard address





/* CT Header defines */
#define FC_CT_REV			0x01000000
#define DIRECTORY_SERVER_APP		0xFC
#define NAME_SERVICE			0x02

/* Port Type defines */
#define PORT_TYPE_IP			0x05000000
#define PORT_TYPE_NX_PORTS		0x7F000000

/* SCR defines */
#define FABRIC_DETECTED_REG		0x00000001
#define N_PORT_DETECTED_REG		0x00000002
#define FULL_REGISTRATION		0x00000003
#define CLEAR_REGISTRATION		0x000000FF

#define NOT_SCSI_XID			0x0001
#define MAX_SCSI_XID			0x0FFF /* X_IDs are from 0-4095 */
#define SCSI_READ_BIT			0x4000 
#define MAX_SCSI_OXID			0x4FFF
#define ALL_OXID				0xFFFF
#define OXID_AVAILABLE			0
#define OXID_INUSE				1

/* Command structure has only one byte to address targets */
#define MAX_SCSI_TARGETS		0xFF 

#define MAX_PENDING_FRAMES		30

#define FC_SCSI_READ			0x80
#define FC_SCSI_WRITE			0x81
#define FC_ELS					0x01
#define FC_BLS                                  0x00
#define FC_IP					0x05
#define FC_FCP                                  0x08
#define FC_FCS					0x20 // fibre channel service
#define FC_SW					0x22 // switch ILS
#define FC_SW_ACC				0x92

/* SEST defines */
#define INB_SEST_VED			0xA0000000 /* V = 1, D = 1 */
#define INB_SEST_INV			0x20000000 /* V = 0, D = 1 */
#define OUTB_SEST_VED			0x80000000 /* V = 1 */
#define OUTB_SEST_LINK			0xFFFF
#define INV_SEQ_LEN				0xFFFFFFFF

/* PRLI defines */
#define PRLI_LEN				0x0014 /* 20 bytes */
#define FCP_TYPE_CODE			0x0800 /* FCP-SCSI */
#define IMAGE_PAIR				0x2000 /* establish image pair */
#define INITIATOR_FUNC			0x00000020
#define TARGET_FUNC				0x00000010
#define READ_XFER_RDY_DISABLED	0x00000002

#define NO_OF_FCP_CMNDS			(MY_PAGE_SIZE/32)

#define EDB_FREE	0
#define EDB_BUSY	1

#define SDB_SIZE				0x128
#define NO_OF_SDB_ENTRIES		((2*MY_PAGE_SIZE)/SDB_SIZE)

#define NODE_NOT_PRESENT		0x2
#define NODE_LOGGED_IN			0x1
#define NODE_LOGGED_OUT			0x0

/* defines to determine what should be returned when a SCSI frame
 * times out.
 */
#define FC_SCSI_BAD_TARGET		0xFFFE0000

/* RSCN Address formats */
#define PORT_ADDRESS_FORMAT		0x00
#define AREA_ADDRESS_FORMAT		0x01
#define DOMAIN_ADDRESS_FORMAT	0x02

/* defines used to determine whether a frame transmission should
 * be indicated by an interrupt or not.
 */
#define NO_COMP_AND_INT			0
#define INT_AND_COMP_REQ		1
#define NO_INT_COMP_REQ			2

/* defines used for FS_RJT reason and explanation codes */
#define FCS_INV_CMND_CODE               0x01
#define FCS_INV_VER_LEVEL		0x02
#define FCS_LOGICAL_ERR                 0x03
#define FCS_INV_SIZE			0x04
#define FCS_LOGICAL_BUSY                0x05
#define FCS_PROTOCOL_ERR                0x07
#define FCS_NOT_PERFORM                 0x09
#define FCS_NOT_SUPPORTED               0x0B

#define NO_PORT_ID			0x01  
#define NO_PORT_NAME			0x02
#define NO_NODE_NAME			0x03
#define NO_CS				0x04
#define NO_IP				0x05
#define NO_IPA				0x06
#define NO_FC4				0x07
#define NO_SPN				0x08
#define NO_SNN                          0x09
#define NO_PT                           0x0A
#define NO_IPP                          0x0B
#define NO_FPN                          0x0C
#define NO_HA                           0x0D
#define ACCESS_DENIED                   0x10
#define UNACCEPTABLE_ID                 0x11
#define DATABASE_EMPTY			0x12
#define NOTHING_IN_SCOPE		0x13


/************************************* E_Port stuff ************************************/

// default mine domian ID and swith name, default my priority is fixed to 0xFE
#define MY_DOMAINID                     0x09

#define MYSW_NAME_HIGH                  0x10000000
#define MYSW_NAME_LOW                   0x77000000

#define EPORT_EE_CREDIT			0x00FF

#define TYPE_DOMAIN_ID	0x01 	// used in EFP

/* the maximum possible domain_ids is 120 
 * MAX_PENDING_FRAMES/=15 * FRAME ENTRY=32WORDS = 480WORDS
 * 480 / 4 words every domain id = 120 domain_ids
 */
#define MAX_DOMAIN_IDS  100 	// largest number of domain ids in EFP payload


// default principal switch name and priority
#define PSW_NAME_HIGH	MYSW_NAME_HIGH
#define PSW_NAME_LOW	MYSW_NAME_LOW	//iphase name header



#define PSW_PRIORITY	0xFE

#define SW_RJT		0x01000000
#define SW_ACC          0x02000000
#define SW_ELP          0x10000000
#define SW_EFP          0x11000000
#define SW_AAI          0x12000000
#define SW_RDI          0x13000000
#define SW_HELLO	0x14000000
#define SW_LSU		0x15000000
#define SW_LSA		0x16000000
#define SW_BF           0x17000000
#define SW_RCF          0x18000000 
#define SW_ESC		0x30000000

#define F_S_TOV		5	// 5 seconds

/* RDI processing check */
#define DOMAINID_MINE           0x00
#define DOMAINID_SAME		0x01
#define DOMAINID_OK		0x02
#define DOMAINID_NOT_OK		0x03
#define DOMAINID_ZERO	        0x04
#define DOMAINID_USED_OUT	0x05
#define DOMAINID_NOT_IN         0x06
#define DOMAINID_CONFLICT	0xFF

#define FIRST_DOMAINID		0x01

/* defines used for from_card in different cases */
#define I_AM_PSW		0xFF
#define EFP_AFTER_NEWID_ADDED	0x98
#define FROM_MYSELF		0x99
#define EFP_TESTING_CASE13	0x90
#define EFP_TESTING_CASE14      0x91

#define MAX_FPORT_CARDS 	2

/* defines used for SW_RJT reason and explanation codes */
#define SWRJT_INVALID_CMD	0x01
#define SWRJT_INVALID_REV       0x02
#define SWRJT_LOGICAL_ERR       0x03
#define SWRJT_INVALID_PLDSZ     0x04
#define SWRJT_LOGICAL_BUSY      0x05
#define SWRJT_PROTOCOL_ERR      0x07
#define SWRJT_UNABLE_PERFORM    0x09
#define SWRJT_NOT_SUPPORTED     0x0B

#define COMMAND_IN_PROGRESS	0x19
#define DOMAINID_NOT_AVAL       0x2A 

/* for OSPF routing */
#define NUM_NODES	20
#define NONE		0xffffffff

#define HELLO_INTV	20 	//seconds
#define DEAD_INTV	80	//seconds
#define LINK_COST	1000	
#define INCARNATION	0x80000001


/* for testing tool */
#define F_BSY 	0xC6000000	// set to the r_ctl value
#define P_BSY   0xC4000000	
#define F_RJT   0xC3000000
#define P_RJT   0xC2000000



#endif /* _TACH_H */



// end of file



// end of file2



