            
/**********************************************************************
 * iol_fc_tester.c : The UNH-IOL Fibre Channel tester.
 *********************************************************************/
/**********************************************************************
Log: Xiangdong Huang 10/23/99 linux-2.2.5 version for switch
Log: Xiangdong Huang 05/12/2000 linux-2.2.9 version for switch

***********************************************************************/

/*
static const char *version =
    "t.c:v.0 05.24.99 Oct. 18 Hitachi)\n";
*/

static const char *version =
    "t.c: Version: 2.0 May 12, 2000 Xiangdong Huang\n";

#include <linux/module.h>
#include <linux/config.h>
#include <linux/kernel.h>
#include <linux/sched.h>
#include <linux/errno.h>
#include <linux/pci.h>
#include <linux/init.h>
#include <linux/mm.h>
#include <linux/delay.h>
#include <linux/skbuff.h>
#include <linux/if_arp.h>
#include <linux/timer.h>
#include <asm/system.h>
#include <asm/io.h>
#include <asm/spinlock.h>
#include <linux/netdevice.h>

#include <linux/blk.h>

/* driver specific header files */
#include "tach.h"
#include "tach_structs.h"
#include "iph5526_novram.c"

#define RUN_AT(x) (jiffies + (x))

#define DEBUG_5526 0

#define DEBUG(format, a...) { printk("%s: ", fi->name); \
							  printk(format, ##a); \
							  printk("\n");}
#if DEBUG_5526
#define DPRINTK(format, a...) {printk("%s: ", fi->name); \
							   printk(format, ##a); \
							   printk("\n");}
#define DPRINTK1(format, a...) {printk("%s: ", fi->name); \
							   printk(format, ##a); \
							   printk("\n");}
#define DPRINTK2(format, a...) {printk("%s: ", fi->name); \
							   printk(format, ##a); \
							   printk("\n");}
#else
#define DPRINTK(format, a...) {}
#define DPRINTK1(format, a...) {}
#define DPRINTK2(format, a...) {}
#endif

#if DEBUG_5526
#define ENTER(x)	{printk("%s: ", fi->name); \
					 printk("iph5526 : entering %s()\n", x);}
#define LEAVE(x)	{printk("%s: ", fi->name); \
					 printk("iph5526 : leaving %s()\n",x);}
#else
#define ENTER(x)	{}
#define LEAVE(x)	{}
#endif

#define T_MSG(format, a...) {printk(KERN_INFO "%s: ", fi->name); \
							 printk(format, ##a);\
							 printk("\n");}

#define ALIGNED_SFS_ADDR(addr) ((((unsigned long)(addr) + (SFS_BUF_LEN - 1)) & ~(SFS_BUF_LEN - 1)) - (unsigned long)(addr))
#define ALIGNED_ADDR(addr, len) ((((unsigned long)(addr) + (len - 1)) & ~(len - 1)) - (unsigned long)(addr))

//Set which HBA to E_Port (the last card will always be a E_Port, but you can change this setting)
static char EPort_Set[4] = {0, 1, 0, 0 };

/* Non-zero only if the current card is a PCI with BIOS-set IRQ. */
static unsigned int pci_irq_line = 0;
static struct {
	unsigned short vendor_id;
	unsigned short device_id;
	char *name;
}
// right now iphase has only two ids: 04 and 05
clone_list[] __initdata  = {
	{0x107e, 0x4, "Interphase Fibre Channel HBA"},
	{0x107e, 0x5, "Interphase Fibre Channel HBA"},
	{0,}
};

int iph5526_probe(struct device *dev);
static void tachyon_interrupt(int irq, void *dev_id, struct pt_regs *regs);
static void tachyon_interrupt_handler(int irq, void* dev_id, struct pt_regs* regs);
static int fcdev_init(struct device *dev);

static int iph5526_open(struct device *dev);
static int iph5526_close(struct device *dev);
static struct net_device_stats * iph5526_get_stats(struct device *dev);
static int iph5526_ioctl(struct device *dev, struct ifreq *rq, int cmd);
static int initialize_register_pointers(struct fc_info *fi);
void clean_up_memory(struct fc_info *fi);

static int  tachyon_init(struct fc_info *fi);
static int  build_queues(struct fc_info *fi);
static void build_tachyon_header(struct fc_info *fi, u_int my_id, u_int r_ctl, u_int d_id, u_int type, u_char seq_id, u_char df_ctl, u_short ox_id, u_short rx_id, char *data);
static int  get_free_header(struct fc_info *fi);
static void build_EDB(struct fc_info *fi, char *data, u_short flags, u_short len);
static int  get_free_EDB(struct fc_info *fi);
static void build_ODB(struct fc_info *fi, u_char seq_id, u_int d_id, u_int len, u_int cntl, u_short mtu, u_short ox_id, u_short rx_id, int NW_header, int int_required, u_int frame_class);
static void write_to_tachyon_registers(struct fc_info *fi);
static void reset_latch(struct fc_info *fi);
static void reset_tachyon(struct fc_info *fi, u_int value);
static void take_tachyon_offline(struct fc_info *fi);
static void read_novram(struct fc_info *fi);
static void reset_ichip(struct fc_info *fi);
static void update_OCQ_indx(struct fc_info *fi);
static void update_IMQ_indx(struct fc_info *fi, int count);
static void update_SFSBQ_indx(struct fc_info *fi);
static void update_MFSBQ_indx(struct fc_info *fi, int count);
static void update_tachyon_header_indx(struct fc_info *fi);
static void update_EDB_indx(struct fc_info *fi);
static void handle_FM_interrupt(struct fc_info *fi);
static void handle_FM_interrupt_EPort(struct fc_info *fi);
static void handle_MFS_interrupt(struct fc_info *fi);
static void handle_OOO_interrupt(struct fc_info *fi);
static void handle_SFS_interrupt(struct fc_info *fi);
static void handle_OCI_interrupt(struct fc_info *fi);
static void handle_SFS_BUF_WARN_interrupt(struct fc_info *fi);
static void handle_MFS_BUF_WARN_interrupt(struct fc_info *fi);
static void handle_IMQ_BUF_WARN_interrupt(struct fc_info *fi);
static void handle_Unknown_Frame_interrupt(struct fc_info *fi);
static void handle_Busied_Frame_interrupt(struct fc_info *fi);
static void completion_message_handler(struct fc_info *fi, u_int imq_int_type);

static void fill_login_frame(struct fc_info *fi, u_int logi);


static int 	tx_exchange(struct fc_info *fi, char *data, u_int len, u_int r_ctl, u_int type, u_int d_id, u_int mtu, int int_required, u_short ox_id, u_int frame_class);
static int 	tx_sequence(struct fc_info *fi, char *data, u_int len, u_int mtu, u_int d_id, u_short ox_id, u_short rx_id, u_char seq_id, int NW_flag, int int_required, u_int frame_class);
static void 	add_to_address_cache(struct fc_info *fi, u_int *base_ptr);
static int 	display_cache(struct fc_info *fi);

static void 	tx_logi(struct fc_info *fi, u_int logi, u_int d_id);
static void 	tx_prli(struct fc_info *fi, u_int d_id);
static void 	tx_generic(struct fc_info *fi, u_int data[], u_int r_ctl, u_int type);
static int 	fcs_query(struct fc_info *fi, u_int s_id, u_short ox_id, u_int inputArray[], u_int cmd);
static void 	tx_fcs_resp(struct fc_info *fi, u_int d_id, u_short received_ox_id, u_char, u_char);
static void 	tx_ack1(struct fc_info *fi, u_int d_id, u_short ox_id, u_short rx_id);
static void 	add_to_name_server(struct fc_info *fi, u_int s_id, u_int wwn_high, u_int wwn_low, u_int node_high, u_int node_low, u_int cs);
static void 	show_names(void);
static int 	fcs_register_id(struct fc_info *fi, u_int id, u_int inputArray[], u_int flag);
static int 	fcs_register_nn(struct fc_info *fi, u_int nn[], u_int inputArray[], u_int flag);
static struct 	fc_info *next_card(u_int);
static struct 	fc_info *find_card(u_int area_id);
  
static void 	tx_logi_acc(struct fc_info *fi, u_int logi, u_int d_id, u_short received_ox_id);
static void 	tx_ls_rjt(struct fc_info *fi, u_int d_id, u_short received_ox_id, u_short reason_code, u_short expln_code);
static u_int 	plogi_ok(struct fc_info *fi, u_int *buff_addr, int size);
static void 	tx_acc(struct fc_info *fi, u_int d_id, u_short received_ox_id);
static void 	tx_rscn(u_int);
static void 	add_to_ox_id_list(struct fc_info *fi, u_int transaction_id, u_int cmnd_code, u_char from_card);
static u_int 	remove_from_ox_id_list(struct fc_info *fi, u_short received_ox_id);

/* Timers... */
static void 	nos_ols_timer(unsigned long data);
static void 	loop_timer(unsigned long data);
static void 	elp_timer(unsigned long data);
static void 	efp_timer(unsigned long data);
static void 	bf_timer(unsigned long data);

/* E_Port stuff */
static void 	increase_oxid(struct fc_info *fi);
static void 	fl_port_init(struct fc_info *fi);
static void 	f_port_init(struct fc_info *fi);
static void 	init_fabric_paras(struct fc_info *fi, int flat);
static void 	efp_process(struct fc_info *fi, u_int *buff_addr, u_char cmd);
static void 	set_efp_timer(struct fc_info *fi);
static u_int  	check_domainID(u_int id, u_int sw_name[]);
static void   	show_domain_ids(void);
static u_int 	grab_a_domainID(void);
static void 	tx_sw_acc(struct fc_info *fi, u_int d_id, u_short received_ox_id);
static void 	tx_sw_rjt(struct fc_info *fi, u_int d_id, u_short received_ox_id, u_char reason, u_char reason_expl);
static void 	tx_elp(u_int flag, struct fc_info *fi, u_int d_id, u_short received_ox_id);
static void 	tx_esc(u_int flag, struct fc_info *fi, u_int d_id, u_short received_ox_id);
static void 	tx_efp(u_int flag, struct fc_info *fi, u_int d_id, u_short received_ox_id);
static void 	tx_aai(u_int flag, struct fc_info *fi, u_int d_id, u_short received_ox_id);
static void 	tx_rdi(u_int flag, struct fc_info *fi, u_int d_id, u_short received_ox_id, u_int id, u_int sw_name[]);
static void 	tx_bf_rcf(u_int cmd, struct fc_info *fi, u_int d_id);
static void 	add_to_domain_ids(u_int new_id, u_int sw_name[]);
static void 	eport_flush(struct fc_info *fi);
static u_int 	efp_domainID_check(u_int id, u_int sw_name[]);

/* routing stuff */
static void add_to_vlist(int order);
static void remove_from_vlist(int order);
static void print_path (struct NODE nodes[], int chNode);
static int  get_first_hop(int k);
static int  add_node(int id);
static int  get_node(int id);
static void link_cost(int from, int to, int cost);
static void show_cost(void);
static void calc_routing_table(void);
static u_int get_src_port(u_int id);
static void ospf_sample(void);
static void hello_timer(unsigned long data);
static void dead_timer(unsigned long data);
static void set_hello_timer(struct fc_info *fi);
static void set_dead_timer(struct fc_info *fi);
static void add_lsr(u_int age, u_int new_id, u_int incarnation, u_int length, struct link_info *link_data, u_int card_order);
static struct lsr_info *get_lsr(u_int id);
static void show_lsrs(void);
static void add_link_data(struct lsr_info *lsr, u_int new_id, u_int outPort, u_int cost);

static void tx_hello(struct fc_info *fi, u_int recv_id);
static void try_lsu(struct fc_info *fi, u_int flags);
static void tx_lsa(struct fc_info *fi, u_int flags, struct domain_id *ids);

/* for testing tool */
static void elp_testing(struct fc_info *fi, u_int cmd, u_int *buff_addr, u_short received_ox_id);
static void efp_testing(struct fc_info *fi, u_int cmd, u_int *buff_addr, u_short received_ox_id);
static void tx_bsy_rjt(struct fc_info *fi, u_int d_id, u_short ox_id, u_int cmd);



#if DEBUG_5526
static void print_out_registers(struct fc_info *fi, int, int);
#endif

#ifdef CONFIG_PCI
static int iph5526_probe_pci(struct device *dev);
#endif

// MAX_FC_CARDS defined in include/linux/fcdevice.h
static struct fc_info *fc[MAX_FC_CARDS+1]; 
static struct fc_info *fcsw;	// the structure which saves the E_Port stuff

__initfunc(int iph5526_probe(struct device *dev))
{
#ifdef CONFIG_PCI
	if (pci_present() && (iph5526_probe_pci(dev) == 0))
		return 0;
#endif
    return -ENODEV;
}

#ifdef CONFIG_PCI
__initfunc(static int iph5526_probe_pci(struct device *dev))
{
struct fc_info *fi = (struct fc_info*)dev->priv;
struct pci_dev *pdev = NULL;
unsigned int pci_maddr = 0;
int count = 0, i;
static int index = 0;

	ENTER("iph5526_probe_pci");
	for (i = 0; i < clone_list[i].vendor_id != 0; i++)
	while ((pdev = pci_find_device(clone_list[i].vendor_id, clone_list[i].device_id, pdev))) {
	unsigned short pci_command;
		DPRINTK("BA[0] = %x", (unsigned int)pdev->base_address[0]);
		if (index != count++)
			continue; /* we have already detected this card */
		pci_maddr = pdev->base_address[0];
		if ( (pci_maddr & PCI_BASE_ADDRESS_SPACE) != PCI_BASE_ADDRESS_SPACE_MEMORY) {
			printk("Iphase 5526 (PCI) : Cannot find proper PCI device base address.\n");
			return -ENODEV;
		}
		pci_maddr &= PCI_BASE_ADDRESS_MEM_MASK;
		DPRINTK("pci_maddr = %x", pci_maddr);
		pci_read_config_word(pdev, PCI_COMMAND, &pci_command);
			
		pci_irq_line = pdev->irq;
		printk("iph5526.c: PCI BIOS reports %s at i/o %#x, irq %d.\n", clone_list[i].name, pci_maddr, pci_irq_line);
		fi->g.mem_base = ioremap(pci_maddr & PAGE_MASK, 1024);
		
		/* We use Memory Mapped IO. The initial space contains the
		 * PCI Configuration registers followed by the (i) chip
		 * registers followed by the Tachyon registers.
		 */
		/* Thatz where (i)chip maps Tachyon Address Space.
		 */
		fi->g.tachyon_base = (u_long)fi->g.mem_base + TACHYON_OFFSET + ( pci_maddr & ~PAGE_MASK );
		//cwl DPRINTK("fi->g.tachyon_base = %x", fi->g.tachyon_base);
		if (fi->g.mem_base == NULL) {
			printk("iphase 5526 (PCI) : ioremap failed!!!\n");
			return -ENODEV;
		}	
		DPRINTK("IRQ1 = %d\n", pci_irq_line);
		printk(version);
		dev->base_addr = (long) pdev;

		if (pci_irq_line) {
		int irqval = 0;
			/* Found it, get IRQ.
			 */
			irqval = request_irq(pci_irq_line, &tachyon_interrupt, pci_irq_line ? SA_SHIRQ : 0, dev->name, dev);
			if (irqval) {
				printk("iph5526.c : Unable to get IRQ %d (irqval = %d).\n", pci_irq_line, irqval);
				kfree(dev->priv);
				dev->priv = NULL;
				return -EAGAIN;
			}
			printk("request_irq succeeded: interrupt number: %d =====================\n", pci_irq_line);
			dev->irq = pci_irq_line;
			fi->irq = pci_irq_line;
			fi->name = dev->name;
			pci_irq_line = 0;
			fcdev_init(dev);
			index++;
			fi->clone_id = clone_list[i].vendor_id;
			return 0;
		}
	}
	LEAVE("iph5526_probe_pci");
	return -ENODEV;
}
#endif  /* CONFIG_PCI */

__initfunc(static int fcdev_init(struct device *dev))
{
	dev->open = iph5526_open;
	dev->stop = iph5526_close;
	dev->get_stats = iph5526_get_stats;
	dev->set_multicast_list = NULL;
	dev->do_ioctl = iph5526_ioctl;
	return 0;
}

/* initialize tachyon and take it OnLine */
static int tachyon_init(struct fc_info *fi)
{
struct device *dev = fi->dev;
	ENTER("tachyon_init");
	if (build_queues(fi) == 0)	// the card order was set here
	{
		T_MSG("build queue failed!");
		return 0;
	}

	/* retrieve your port/node name */
	read_novram(fi);

	reset_ichip(fi);

	reset_tachyon(fi, SOFTWARE_RESET);  // thic function is calling write_to_tachyon_registers(fi);

	DPRINTK1("TACH_config_reg = %x",readl(fi->t_r.ptr_tach_config_reg));

	DPRINTK1("ocq_base_reg = %x",(u_int)fi->q.ptr_ocq_base);
	DPRINTK1("imq_base_reg = %x",(u_int)fi->q.ptr_imq_base);
	DPRINTK1("sfsbq_base_reg = %x",(u_int)fi->q.ptr_sfsbq_base);
	DPRINTK1("mfsbq_base_reg = %x",(u_int)fi->q.ptr_mfsbq_base);

	/* assign ur MAC address */
	dev->dev_addr[0] = (fi->g.my_port_name_high & 0x0000FF00) >> 8;
	dev->dev_addr[1] = fi->g.my_port_name_high;
	dev->dev_addr[2] = (fi->g.my_port_name_low & 0xFF000000) >> 24;
	dev->dev_addr[3] = (fi->g.my_port_name_low & 0x00FF0000) >> 16;
	dev->dev_addr[4] = (fi->g.my_port_name_low & 0x0000FF00) >> 8;
	dev->dev_addr[5] = fi->g.my_port_name_low;

	
	// init_fabric_paras(fi, 0);          // Fabric and E_Port parameters setting

	LEAVE("tachyon_init");
	return 1;
}

/* Build the 4 Qs - IMQ, OCQ, MFSBQ, SFSBQ */
/* Lots of dma_pages needed as Tachyon DMAs almost everything into 
 * host memory.
 */
static int build_queues(struct fc_info *fi)
{
int i,j;
u_char *addr;
	ENTER("build_queues");
	/* Initializing Queue Variables */
	fi->q.ptr_host_ocq_cons_indx = NULL;
	fi->q.ptr_host_hpcq_cons_indx = NULL;
	fi->q.ptr_host_imq_prod_indx = NULL;

	fi->q.ptr_ocq_base = NULL;
	fi->q.ocq_len = 0;
	fi->q.ocq_end = 0;
	fi->q.ocq_prod_indx = 0;

	fi->q.ptr_imq_base = NULL;
	fi->q.imq_len = 0;
	fi->q.imq_end = 0;
	fi->q.imq_cons_indx = 0;
	fi->q.imq_prod_indx = 0;

	fi->q.ptr_mfsbq_base = NULL;
	fi->q.mfsbq_len = 0;
	fi->q.mfsbq_end = 0;
	fi->q.mfsbq_prod_indx = 0;
	fi->q.mfsbq_cons_indx = 0;
	fi->q.mfsbuff_len = 0;
	fi->q.mfsbuff_end = 0;
	fi->g.mfs_buffer_count = 0;

	fi->q.ptr_sfsbq_base = NULL;
	fi->q.sfsbq_len = 0;
	fi->q.sfsbq_end = 0;
	fi->q.sfsbq_prod_indx = 0;
	fi->q.sfsbq_cons_indx = 0;
	fi->q.sfsbuff_len = 0;
	fi->q.sfsbuff_end = 0;

	fi->q.sdb_indx = 0;
	fi->q.fcp_cmnd_indx = 0;

	fi->q.ptr_edb_base = NULL;
	fi->q.edb_buffer_indx = 0;
	fi->q.ptr_tachyon_header_base = NULL;
	fi->q.tachyon_header_indx = 0;
	fi->node_info_list = NULL;
	fi->ox_id_list = NULL;
	fi->g.loop_up = FALSE;
	fi->g.ptp_up = FALSE;
	fi->g.link_up = FALSE;
	fi->g.n_port_try = FALSE;
	fi->g.dont_init = FALSE;
	fi->g.nport_timer_set = FALSE;
	fi->g.lport_timer_set = FALSE;
	fi->g.no_of_targets = 0;
	fi->g.sem = 0;
	fi->g.e_i = 0;
	
	/* switch related stuff */
	fi->g.card_order = (fi->name)[2] - 48;   // the ASCII value for 0 is 48, 
			// for example: fc1 , the card_order is 1w 

	/* build OCQ */
	if ( (fi->q.ptr_ocq_base = (u_int *)__get_dma_pages(GFP_KERNEL, 0)) == 0) {
		T_MSG("failed to get OCQ page");
		return 0;
	}
	/* set up the OCQ structures */
	for (i = 0; i < OCQ_LEN; i++)
		fi->q.ptr_odb[i] = fi->q.ptr_ocq_base + BUFF_ADDRS*i;

	/* build IMQ */
	if ( (fi->q.ptr_imq_base = (u_int *)__get_dma_pages(GFP_KERNEL, 0)) == 0) {
		T_MSG("failed to get IMQ page");
		return 0;
	}
	for (i = 0; i < IMQ_LEN; i++)
		fi->q.ptr_imqe[i] = fi->q.ptr_imq_base + BUFF_ADDRS*i;

	/* build MFSBQ */
	if ( (fi->q.ptr_mfsbq_base = (u_int *)__get_dma_pages(GFP_KERNEL, 0)) == 0) {
		T_MSG("failed to get MFSBQ page");
		return 0;
	}
	memset((char *)fi->q.ptr_mfsbq_base, 0, MFSBQ_LEN * 32);
	/* Allocate one huge chunk of memory... helps while reassembling
	 * frames.
	MFSBQ_LEN = 8; 8*8*2048 = 32 * Pages
	 */
	if ( (addr = (u_char *)__get_dma_pages(GFP_KERNEL, 5) ) == 0) {
		T_MSG("failed to get MFSBQ page");
		return 0;
	}
	/* fill in addresses of empty buffers */
	/* MFSBQ_LEN = 8 */
	for (i = 0; i < MFSBQ_LEN; i++) {
		for (j = 0; j < BUFF_ADDRS; j++) {
				*(fi->q.ptr_mfsbq_base + i*BUFF_ADDRS + j) = htonl(virt_to_bus(addr));
				addr += MFS_BUF_LEN;
		}
	}

	/* The number of entries in each MFS buffer is 8. There are 8
	 * MFS buffers. That leaves us with 4096-256 (256 = 8*32)bytes. We use them
	 * as temporary space for ELS frames. This is done to make sure that
	 * the addresses are aligned.
   	  3840/4 = 960 words; Every els entry has 32 words ==> 960/32=30 ELS frames maximum

	 */
	fi->g.els_buffer[0] = fi->q.ptr_mfsbq_base + MFSBQ_LEN*BUFF_ADDRS;
	for (i = 1; i < MAX_PENDING_FRAMES; i++)
		fi->g.els_buffer[i] = fi->g.els_buffer[i-1] + 32;
	
	/* build SFSBQ */
	if ( (fi->q.ptr_sfsbq_base = (u_int *)__get_dma_pages(GFP_KERNEL, 0)) == 0) {
		T_MSG("failed to get SFSBQ page");
		return 0;
	}
	memset((char *)fi->q.ptr_sfsbq_base, 0, SFSBQ_LEN * 32);
	/* fill in addresses of empty buffers */
	for (i = 0; i < SFSBQ_LEN; i++)
		for (j = 0; j < BUFF_ADDRS; j++){
			addr = kmalloc(SFS_BUF_LEN*2, GFP_KERNEL);
			if (addr == NULL){ 
				T_MSG("ptr_sfs_buffer : memory not allocated");
				return 0;
			}
			else {
			int offset = ALIGNED_SFS_ADDR(addr);
				memset((char *)addr, 0, SFS_BUF_LEN);
				fi->q.ptr_sfs_buffers[i*BUFF_ADDRS +j] = (u_int *)addr;
				addr += offset;
				*(fi->q.ptr_sfsbq_base + i*BUFF_ADDRS + j) = htonl(virt_to_bus(addr));
			}
		}

	/* The number of entries in each SFS buffer is 8. There are 8
	 * MFS buffers. That leaves us with 4096-256=3840 bytes. We use them
	 * as temporary space for ARP frames. This is done inorder to 
	 * support HW_Types of 0x1 and 0x6. 
	*/
	fi->g.arp_buffer = (char *)fi->q.ptr_sfsbq_base + SFSBQ_LEN*BUFF_ADDRS*4;
	
	/* build EDB */
	if ((fi->q.ptr_edb_base = (u_int *)__get_dma_pages(GFP_KERNEL, 3) ) == 0) {
		T_MSG("failed to get EDB page");
		return 0;
	}
	for (i = 0; i < EDB_LEN; i++)
		fi->q.ptr_edb[i] = fi->q.ptr_edb_base + 2*i;

	/* build SEST */
	/* We can handle 4096 concurrent transactions : limited by the fact
	 * that we can allocate only 2^5 pages using __get_dma_pages().
	 * Page Length = 4096 bytes. Each SEST entry = 32 bytes.
	 * Did this, cuz it was easier and less messier. BTW, is there any
	 * way you can allocate more than 128K of contiguous memory?
	 */

	if ((fi->q.ptr_sest_base = (u_int *)__get_dma_pages(GFP_KERNEL, 0)) == 0) {
		T_MSG("failed to get SEST page");
		return 0;
	}
	for (i = 0; i < SEST_LEN; i++)
		fi->q.ptr_sest[i] = fi->q.ptr_sest_base + BUFF_ADDRS*i;
	
	if ((fi->q.ptr_sdb_base = (u_int *)__get_dma_pages(GFP_KERNEL, 1)) == 0) {
		T_MSG("failed to get SDB page");
		return 0;
	}
	for (i = 0 ; i < NO_OF_SDB_ENTRIES; i++)
		fi->q.ptr_sdb_slot[i] = fi->q.ptr_sdb_base + 32*i;

	if ((fi->q.ptr_fcp_cmnd_base = (u_int *)__get_dma_pages(GFP_KERNEL, 0)) == 0) {
		T_MSG("failed to get FCP_CMND page");
		return 0;
	}
	for (i = 0; i < NO_OF_FCP_CMNDS; i++)
		fi->q.ptr_fcp_cmnd[i] = fi->q.ptr_fcp_cmnd_base + BUFF_ADDRS*i;

	/* Allocate space for Tachyon Header as well... 
	 */
	if ((fi->q.ptr_tachyon_header_base = (u_int *)__get_dma_pages(GFP_KERNEL, 0) ) == 0) {
		T_MSG("failed to get tachyon_header page");
		return 0;
	}
	for (i = 0; i < NO_OF_TACH_HEADERS; i++) 
		fi->q.ptr_tachyon_header[i] = fi->q.ptr_tachyon_header_base + 16*i;
	
	/* Allocate memory for indices.
	 * Indices should be aligned on 32 byte boundries. 
	 */
	fi->q.host_ocq_cons_indx = kmalloc(2*32, GFP_DMA | GFP_KERNEL);
	if (fi->q.host_ocq_cons_indx == NULL){ 
		T_MSG("fi->q.host_ocq_cons_indx : memory not allocated");
		return 0;
	}
	fi->q.ptr_host_ocq_cons_indx = fi->q.host_ocq_cons_indx; 
	if ((u_long)(fi->q.host_ocq_cons_indx) % 32)
		fi->q.host_ocq_cons_indx++;
	
	fi->q.host_hpcq_cons_indx = kmalloc(2*32, GFP_DMA | GFP_KERNEL);
	if (fi->q.host_hpcq_cons_indx == NULL){ 
		T_MSG("fi->q.host_hpcq_cons_indx : memory not allocated");
		return 0;
	}
	fi->q.ptr_host_hpcq_cons_indx= fi->q.host_hpcq_cons_indx;
	if ((u_long)(fi->q.host_hpcq_cons_indx) % 32)
		fi->q.host_hpcq_cons_indx++;

	fi->q.host_imq_prod_indx = kmalloc(2*32, GFP_DMA | GFP_KERNEL);
	if (fi->q.host_imq_prod_indx == NULL){ 
		T_MSG("fi->q.host_imq_prod_indx : memory not allocated");
		return 0;
	}
	fi->q.ptr_host_imq_prod_indx = fi->q.host_imq_prod_indx;
	if ((u_long)(fi->q.host_imq_prod_indx) % 32)
		fi->q.host_imq_prod_indx++;

	LEAVE("build_queues");
	return 1;
}

static void init_fabric_paras(struct fc_info *fi, int flag)
{
// flag: 0 means this is the first initialization, any pointer should be set to null
// flag: 1 means this is the initalization after the first, some of the pointer is not null, and should be freed 
u_int i = fi->g.card_order;
int   m, n;

        // the switch E_Port stuff only saved in the card whose card_oder == SW_CARD
        // it must be set the first E_Port, because the fcsw has sharing pointer needed by other E_Ports
                        
	for(m=0; m<MAX_FC_CARDS; m++) {
		if(EPort_Set[m] == 1)
		  	break;	
	}
	if(i == m) {
        	fcsw = fi;    
			// if this card is set to E_Port and it is first E_Port card, set to fcsw
		printk("Set HBA: %d to FCSW card! \n", i);
	}	
                        
        // the last card or the card except the first two cards are set to E_Port
                        
                if(EPort_Set[i] == 1) {   // E_Port
			T_MSG("########      HAB %x is set to E_Port      #######", i);
                        fi->g.i_am_fabric = TRUE;  
                        fi->g.i_am_E_Port = TRUE;
            
	            	fi->sm.psw_selected = 0;        // principal switch not selected yet
                        fi->sm.elp_done = 0;
			fi->sm.my_efp_done = 0;
			fi->sm.esc_done = 0; 
                        if(flag == 1) {
				if(fi->rt.hello_timer_set == 1)
                                	del_timer(&fi->rt.hello_timer);
                        	if(fi->rt.dead_timer_set == 1)
                                	del_timer(&fi->rt.dead_timer);
                        	if(fi->sm.elp_timer_set == 1)
                                	del_timer(&fi->sm.elp_timer);
                        	if(fi->sm.efp_timer_set == 1)
                                	del_timer(&fi->sm.efp_timer);
                        	if(fi->sm.bf_timer_set == 1)
                                	del_timer(&fi->sm.bf_timer);
			}
			fi->sm.elp_timer_set = 0;
                       	fi->sm.efp_timer_set = 0; 
                       	fi->sm.bf_timer_set = 0;
                        fi->rt.hello_timer_set = 0;
                        fi->rt.dead_timer_set  = 0;
        
                	fi->sm.cur_domainID  = FIRST_DOMAINID;
			fi->sm.got_domainID  = 0;
			fi->sm.testing = 0; // default not for testing tool
			fi->g.mysw_name[0] = MYSW_NAME_HIGH;
                        fi->g.mysw_name[1] = MYSW_NAME_LOW;
                        fi->g.psw_priority = PSW_PRIORITY;
                        fi->g.psw_name[0]  = PSW_NAME_HIGH;
                        fi->g.psw_name[1]  = PSW_NAME_LOW;      // principal switch priority and name
			if (flag == 0) 
				fi->domain_ids     = NULL;
			else 
			while (fi->domain_ids) {
        			struct domain_id *temp = fi->domain_ids;
               		 	fi->domain_ids = fi->domain_ids->next;
                		kfree(temp);
        		}

			fi->rt.node_num = 0;   // node number for routing 

			fi->rt.vhead = NULL;
				// this pointer already be cleared in calc_routing_table()
			for(m=0; m<NUM_NODES; m++)
				for(n=0; n<NUM_NODES; n++)
					fi->rt.AdjMatrix[m][n] = NONE;	
			fi->rt.hello_rxed = 0;	
			fi->rt.first_lsu_txed = 0;
			fi->rt.done_db_init = 0;
			fi->rt.adj_domain = NONE;
			fi->rt.adj_port   = NONE;
			if(flag == 0)
				fi->rt.lsr_list = NULL;
			else 
			while(fi->rt.lsr_list) {
        			struct lsr_info *temp = fi->rt.lsr_list;
                		while(temp->link_list) {
                			struct link_info *ltemp = temp->link_list;
                        		temp->link_list = temp->link_list->next;
                        		kfree(ltemp);
                		}
                		fi->rt.lsr_list = fi->rt.lsr_list->next;
                		kfree(temp);
        		}
                } else {  // F/FL_Port
			T_MSG("########      HAB %x is set to F/FL_Port      #######", i);
                        fi->g.i_am_fabric = TRUE;
                        fi->g.i_am_E_Port = FALSE;
			fi->name_list = NULL;
                }
                        
        fi->g.my_domainID = MY_DOMAINID;
}
                        



static void write_to_tachyon_registers(struct fc_info *fi)
{
u_int bus_addr, bus_indx_addr;

	ENTER("write_to_tachyon_registers");

	/* Clear Queues each time Tachyon is reset */
	memset((char *)fi->q.ptr_ocq_base, 0, OCQ_LEN * 32);
	memset((char *)fi->q.ptr_imq_base, 0, IMQ_LEN * 32);
	memset((char *)fi->q.ptr_edb_base, 0, EDB_LEN * 8);
	memset((char *)fi->q.ptr_sdb_base, 0, NO_OF_SDB_ENTRIES * SDB_SIZE);
	memset((char *)fi->q.ptr_tachyon_header_base, 0xFF, NO_OF_TACH_HEADERS * TACH_HEADER_SIZE);

	take_tachyon_offline(fi);
//	writel(readl(fi->t_r.ptr_tach_config_reg) | SCSI_ASSISTS | W_STREAM16 | R_STREAM64 | PARITY_EVEN | DISABLE_OOO,fi->t_r.ptr_tach_config_reg);

	writel(readl(fi->t_r.ptr_tach_config_reg) | W_STREAM16 | R_STREAM64 | PARITY_EVEN | DISABLE_OOO, fi->t_r.ptr_tach_config_reg);
            

	DPRINTK1("tach_config_reg = %x",readl(fi->t_r.ptr_tach_config_reg));

	/* Write OCQ registers */
	fi->q.ocq_prod_indx = 0;
	*(fi->q.host_ocq_cons_indx) = 0;
	
	/* The Tachyon needs to be passed the "real" address */
	bus_addr = virt_to_bus(fi->q.ptr_ocq_base);
	writel(bus_addr, fi->t_r.ptr_ocq_base_reg);
	writel(OCQ_LEN - 1, fi->t_r. ptr_ocq_len_reg);
	bus_indx_addr = virt_to_bus(fi->q.host_ocq_cons_indx);
	writel(bus_indx_addr, fi->t_r.ptr_ocq_cons_indx_reg);

	/* Write IMQ registers */
	fi->q.imq_cons_indx = 0;
	*(fi->q.host_imq_prod_indx) = 0;
	bus_addr = virt_to_bus(fi->q.ptr_imq_base);
	writel(bus_addr, fi->t_r.ptr_imq_base_reg);
	writel(IMQ_LEN - 1, fi->t_r.ptr_imq_len_reg);
	bus_indx_addr = virt_to_bus(fi->q.host_imq_prod_indx);
	writel(bus_indx_addr, fi->t_r.ptr_imq_prod_indx_reg);
	
	/* Write MFSBQ registers */
	fi->q.mfsbq_prod_indx = MFSBQ_LEN - 1;
	fi->q.mfsbuff_end = MFS_BUF_LEN - 1;
	fi->q.mfsbq_cons_indx = 0;
	bus_addr = virt_to_bus(fi->q.ptr_mfsbq_base);
	writel(bus_addr, fi->t_r.ptr_mfsbq_base_reg);
	writel(MFSBQ_LEN - 1, fi->t_r.ptr_mfsbq_len_reg);
	writel(fi->q.mfsbuff_end, fi->t_r.ptr_mfsbuff_len_reg);
	/* Do this last as tachyon will prefetch the 
	 * first entry as soon as we write to it.
	 */
	writel(fi->q.mfsbq_prod_indx, fi->t_r.ptr_mfsbq_prod_reg);

	/* Write SFSBQ registers */
	fi->q.sfsbq_prod_indx = SFSBQ_LEN - 1;
	fi->q.sfsbuff_end = SFS_BUF_LEN - 1;
	fi->q.sfsbq_cons_indx = 0;
	bus_addr = virt_to_bus(fi->q.ptr_sfsbq_base);
	writel(bus_addr, fi->t_r.ptr_sfsbq_base_reg);
	writel(SFSBQ_LEN - 1, fi->t_r.ptr_sfsbq_len_reg);
	writel(fi->q.sfsbuff_end, fi->t_r.ptr_sfsbuff_len_reg);
	/* Do this last as tachyon will prefetch the first 
	 * entry as soon as we write to it. 
	 */
	writel(fi->q.sfsbq_prod_indx, fi->t_r.ptr_sfsbq_prod_reg);

	/* Write SEST registers */
	bus_addr = virt_to_bus(fi->q.ptr_sest_base);
	writel(bus_addr, fi->t_r.ptr_sest_base_reg);
	writel(SEST_LEN - 1, fi->t_r.ptr_sest_len_reg);
	/* the last 2 bits _should_ be 1 */
	writel(SEST_BUF_LEN - 1, fi->t_r.ptr_sestbuff_len_reg);


	/* write AL_TIME & E_D_TOV into the registers */
	writel(TOV_VALUES, fi->t_r.ptr_fm_tov_reg);
	/* tell Tachyon to take a Soft assigned AL_PA */
	//writel(FM_SOFT_ADDRESS, fi->t_r.ptr_fm_config_reg);   //commented 10/30/99

	/* Read the WWN from EEPROM . But, for now we assign it here. */
	writel(WORLD_WIDE_NAME_LOW, fi->t_r.ptr_fm_wwn_low_reg);
	writel(WORLD_WIDE_NAME_HIGH, fi->t_r.ptr_fm_wwn_hi_reg);

	init_fabric_paras(fi, 0);          // Fabric and E_Port parameters setting
	
	if(EPort_Set[fi->g.card_order] == 1)    // E_Port
		 writel(FM_PT2PT_BB_CR | FM_NPORT_CONNECTION, fi->t_r.ptr_fm_config_reg);
	else writel(FM_FABRIC_ADDRESS | FM_INIT_FABRIC | FM_FABRIC_LOGIN | FM_RESPOND_FABRIC , fi->t_r.ptr_fm_config_reg);	

	DEBUG("Initial TACHYON initializing...\n");
	writel(FMCNTL_CMD_INITIALIZE, fi->t_r.ptr_fm_control_reg);

	LEAVE("write_to_tachyon_registers");
}


static void tachyon_interrupt(int irq, void* dev_id, struct pt_regs* regs)
{
struct device *dev = dev_id;
struct fc_info *fi = (struct fc_info*)dev->priv; 
u_long flags;
	spin_lock_irqsave(&fi->fc_lock, flags);
	tachyon_interrupt_handler(irq, dev_id, regs);
	spin_unlock_irqrestore(&fi->fc_lock, flags);
}

static void tachyon_interrupt_handler(int irq, void* dev_id, struct pt_regs* regs)
{
struct device *dev = dev_id;
struct fc_info *fi = (struct fc_info*)dev->priv; 

u_int *ptr_imq_entry;
u_int imq_int_type, current_IMQ_index = 0, prev_IMQ_index;
int index, no_of_entries = 0;
#if DEBUG_5526
int i;
#endif

	DPRINTK("\n");
	ENTER("tachyon_interrupt");
#if defined(__i386__)
        if (test_and_set_bit(0, (void *)&dev->interrupt)) {
                printk(KERN_ERR "%s: SMP simultanenous entry of an interrupt handler\n", dev->name);
                dev->interrupt = 0; /* Avoid halting machine */
                return;
        }
#else
        if (dev->interrupt) {
                printk(KERN_ERR "%s: Re-entering the interrupt handler\n", dev->name);
                return;
        }
        dev->interrupt = 1;
#endif

	if (dev == NULL){
		printk(KERN_WARNING "%s: irq %d for unknown device.\n", dev->name, irq);
		return;
	}

 if (fi->q.host_imq_prod_indx != NULL) {
		current_IMQ_index =  ntohl(*(fi->q.host_imq_prod_indx));
 }
 else {
 		/* _Should not_ happen */
		dev->interrupt = 0;
		T_MSG("IMQ_indx NULL. DISABLING INTERRUPTS!!!\n");
		writel(0x0, fi->i_r.ptr_ichip_hw_control_reg);
		return;
 }

 if (current_IMQ_index > fi->q.imq_cons_indx)
 	no_of_entries = current_IMQ_index - fi->q.imq_cons_indx;
 else
 if (current_IMQ_index < fi->q.imq_cons_indx)
 	no_of_entries = IMQ_LEN - (fi->q.imq_cons_indx - current_IMQ_index);

if (no_of_entries == 0) {
u_int ichip_status;
	ichip_status = readl(fi->i_r.ptr_ichip_hw_status_reg);
	if (ichip_status & 0x20) {
		/* Should _never_ happen. Might require a hard reset */
		T_MSG("Too bad... PCI Bus Error. Resetting (i)chip"); 
		reset_ichip(fi);
		T_MSG("DISABLING INTERRUPTS!!!\n");
		writel(0x0, fi->i_r.ptr_ichip_hw_control_reg);
		return;
	}
}
 prev_IMQ_index = current_IMQ_index;
 for (index = 0; index < no_of_entries; index++) {
	ptr_imq_entry = fi->q.ptr_imqe[fi->q.imq_cons_indx];
	imq_int_type = ntohl(*ptr_imq_entry);

	#if DEBUG_5526
	/*for(i = 0; i < 4; i++) {
		DPRINTK("IMQ_int_type = %x", (u_int)ntohl(*ptr_imq_entry));
		ptr_imq_entry++;
	}*/
	#endif

	completion_message_handler(fi, imq_int_type);
	if ((fi->g.link_up == FALSE) && ((imq_int_type == MFS_BUF_WARN) || (imq_int_type == SFS_BUF_WARN) || (imq_int_type == IMQ_BUF_WARN)))
		return;
	update_IMQ_indx(fi, 1);
	
	/* Check for more entries */
	current_IMQ_index =  ntohl(*(fi->q.host_imq_prod_indx));
	if (current_IMQ_index != prev_IMQ_index) {
		no_of_entries++;
		prev_IMQ_index = current_IMQ_index;
	}
	
  } /*end of for loop*/		
#if defined(__i386__)
        clear_bit(0, (void *)&dev->interrupt);
#else
        dev->interrupt = 0;
#endif          
        return;
	LEAVE("tachyon_interrupt");
}


static void handle_SFS_BUF_WARN_interrupt(struct fc_info *fi)
{
int i;
	ENTER("handle_SFS_BUF_WARN_interrupt");
	if (fi->g.link_up == FALSE) {
		reset_tachyon(fi, SOFTWARE_RESET);
		return;
	}
	/* Free up all but one entry in the Q. 
	 */
	for (i = 0; i < ((SFSBQ_LEN - 1) * BUFF_ADDRS); i++) {
		handle_SFS_interrupt(fi);
		update_IMQ_indx(fi, 1);
	}
	reset_latch(fi);
	LEAVE("handle_SFS_BUF_WARN_interrupt");
}

/* Untested_Code_Begin */ 
static void handle_MFS_BUF_WARN_interrupt(struct fc_info *fi)
{
int i;
	ENTER("handle_MFS_BUF_WARN_interrupt");
	if (fi->g.link_up == FALSE) {
		reset_tachyon(fi, SOFTWARE_RESET);
		return;
	}
	/* FIXME: free up 8 entries. 
	 */
	for (i = 0; i < BUFF_ADDRS; i++) {
		handle_MFS_interrupt(fi);
		update_IMQ_indx(fi, 1);
	}
	reset_latch(fi);
	LEAVE("handle_MFS_BUF_WARN_interrupt");
}
/*Untested_Code_End */

static void handle_IMQ_BUF_WARN_interrupt(struct fc_info *fi)
{
u_int *ptr_imq_entry;
u_int imq_int_type, current_IMQ_index = 0, temp_imq_cons_indx;
int index, no_of_entries = 0;

	ENTER("handle_IMQ_BUF_WARN_interrupt");
	if (fi->g.link_up == FALSE) {
		reset_tachyon(fi, SOFTWARE_RESET);
		return;
	}
	current_IMQ_index =  ntohl(*(fi->q.host_imq_prod_indx));

	if (current_IMQ_index > fi->q.imq_cons_indx)
 		no_of_entries = current_IMQ_index - fi->q.imq_cons_indx;
	else
		if (current_IMQ_index < fi->q.imq_cons_indx)
			no_of_entries = IMQ_LEN - (fi->q.imq_cons_indx - current_IMQ_index);
	/* We dont want to look at the same IMQ entry again 
	 */
	temp_imq_cons_indx = fi->q.imq_cons_indx + 1;
	if (no_of_entries != 0)
		no_of_entries -= 1;
	for (index = 0; index < no_of_entries; index++) {
		ptr_imq_entry = fi->q.ptr_imqe[temp_imq_cons_indx];
		imq_int_type = ntohl(*ptr_imq_entry);
		printk("imq_int_type = %x", imq_int_type);
		if (imq_int_type != IMQ_BUF_WARN)
			completion_message_handler(fi, imq_int_type);
		temp_imq_cons_indx++;
		if (temp_imq_cons_indx == IMQ_LEN)
			temp_imq_cons_indx = 0;
	} /*end of for loop*/	
	if (no_of_entries != 0)
		update_IMQ_indx(fi, no_of_entries);
	reset_latch(fi);
	LEAVE("handle_IMQ_BUF_WARN_interrupt");
}

static void completion_message_handler(struct fc_info *fi, u_int imq_int_type)
{
	switch(imq_int_type) {
		case OUTBOUND_COMPLETION:
			DPRINTK("OUTBOUND_COMPLETION message received");
			break;
		case OUTBOUND_COMPLETION_I:
			DEBUG("OUTBOUND_COMPLETION_I message received");
			handle_OCI_interrupt(fi);
			break;
		case OUT_HI_PRI_COMPLETION:
			DPRINTK("OUT_HI_PRI_COMPLETION message received");
			break;
		case OUT_HI_PRI_COMPLETION_I:
			DPRINTK("OUT_HI_PRI_COMPLETION_I message received");
			break;
		case INBOUND_MFS_COMPLETION:
			DPRINTK("INBOUND_MFS_COMPLETION message received");
			handle_MFS_interrupt(fi);
			break;
		case INBOUND_OOO_COMPLETION:
			DPRINTK("INBOUND_OOO_COMPLETION message received");
			handle_OOO_interrupt(fi);
			break;
		case INBOUND_SFS_COMPLETION:
			DPRINTK("INBOUND_SFS_COMPLETION message received");
			handle_SFS_interrupt(fi);
			break;
		case INBOUND_UNKNOWN_FRAME:
			DPRINTK("INBOUND_UNKNOWN_FRAME message received");
			handle_Unknown_Frame_interrupt(fi);
			break;
		case INBOUND_BUSIED_FRAME:
			DPRINTK("INBOUND_BUSIED_FRAME message received");
			handle_Busied_Frame_interrupt(fi);
			break;
		case FRAME_MGR_INTERRUPT:
			DPRINTK("FRAME_MGR_INTERRUPT message received");
		 	if(fi->g.i_am_E_Port == TRUE) 
			  handle_FM_interrupt_EPort(fi);
			else    
			  handle_FM_interrupt(fi);  
			break;
		case READ_STATUS:
			DPRINTK("READ_STATUS message received");
			break;
		case SFS_BUF_WARN:
			DEBUG("SFS_BUF_WARN message received");
			handle_SFS_BUF_WARN_interrupt(fi);
			break;
		case MFS_BUF_WARN:
			DEBUG("MFS_BUF_WARN message received");
			handle_MFS_BUF_WARN_interrupt(fi);
			break;
		case IMQ_BUF_WARN:
			DEBUG("IMQ_BUF_WARN message received");
			handle_IMQ_BUF_WARN_interrupt(fi);
			break;
		case INBOUND_C1_TIMEOUT:
			DPRINTK("INBOUND_C1_TIMEOUT message received");
			break;
		case BAD_SCSI_FRAME:
			printk(KERN_INFO "BAD_SCSI_FRAME message received");
			/* discarded */
			break;
		case INB_SCSI_STATUS_COMPL:
			printk(KERN_INFO "INB_SCSI_STATUS_COMPL message received");
			/* discarded */
			break;
		case INBOUND_SCSI_COMMAND:
			printk(KERN_INFO "INBOUND_SCSI_COMMAND message received");
			/* discarded */
			break;
		case INBOUND_SCSI_DATA:
			DPRINTK("INBOUND_SCSI_DATA message received");
			/* Only for targets */
			break;
		default:		
			T_MSG("DEFAULT message received, type = %x", imq_int_type);
			return;
	}

}


static void handle_FM_interrupt_EPort(struct fc_info *fi)
{
u_int fm_status;
u_int tachyon_status;

	fm_status = readl(fi->t_r.ptr_fm_status_reg);
	tachyon_status = readl(fi->t_r.ptr_tach_status_reg);
	DEBUG("%lu handle_FM_interrupt_EPort  FM_status = %x, Tachyon_status = %x", jiffies, fm_status, tachyon_status);

 	if ( (fm_status & FM_STATUS_LDWN)  ) // for E_Port stuff
        {
        DEBUG("%lu  E_Port LINK down, it will be back up ......: fm_status: %X,   tachyon_status: %X", jiffies, fm_status, tachyon_status);
                        
        }
         
        if (fm_status & FM_STATUS_LUP) {
                DEBUG("%lu    LINK UP: %X", jiffies, fm_status);
                        
                if (tachyon_status & OSM_FROZEN) {
                        reset_tachyon(fi, ERROR_RELEASE);
                        reset_tachyon(fi, OCQ_RESET);
                }
		if (((fm_status & 0xF0) == FM_STATUS_OLD_PORT) && ((fm_status & 0x0F) == PORT_STATE_ACTIVE)) {
			fi->g.ptp_up  = TRUE;  // xh added 05/13/2000
			fi->g.my_id = 0x0;
			/* In a point-to-point configuration, we expect to be
			 * connected to an F_Port. This driver does not yet support
			 * a configuration where it is connected to another N_Port
			 * directly.
			 */
		}
		fi->g.fabric_present = FALSE; 
		fi->g.ox_id = NOT_SCSI_XID;
		fi->g.my_mtu = FRAME_SIZE;
		fi->g.interrupt_count = 0;
		
		fi->g.link_up = TRUE;

		fi->name_list = NULL; //reset the name server
                fi->g.my_ddaa = fi->g.my_domainID*0x10000 + (fi->g.card_order + 1)*0x100; //increase the area ID by 1

		
		if(fi->sm.elp_done == FALSE && fi->sm.elp_timer_set == FALSE && fi->sm.testing == 0) {
				// if it's for testing tool, don't set this timer
			DEBUG("%lu  Set ELP timer: FM_stutus: %x", jiffies, fm_status);
                	fi->sm.elp_timer.function = elp_timer;
                	fi->sm.elp_timer.data = (unsigned long)fi;
                	fi->sm.elp_timer.expires = RUN_AT(2*HZ); 	// 2 sec
                	init_timer(&fi->sm.elp_timer);
                	add_timer(&fi->sm.elp_timer);
                	fi->sm.elp_timer_set = TRUE;
		} else if(fi->sm.elp_done == TRUE) {
			if(fi->sm.esc_done == FALSE) {
				tx_esc(1, fi, FABRIC_CONTROLLER, 0); 
				add_to_ox_id_list(fi, fi->g.ox_id - 1, SW_ESC, FROM_MYSELF);
			}
		}


	} // end of Link UP

	if (fm_status & FM_STATUS_EW) {
		/* Too much junk on the Link 
		 */
		//DEBUG("Elastic Store Error");
		/* Trying to clear it up by Txing PLOGI to urself */
		if (fi->g.link_up == TRUE)
			tx_logi(fi, ELS_PLOGI, fi->g.my_id); 
			// I don't need this plogi floating around   -xiangdong 05/12/2000
	}

	if (fm_status & FM_STATUS_MONITORING) {
		if (tachyon_status & OSM_FROZEN) {
			reset_tachyon(fi, ERROR_RELEASE);
			reset_tachyon(fi, OCQ_RESET);
		}
	}
	
	if (((fm_status & 0xF0) == FM_STATUS_OLD_PORT) && (((fm_status & 0x0F) == PORT_STATE_LF1) || ((fm_status & 0x0F) == PORT_STATE_LF2))) {
		DEBUG("Link Fail in OLD-PORT.");
		take_tachyon_offline(fi);
		writel(FMCNTL_CMD_CLEAR_LF, fi->t_r.ptr_fm_control_reg);
		/* write R_T_TOV & E_D_TOV into the registers */
		writel(PTP_TOV_VALUES, fi->t_r.ptr_fm_tov_reg);
		writel(FM_PT2PT_BB_CR | FM_NPORT_CONNECTION, fi->t_r.ptr_fm_config_reg);
		fi->g.n_port_try = TRUE;
		DEBUG("TACHYON initializing as N_Port...:  status: %x\n", fm_status);
		writel(FMCNTL_CMD_INITIALIZE, fi->t_r.ptr_fm_control_reg);
	}

	
	if (fm_status & FM_STATUS_LIPF){
		DEBUG("LIP(F8) Received");
	}

	if (fm_status & FM_STATUS_LF) {
		fi->name_list = NULL;
		if (fm_status & FM_STATUS_LS)
			DEBUG("Detected Loss of Signal.");
		if (fm_status & FM_STATUS_OS)
			DEBUG("Detected Loss of Synchronization.");
	}

	if (fm_status & FM_STATUS_TP) {
		/* Bad! Should not happen. Solution-> Hard Reset */
		T_MSG("Parity Error. Perform Hard Reset!");
	}

	if (fi->g.alpa_list_index >= MAX_NODES){
		fi->g.alpa_list_index = MAX_NODES;
	}

	reset_latch(fi);
	LEAVE("handle_FM_interrupt_EPort");
	return;
}



static void handle_FM_interrupt(struct fc_info *fi)
{
u_int fm_status;
u_int tachyon_status;

	ENTER("handle_FM_interrupt");
	fm_status = readl(fi->t_r.ptr_fm_status_reg);
	tachyon_status = readl(fi->t_r.ptr_tach_status_reg);
	DPRINTK("FM_status = %x, Tachyon_status = %x", fm_status, tachyon_status);
	if (fm_status & FM_STATUS_LDWN) {
		DEBUG("LINK reinitialized...");
		fm_status = readl(fi->t_r.ptr_fm_status_reg);
		fi->g.loop_up = FALSE;
		fi->g.link_up = FALSE;
		if (fi->g.ptp_up == TRUE)
			fi->g.n_port_try = FALSE;
		fi->g.ptp_up = FALSE;
		fi->name_list = NULL;  // clear the name server contents
		if ((fi->g.n_port_try == FALSE) && (fi->g.dont_init == FALSE) && (fi->g.i_am_fabric == FALSE)){
			take_tachyon_offline(fi);
			/* write AL_TIME & E_D_TOV into the registers */
			writel(TOV_VALUES, fi->t_r.ptr_fm_tov_reg);
			writel(FM_SOFT_ADDRESS, fi->t_r.ptr_fm_config_reg);
			DEBUG("In LDWN TACHYON initializing as L_Port...\n");
			writel(FMCNTL_CMD_INITIALIZE, fi->t_r.ptr_fm_control_reg);
		}
		/*the link could go down while port_discovery*/
	}

	if ((fm_status & FM_STATUS_LUP) && ((fm_status & FM_STATUS_LDWN) == 0)) {
		DEBUG("LINK UP");
		if (tachyon_status & OSM_FROZEN) {
			reset_tachyon(fi, ERROR_RELEASE);
			reset_tachyon(fi, OCQ_RESET);
		}
		del_timer(&fi->nport_timer);
		del_timer(&fi->lport_timer);
		if ((fm_status & FM_STATUS_OLD_PORT) == 0) {
			fi->g.loop_up = TRUE;
			fi->g.ptp_up = FALSE;
			fi->g.my_id = readl(fi->t_r.ptr_fm_config_reg) >> 24;
			DEBUG("My AL_PA = %x", fi->g.my_id);
		}
		else
		if (((fm_status & 0xF0) == FM_STATUS_OLD_PORT) && ((fm_status & 0x0F) == PORT_STATE_ACTIVE)) {
			fi->g.loop_up = FALSE;
			fi->g.my_id = 0x0;
			/* In a point-to-point configuration, we expect to be
			 * connected to an F_Port. This driver does not yet support
			 * a configuration where it is connected to another N_Port
			 * directly.
			 */
			if (fi->g.n_port_try == FALSE) {
				take_tachyon_offline(fi);
				/* write R_T_TOV & E_D_TOV into the registers */
				writel(PTP_TOV_VALUES, fi->t_r.ptr_fm_tov_reg);
				writel(FM_PT2PT_BB_CR | FM_NPORT_CONNECTION, fi->t_r.ptr_fm_config_reg);
				fi->g.n_port_try = TRUE;
				DEBUG("In LUP TACHYON initializing as N_Port...\n");
				writel(FMCNTL_CMD_INITIALIZE, fi->t_r.ptr_fm_control_reg);
			}
			else {
				if (fi->g.i_am_fabric == FALSE) {
					fi->g.ptp_up = TRUE;
					tx_logi(fi, ELS_FLOGI, F_PORT); 
				}
			}
		}
		fi->g.my_ddaa = 0x0;
		fi->g.fabric_present = FALSE; 
		/* We havn't sent out any Name Server Reqs */
		fi->g.alpa_list_index = 0;
		fi->g.ox_id = NOT_SCSI_XID;
		fi->g.my_mtu = FRAME_SIZE;
		fi->g.interrupt_count = 0;
		
		/* Implicitly LOGO with all logged-in nodes. 
		 */
		if (fi->node_info_list) {
		struct fc_node_info *temp_list = fi->node_info_list;
			while(temp_list) {
				temp_list->login = LOGIN_ATTEMPTED;
				temp_list = temp_list->next;
			}
		}
		else { 
			/* If Link coming up for the _first_ time or no nodes
			 * were logged in before...
			 */
			fi->g.scsi_oxid = 0;
			fi->g.seq_id = 0x00;
		}

		/* reset OX_ID table */
		while (fi->ox_id_list) {
		struct ox_id_els_map *temp = fi->ox_id_list;
			fi->ox_id_list = fi->ox_id_list->next;
			kfree(temp);
		}
		fi->ox_id_list = NULL;
		fi->g.link_up = TRUE;

		if (fi->g.i_am_fabric == TRUE) 
		{
			fi->name_list = NULL;     //reset the name server
 		  	fi->g.my_ddaa = fi->g.my_domainID*0x10000 + (fi->g.card_order + 1)*0x100; //increase the area ID by 1	
			DEBUG("FM_interrupt: set my_ddaa to: %x", fi->g.my_ddaa);
		}
	}

	if (fm_status & FM_STATUS_EW) {
		/* Too much junk on the Link 
		 */
		//DEBUG("Elastic Store Error");
		/* Trying to clear it up by Txing PLOGI to urself */
		if (fi->g.link_up == TRUE)
			tx_logi(fi, ELS_PLOGI, fi->g.my_id); 
	}

	if (fm_status & FM_STATUS_MONITORING) {
		if (tachyon_status & OSM_FROZEN) {
			reset_tachyon(fi, ERROR_RELEASE);
			reset_tachyon(fi, OCQ_RESET);
		}
	}
	
	if (fm_status & FM_STATUS_NOS_OLS){
		if (fi->g.nport_timer_set == FALSE) {
			DEBUG("NOS/OLS Received");
			DEBUG("FM_status = %x", fm_status);
			fi->nport_timer.function = nos_ols_timer;
			fi->nport_timer.data = (unsigned long)fi;
			fi->nport_timer.expires = RUN_AT((3*HZ)/100); /* 30 msec */
			init_timer(&fi->nport_timer);
			add_timer(&fi->nport_timer);
			fi->g.nport_timer_set = TRUE;
		}
	}

	if (((fm_status & 0xF0) == FM_STATUS_OLD_PORT) && (((fm_status & 0x0F) == PORT_STATE_LF1) || ((fm_status & 0x0F) == PORT_STATE_LF2))) {
		DEBUG("Link Fail in OLD-PORT.");
		take_tachyon_offline(fi);
		writel(FMCNTL_CMD_CLEAR_LF, fi->t_r.ptr_fm_control_reg);
		/* write R_T_TOV & E_D_TOV into the registers */
		writel(PTP_TOV_VALUES, fi->t_r.ptr_fm_tov_reg);
		writel(FM_PT2PT_BB_CR | FM_NPORT_CONNECTION, fi->t_r.ptr_fm_config_reg);
		fi->g.n_port_try = TRUE;
		DEBUG("TACHYON initializing as N_Port...\n");
		writel(FMCNTL_CMD_INITIALIZE, fi->t_r.ptr_fm_control_reg);
	}

	if (fm_status & FM_STATUS_LST){
		if ((fm_status & 0xF0) == FM_STATUS_ARBITRATING)
			DEBUG("ED_TOV timesout.In ARBITRATING state...");
		if ((fm_status & 0xF0) == FM_STATUS_ARB_WON)
			DEBUG("ED_TOV timesout.In ARBITRATION WON state...");
		if ((fm_status & 0xF0) == FM_STATUS_OPEN)
			DEBUG("ED_TOV timesout.In OPEN state...");
		if ((fm_status & 0xF0) == FM_STATUS_OPENED)
			DEBUG("ED_TOV timesout.In OPENED state...");
		if ((fm_status & 0xF0) == FM_STATUS_TX_CLS)
			DEBUG("ED_TOV timesout.In XMITTED CLOSE state...");
		if ((fm_status & 0xF0) == FM_STATUS_RX_CLS)
			DEBUG("ED_TOV timesout.In RECEIVED CLOSE state...");
		if ((fm_status & 0xF0) == FM_STATUS_INITIALIZING)
			DEBUG("ED_TOV timesout.In INITIALIZING state...");
		DEBUG("Initializing Loop...");
		writel(FMCNTL_CMD_INITIALIZE, fi->t_r.ptr_fm_control_reg);
	}
	
	if ((fm_status & FM_STATUS_BA) && (fi->g.loop_up == TRUE)) {
		if (tachyon_status & OSM_FROZEN) {
			reset_tachyon(fi, ERROR_RELEASE);
			reset_tachyon(fi, OCQ_RESET);
		}
		if (fi->g.i_am_fabric == TRUE)
			fi->g.my_id = 0x0;

		/* Fix for B34 */
		tx_logi(fi, ELS_PLOGI, fi->g.my_id); 

	}
	
	if (fm_status & FM_STATUS_LIPF){
		DEBUG("LIP(F8) Received");
	}

	if (fm_status & FM_STATUS_LF) {
		fi->name_list = NULL;
		if (fm_status & FM_STATUS_LS)
			DEBUG("Detected Loss of Signal.");
		if (fm_status & FM_STATUS_OS)
			DEBUG("Detected Loss of Synchronization.");
	}

	if (fm_status & FM_STATUS_TP) {
		/* Bad! Should not happen. Solution-> Hard Reset */
		T_MSG("Parity Error. Perform Hard Reset!");
	}

	if (fi->g.alpa_list_index >= MAX_NODES){
		fi->g.alpa_list_index = MAX_NODES;
	}

	reset_latch(fi);
	LEAVE("handle_FM_interrupt");
	return;
}


static void handle_OCI_interrupt(struct fc_info *fi)
{
u_int *ptr_imq_entry, transaction_id = 0;
unsigned short status, seq_count, transmitted_ox_id;
u_int tag;

	ENTER("handle_OCI_interrupt");
	ptr_imq_entry = fi->q.ptr_imqe[fi->q.imq_cons_indx];
	transaction_id = ntohl(*(ptr_imq_entry + 1));
	status = ntohl(*(ptr_imq_entry + 2)) >> 16;
	seq_count = ntohl(*(ptr_imq_entry + 3));
	DPRINTK("transaction_id= %x", transaction_id);
	tag = transaction_id & 0xFFFF0000;
	transmitted_ox_id = transaction_id;

	T_MSG("Something transmitted: transaction_id: %x, tag: %x", transaction_id, tag);
	/* The INT cound be either TIME_OUT | BAD_ALPA. But we check only
	 * for TimeOuts. Bad AL_PA will caught by FM_interrupt handler. 
	 */

		/* Frame was transmitted successfully. Check if it was an ELS
		 * frame or an IP frame or a Bad_Target_Notification frame (in
		 * case of a ptp_link). Ugly!
		 */
		if ((status == 0) && (seq_count == 0)) {
		u_int tag = transaction_id & 0xFFFF0000;
		/* Continue with port discovery after an ELS is successfully 
		 * transmitted. (status == 0). 
		 */
			DPRINTK("tag = %x", tag);
			switch(tag) {
				case ELS_FLOGI:
					/* Letz use the Name Server instead */
					fi->g.alpa_list_index = MAX_NODES;
					//add_to_ox_id_list(fi, transaction_id, tag);
					break;
				case ELS_PLOGI:
					//	add_to_ox_id_list(fi, transaction_id, tag);
					break;
				default:
					// add_to_ox_id_list(fi, transaction_id, tag);
			}
		
			if (fi->g.alpa_list_index >= MAX_NODES) {
				fi->g.alpa_list_index = MAX_NODES;
			}
		}  // end of if((status == 0) && (seq_count == 0))
	
 	reset_latch(fi);
	LEAVE("handle_OCI_interrupt");
}

/* Right now we discard OOO frames */
static void handle_OOO_interrupt(struct fc_info *fi)
{
u_int *ptr_imq_entry;
int queue_indx, offset, payload_size;
int no_of_buffers = 1; /* header is in a separate buffer */
	ptr_imq_entry = fi->q.ptr_imqe[fi->q.imq_cons_indx];
	offset = ntohl(*(ptr_imq_entry + 1)) & 0x00000007;
	queue_indx = ntohl(*(ptr_imq_entry + 1)) & 0xFFFF0000;
	queue_indx = queue_indx >> 16;
	payload_size = ntohl(*(ptr_imq_entry + 2)) - HEADER_LEN;
	/* Calculate total number of buffers */
	no_of_buffers += payload_size / MFS_BUF_LEN;
	if (payload_size % MFS_BUF_LEN)
		no_of_buffers++;
	DPRINTK2("no_of_buffers = %d", no_of_buffers);

	/* provide Tachyon will another set of buffers */
	fi->g.mfs_buffer_count += no_of_buffers;
	if (fi->g.mfs_buffer_count >= BUFF_ADDRS) {
	int count = fi->g.mfs_buffer_count / BUFF_ADDRS;
		fi->g.mfs_buffer_count -= BUFF_ADDRS * count;
		update_MFSBQ_indx(fi, count);
	}
	reset_latch(fi); 
}

static void handle_MFS_interrupt(struct fc_info *fi)
{
//xiangdong
u_int 	*ptr_imq_entry, *buff_addr;
int 	queue_indx, offset, payload_size, starting_indx, starting_offset;
int 	no_of_buffers = 1; /* header is in a separate buffer */
int 	wrap_around = FALSE, no_of_wrap_buffs = BUFF_ADDRS - 1;

struct	fc_info *fi_dest;  // destination card
u_int 	s_id, d_id, r_ctl, type_of_frame;
u_short received_ox_id, received_rx_id;
u_char	alpa, seq_id;

  	//fi->q.edb_buffer_indx = 0;
	//fi->q.mfsbq_prod_indx = 0;

	DPRINTK2("entering handle_MFS_interrupt");
	ptr_imq_entry = fi->q.ptr_imqe[fi->q.imq_cons_indx];
	offset = ntohl(*(ptr_imq_entry + 1)) & 0x00000007;
	queue_indx = ntohl(*(ptr_imq_entry + 1)) & 0xFFFF0000;
	queue_indx = queue_indx >> 16;
	DEBUG("queue_indx = %d, offset  = %d\n", queue_indx, offset);
	payload_size = ntohl(*(ptr_imq_entry + 2)) - HEADER_LEN;
	DEBUG("payload_size = %d   -------------------------------------------------", payload_size);
	/* Calculate total number of buffers */
	no_of_buffers += payload_size / MFS_BUF_LEN;
	if (payload_size % MFS_BUF_LEN)
		no_of_buffers++;
	DEBUG("no_of_buffers = %d", no_of_buffers);

	if ((no_of_buffers - 1) <= offset) {
		starting_offset = offset - (no_of_buffers - 1);
		starting_indx = queue_indx;
	}
	else {
	int temp = no_of_buffers - (offset + 1);
	int no_of_queues = temp / BUFF_ADDRS;
		starting_offset = temp % BUFF_ADDRS;
		if (starting_offset != 0) {
			no_of_wrap_buffs = starting_offset - 1; //exclude header
			starting_offset = BUFF_ADDRS - starting_offset;
			no_of_queues++;
		}
		starting_indx = queue_indx - no_of_queues;
		if (starting_indx < 0) {
			no_of_wrap_buffs -= (starting_indx + 1) * BUFF_ADDRS; 
			starting_indx = MFSBQ_LEN + starting_indx;
			wrap_around = TRUE;
		}
	}
	
	DEBUG("starting_indx = %d, starting offset = %d no_of_wrap_buffs = %d\n", starting_indx, starting_offset, no_of_wrap_buffs);
	/* Get Tachyon Header from first buffer */
	buff_addr = bus_to_virt(ntohl(*(fi->q.ptr_mfsbq_base + starting_indx*BUFF_ADDRS + starting_offset)));
	
	#if DEBUG_5526
	/*{
	int i; u_int *temp_addr = buff_addr;
		for (i=0; i<HEADER_LEN/16; i++) {
			DPRINTK2("PL[%d] = %x", i, (u_int)ntohl(*temp_addr));
			temp_addr++;
		}
	}*/
	#endif

	/* extract Type of Frame */
        r_ctl = ntohl(*(buff_addr + 2)) & 0xFF000000;
        type_of_frame = ntohl(*(buff_addr + 4)) & 0xFF000000;
        d_id = ntohl(*(buff_addr + 2)) & 0x00FFFFFF;
        s_id = ntohl(*(buff_addr + 3)) & 0x00FFFFFF;
        alpa = d_id & 0x0000FF;  // used for tachyon_header pg156
        seq_id = ntohl(*(buff_addr + 5)) >> 24;
        received_ox_id = ntohl(*(buff_addr + 6)) >> 16;
        received_rx_id = ntohl(*(buff_addr + 6)) & 0x0000FFFF;
                                       
        
        DPRINTK("S_AREA: %x", s_id & 0xFFFF00);
        DPRINTK("D_AREA: %x", d_id & 0xFFFF00);

	DEBUG("type_of_frame = %x, s_id = %x, ox_id = %x", type_of_frame, s_id, received_ox_id);


	if( (d_id & 0xFFFF00) != (s_id & 0xFFFF00) )
        {       // two different cases: 1: same domain, different ports    2: different domain
                if( (d_id & 0xFF0000) != (s_id & 0xFF0000) )
                {
                //domain IDs are different
                }
                else if ( (d_id & 0x00FF00) != (s_id & 0x00FF00) )
                {
                        //same domain ID, but different Area IDs
                fi_dest = find_card((d_id & 0x00FF00) >> 8);   // find the card using area ID
                if(!fi_dest)
                        {
                        DPRINTK("Find a area ID without corresponding card: %d", d_id);
                         return; // null, return
                        }
                //xiaoyi
                fi_dest->g.tach_header.resv = 0x00000000;
                fi_dest->g.tach_header.sof_and_eof = SOFI3 | EOFN;
                fi_dest->g.tach_header.dest_alpa = alpa;
                fi_dest->g.tach_header.lcr_and_time_stamp = 0;
                              // set the first two words
                                         
                if( get_free_EDB(fi_dest) )
                  return;
                memcpy(fi_dest->q.ptr_tachyon_header[fi_dest->q.tachyon_header_indx], &(fi_dest->g.tach_header), 8);
                //copy first two words
                memcpy(fi_dest->q.ptr_tachyon_header[fi_dest->q.tachyon_header_indx] + 2, buff_addr+2, 24);
                // move real 6 words header from buff_addr + 2
                // header was build ***************************************

        	
		buff_addr += MFS_BUF_LEN/4;   // move to the next buffer (data buffer); + word number
                                      // this is different from SFS which the header is in the same data buffer


                build_EDB(fi_dest, (char *)buff_addr, EDB_FRAME_BOUNDRY, payload_size/4);  
                memcpy(fi_dest->q.ptr_edb[fi_dest->q.edb_buffer_indx], &(fi_dest->g.edb), sizeof(EDB));
                //update_EDB_indx(fi_dest);		
                /* we can not update the EDB index here, because the ODB need the first EDB pair addree
                   if we update here, then the ODB get the next pair addrer , we will miss the first pair.
                */

                build_ODB(fi_dest, seq_id, d_id, payload_size, r_ctl, FRAME_SIZE/2, received_ox_id, received_rx_id, 0, 1, 1);
                // mtu: 2048    NW_flag=0     int_required =1     frame_class=1
                update_EDB_indx(fi_dest);                 

                              
		buff_addr += (payload_size/4) / 4;
                build_EDB(fi_dest, (char *)buff_addr, EDB_FRAME_BOUNDRY, payload_size/4);  
                memcpy(fi_dest->q.ptr_edb[fi_dest->q.edb_buffer_indx], &(fi_dest->g.edb), sizeof(EDB));
                update_EDB_indx(fi_dest);                
                
                buff_addr += (payload_size/4) / 4;
                build_EDB(fi_dest, (char *)buff_addr, EDB_FRAME_BOUNDRY, payload_size/4);
                memcpy(fi_dest->q.ptr_edb[fi_dest->q.edb_buffer_indx], &(fi_dest->g.edb), sizeof(EDB));
                update_EDB_indx(fi_dest);


                buff_addr += (payload_size/4) / 4;  
                build_EDB(fi_dest, (char *)buff_addr, EDB_END_BIT, payload_size/4);  
                memcpy(fi_dest->q.ptr_edb[fi_dest->q.edb_buffer_indx], &(fi_dest->g.edb), sizeof(EDB));
                update_EDB_indx(fi_dest);


                 
                memcpy(fi_dest->q.ptr_odb[fi_dest->q.ocq_prod_indx], &(fi_dest->g.odb), sizeof(ODB));
                if ((fi_dest->g.loop_up != TRUE) && (fi_dest->g.ptp_up != TRUE))
                  {
                    T_MSG("MFS: Fibre Channel Link not up. Dropping Exchange! fi_dest.card_order: %x", fi_dest->g.card_order);
                  }
                else
                  {
                    /* To be on the safe side, a check should be included
                     * at this point to check if we are overrunning
                     * Tachyon.
                     */  
                    update_OCQ_indx(fi_dest);
                  }
//                update_EDB_indx(fi_dest);
                update_tachyon_header_indx(fi_dest);
                
	     }  // end of if(area_id comparation ....
       }  // end of if (d_id ... s_id)

	/* provide Tachyon will another set of buffers */
	fi->g.mfs_buffer_count += no_of_buffers;
	if (fi->g.mfs_buffer_count >= BUFF_ADDRS) {
	int count = fi->g.mfs_buffer_count / BUFF_ADDRS;
		fi->g.mfs_buffer_count -= BUFF_ADDRS * count;
		update_MFSBQ_indx(fi, count);
	}
	reset_latch(fi); 
	DPRINTK2("leaving handle_MFS_interrupt");
}



static void handle_Unknown_Frame_interrupt(struct fc_info *fi)
{
//xiaoyi
u_int 	*ptr_imq_entry, *buff_addr;
u_int 	cmd, type, s_id, rctl, for_cmd;
int 	queue_indx, offset, payload_size;
u_short received_ox_id, received_rx_id;
u_int	temp, status, length;
u_int 	i, id, sw_name[2], temp_name[2];
u_char 	from_card;
u_int	recv_id, hello_intv, dead_intv;
u_int	flags, num, age, incarnation;
u_int   *base, *link_base;
u_int   link_num, neighbor, outPort, neighborPort, cost, port_id;
struct  link_info *link_list, *new_link, *last_link;
struct  domain_id *id_list, *new_id, *last_id;

        ptr_imq_entry = fi->q.ptr_imqe[fi->q.imq_cons_indx];
        offset = ntohl(*(ptr_imq_entry + 1)) & 0x00000007;
        queue_indx = ntohl(*(ptr_imq_entry + 1)) & 0xFFFF0000;
        queue_indx = queue_indx >> 16;   
        DPRINTK("queue_indx = %d, offset  = %d\n", queue_indx, offset); 
        payload_size = ntohl(*(ptr_imq_entry + 2));
        //DEBUG("payload_size = %d", payload_size);
                
        buff_addr = bus_to_virt(ntohl(*(fi->q.ptr_sfsbq_base + queue_indx*SFSBQ_LEN + offset)));
					// changed the NO_OF_ENTRIES to SFSBQ_LEN; both = 8;                        
        /* extract Type of Frame */
    
	rctl = ntohl(*(buff_addr + 2)) & 0xFF000000;
        s_id = ntohl(*(buff_addr + 3)) & 0x00FFFFFF;   
        type = ntohl(*(buff_addr + 4)) & 0xFF000000;
        received_ox_id = ntohl(*(buff_addr + 6)) >> 16;
	received_rx_id = ntohl(*(buff_addr + 6)) & 0x0000FFFF;
        cmd  = ntohl(*(buff_addr + 8)) & 0xFF000000;
	 //DEBUG("Unknown_Frame_interrupt: command: %x", ntohl(*(buff_addr + 8)));      

	switch (rctl) 
	{      
	case RCTL_SW:
		fi->g.type_of_frame = FC_SW_ACC;
		tx_ack1(fi, s_id, received_ox_id, fi->g.ox_id);  // for every command rxed, tx a ACK_1
		switch (cmd)   // tx acc to these different commands
		{
		case SW_ELP:
        		DEBUG("%lu  ELP received~~~~~~~~~~~~~~~~~~" , jiffies );
			if(fi->sm.testing >=1 && fi->sm.testing <=9) {   // if this driver is being used as a testing tool
				DEBUG("The testing code is: %d", fi->sm.testing);
				elp_testing(fi, fi->sm.testing, buff_addr, received_ox_id);
			} else {
				tx_elp(0, fi, s_id, received_ox_id); //flag ==0 : it is a ACC to ELP
				fi->sm.elp_done = 1;
				if(fi->sm.elp_timer_set == TRUE) {	
					// if received other machine's ELP, remove my elp_timer
					del_timer(&fi->sm.elp_timer);
					fi->sm.elp_timer_set = FALSE;
					DEBUG("ELP timer deleted-------******");
				}
			}
			break;
		case SW_ESC:
			DEBUG("%lu  ESC received~~~~~~~~~~~~~~~~~~" , jiffies );
			udelay(600);
                        tx_esc(0, fi, s_id, received_ox_id); //flag ==0 : it is a ACC to ESC
			fi->sm.esc_done = 1;
                        break;
                case SW_EFP:
			DEBUG("%lu  EFP received~~~~~~~~~~~~~~~~~~" , jiffies );
			 if(fi->sm.testing != 0) {   // if this driver is being used as a testing tool
                                DEBUG("The testing code is: %d", fi->sm.testing);
                                efp_testing(fi, fi->sm.testing, buff_addr, received_ox_id);
                        } else {
				if(fi->sm.efp_timer_set == TRUE) {
                                		// if received EFP during 2FS_TOV expiring, remove my efp_timer
                                	del_timer(&fi->sm.efp_timer);
                                	fi->sm.efp_timer_set = FALSE;
                                	DEBUG("EFP timer deleted-------******");
                        	}
			 	efp_process(fi, buff_addr, 1);
			}	
                        break;
                case SW_AAI:
			DEBUG("%lu  AAI received~~~~~~~~~~~~~~~~~~" , jiffies );
			tx_aai(0, fi, s_id, received_ox_id); //flag ==0 : it is a ACC to AAI
			tx_rdi(1, fi, s_id, 0, fcsw->g.my_domainID, fcsw->g.mysw_name);
			add_to_ox_id_list(fi, fi->g.ox_id - 1, SW_RDI, FROM_MYSELF);
                		// the ox_id will was increased by one in the tx_exchange function

                       break;
                case SW_RDI:
			DEBUG("%lu  RDI received~~~~~~~~~~~~~~~~~~" , jiffies );
			if(fcsw->sm.psw_card == I_AM_PSW) {
				T_MSG("I am principal switch and I will process this RDI");
				sw_name[0]  = ntohl(*(buff_addr + 9));
                        	sw_name[1]  = ntohl(*(buff_addr + 10));
                        	id  = ntohl(*(buff_addr + 11)) & 0x000000FF;
				status = check_domainID(id, sw_name);
				switch (status) {
					case DOMAINID_SAME: 	// this switch already had this id
						DEBUG("A switch is requesting for a same ID");
						tx_rdi(0, fi, s_id, received_ox_id, id, sw_name);
						break;
					case DOMAINID_OK: 	// this id is available
						tx_rdi(0, fi, s_id, received_ox_id, id, sw_name);
						add_to_domain_ids(id, sw_name);
						show_domain_ids();
						for(i=0; i<MAX_FC_CARDS; i++) {   // flooding this EFP to every E_Ports
							if(fc[i]->g.i_am_E_Port == TRUE && fc[i]->g.ptp_up == TRUE) {
								tx_efp(1, fc[i], s_id, 0);
								add_to_ox_id_list(fc[i], fc[i]->g.ox_id - 1, SW_EFP, EFP_AFTER_NEWID_ADDED);
							}
						}
						break;
					case DOMAINID_MINE:	// its name and id are same as mine: happens only in my testing
						T_MSG("Fatal error: a switch has the same switch name and id as mine");
						break; 
					case DOMAINID_CONFLICT:
						DEBUG("A switch is requesting for a different ID: transmit BF");
						tx_bf_rcf(SW_BF, fi, s_id);   //pending 
						break;
					case DOMAINID_ZERO:
					case DOMAINID_NOT_OK:
						DEBUG("Received a RDI requesting for a domain ID not available or equal zero");
						id = grab_a_domainID();
						if (id != DOMAINID_USED_OUT) {
							tx_rdi(0, fi, s_id, received_ox_id, id, sw_name);
                                                	add_to_domain_ids(id, sw_name);
                                                	show_domain_ids();
							for(i=0; i<MAX_FC_CARDS; i++) {   // flooding this EFP to every E_Ports
                                                        	if(fc[i]->g.i_am_E_Port == TRUE && fc[i]->g.ptp_up == TRUE) {
                                                                	tx_efp(1, fc[i], s_id, 0);
                                                                	add_to_ox_id_list(fc[i], fc[i]->g.ox_id - 1, SW_EFP, EFP_AFTER_NEWID_ADDED);
                                                        	}
                                                	}
						} else {
							tx_sw_rjt(fi, s_id, received_ox_id, SWRJT_UNABLE_PERFORM, DOMAINID_NOT_AVAL);
						}
						break;
					default:
				}
			}    // end of if i am principal switch
			else {	// forwrd this RDI to the psw_card
				//pending
			}
                        break;
                case SW_HELLO:
                        DEBUG("HELLO received! ");
			//show_cost();
			fi->rt.hello_rxed = 1;
                        hello_intv = ntohl(*(buff_addr + 14));
                        dead_intv  = ntohl(*(buff_addr + 15));
                        if(hello_intv != HELLO_INTV || dead_intv != DEAD_INTV) {
                                T_MSG("Incoming HELLO has unmatched Hello or Dead interval!. Dropped!");
                                break;
                        }
                        id = ntohl(*(buff_addr + 10));
                        recv_id = ntohl(*(buff_addr + 16));
			port_id = ntohl(*(buff_addr + 17));
			if(recv_id == 0xFFFFFFFF)
                            	tx_hello(fi, id);                        
			else 	
                        if(fi->rt.hello_txed == 1) // this is the response to my previous hello
                        {
                                fi->rt.hello_txed = 0;
                                if(recv_id == fi->g.my_domainID) {
                                        T_MSG("Two-way Hello communication succeeded!");
					fi->rt.adj_domain = id;
					fi->rt.adj_port   = port_id;
					// try LSU after the hello two-way communication finished
					if(fcsw->rt.first_lsu_txed == 0) {
						try_lsu(fi, 0x00000001);  
                                		try_lsu(fi, 0x00000003);
						fcsw->rt.first_lsu_txed = 1;
					}
                                }
				else 
				{
                                        T_MSG("It's weird. This Hello is not for me!");
                                }
                        } else  // this is the fresh hello received
                        {
                                tx_hello(fi, id);
                        }

                        break;
                case SW_LSU:
                        DEBUG("LSU received! ");
			flags = ntohl(*(buff_addr + 13));
			num   = ntohl(*(buff_addr + 14));
			base  = buff_addr + 15;  	// move the base address to the start address of the LSRs
			id_list = NULL;
			DEBUG("record number: %d", num);
			while(num > 0) {
				link_list = NULL;	
				age = ntohl(*(base)) & 0x0000FFFF;
				id  = ntohl(*(base + 2));
				incarnation = ntohl(*(base + 4));
				length = ntohl(*(base + 5));
				link_num = ntohl(*(base + 6));
				link_base = base + 7;
				 DEBUG("link number: %d, domain id: %x  age: %x", link_num, id, age);
				while(link_num > 0) {
					neighbor = ntohl(*(link_base));
					outPort  = ntohl(*(link_base + 1));
					neighborPort  = ntohl(*(link_base + 2));
					cost = ntohl(*(link_base + 3)) & 0x0000FFFF;
					new_link = (struct link_info *)kmalloc(sizeof(struct link_info), GFP_KERNEL);
					if (!new_link) {
                				printk(KERN_INFO "Out of memory while creating new LSR link.\n");
                				return;
        				}
					new_link->neighbor = neighbor;
					new_link->outPort  = outPort;
					new_link->neighborPort = neighborPort;
					new_link->type = 0x0100;  	// the Link Type for point-to-point is always 1
					new_link->cost = cost;
					new_link->next = NULL;	// must be set to NULL, if the next is not set to any value
					last_link = link_list;
					if(!last_link) 
						link_list = new_link;
        				else {
                        			while(last_link->next)
                                			last_link = last_link->next;
                        			last_link->next = new_link;   // put the new item at the end of the list
        				}
					link_base = link_base + 4; 	// the Link Descriptor length is 4 words
					link_num--;
					DEBUG("Link_num: %d, neighbor: %x cost %x", link_num, neighbor, cost);
				}
				new_id = (struct domain_id *)kmalloc(sizeof(struct domain_id), GFP_KERNEL);
                                if (!new_id) {
                                  	printk(KERN_INFO "Out of memory while creating new domain_id in LSU handler.\n");
                                	return;
                                }
				new_id->id = id;
				new_id->next = NULL;
				last_id = id_list;
                                if(!last_id) 
                                        id_list = new_id;
                                else {
                                        while(last_id->next)
                                        	last_id = last_id->next;
                                        last_id->next = new_id;   // put the new item at the end of the list
                                }
				T_MSG("Before add_lsr: domain id: %X", id);
				add_lsr(age, id, incarnation, length, link_list, fi->g.card_order);
				num--;
				base = base + length/4; // align to next LSR start
			}
			tx_lsa(fi, flags, id_list);
			if(flags == 0x00000001 && fi->rt.done_db_init == 0) {
				try_lsu(fi, 0x00000001);
				try_lsu(fi, 0x00000003); 
				fi->rt.done_db_init = 1;
			} 
                        break;
                case SW_LSA:
                        DEBUG("LSA received! ");
                        break;
                case SW_BF:
                case SW_RCF:
			if(cmd == SW_BF) {
				DEBUG("BF received! ");
			}
			else    {
				DEBUG("RCF received! ");
			}
			tx_sw_acc(fi, 0xFFFFFD, received_ox_id);
			for(i=0; i<MAX_FC_CARDS; i++) {   
				// flooding this BF or RCF to every E_Ports other than the one which received the BF_RCF
                		if(fc[i]->g.i_am_E_Port == TRUE && fc[i]->g.ptp_up == TRUE) {
					init_fabric_paras(fc[i], 1);
					if( fc[i]->g.card_order != i)  {
                                		tx_bf_rcf(cmd, fc[i], 0xFFFFFD);
                        			//add_to_ox_id_list(fc[i], fc[i]->g.ox_id - 1, SW_EFP, EFP_AFTER_NEWID_ADDED);
					}
        			}
			}
			if(fi->sm.bf_timer_set == 0) {
				// remember to check this, because if add a timer already existed you will cause problem.
				DEBUG("%lu  Set BF timer", jiffies);
        			fi->sm.bf_timer.function = bf_timer;
        			fi->sm.bf_timer.data = (unsigned long)fi;
        			fi->sm.bf_timer.expires = RUN_AT(2 * F_S_TOV * HZ);        // 2*F_S_TOV
        			init_timer(&fi->sm.bf_timer);
        			add_timer(&fi->sm.bf_timer);
        			fi->sm.bf_timer_set = TRUE;
			}
                        break;
		default:
		} // end of the switch(cmd)
		break;

	case RCTL_SW_RESP:
		tx_ack1(fi, s_id, received_ox_id, received_rx_id);  // for every command rxed, tx a ACK_1		
		for_cmd = remove_from_ox_id_list(fi, received_ox_id);
			// this reply is for which command
		from_card = fi->g.from_card;	// this was set during the remove_from_ox_id_list function
		switch (for_cmd)   // tx acc to these different commands
                {
                case SW_ELP:
			DEBUG("ELP_ACC received");
			take_tachyon_offline(fi);
			 writel(FM_FABRIC_ADDRESS | FM_INIT_FABRIC | FM_FABRIC_LOGIN | FM_RESPOND_FABRIC , fi->t_r.ptr_fm_config_reg);
			writel(FMCNTL_CMD_INITIALIZE, fi->t_r.ptr_fm_control_reg);
			fi->sm.elp_done = 1;
                        break;
                case SW_ESC:
			DEBUG("ESC_ACC received");
			fi->sm.esc_done = 1;
			if(fcsw->sm.testing == 0) { 	// if the driver is not acting as testing tool, it tx efp after ESC
				tx_efp(1, fi, s_id, 0);
				add_to_ox_id_list(fi, fi->g.ox_id - 1, SW_EFP, FROM_MYSELF);
			}
                        break;
                case SW_EFP:
			DEBUG("EFP_ACC received---------------------------");
			switch(from_card) {
				case I_AM_PSW:	// received the EFP which was txed after the F_S_TOV
				 	DEBUG("############### I am the principal switch ##################");
					fcsw->sm.psw_card = I_AM_PSW; 	// 0xFF means myself is the psw
					fcsw->sm.got_domainID = 1;
					tx_aai(1, fi, s_id, 0);
                        		add_to_ox_id_list(fi, fi->g.ox_id - 1, SW_AAI, FROM_MYSELF);
					tx_hello(fi, 0xFFFFFFFF);  // after I got my domain_ID, I should transmit a Hello
								   // the first hello doesn't know to whom it is transmitted
					add_lsr(1, fcsw->g.my_domainID, INCARNATION + 1, 28, NULL, fi->g.card_order);
                                                // Add myself to the LSR database
					fi->rt.hello_txed = 1;
					set_hello_timer(fi);
					set_dead_timer(fi);
					break;
				case FROM_MYSELF: 	// received the EFP tried by me 
					if(fi->sm.testing != 0) {   // if this driver is being used as a testing tool
                                		DEBUG("Don't process the EFP_ACC, the testing code is: %d", fi->sm.testing);
                        		} else {
						efp_process(fi, buff_addr, 0);
					}
					break;
				case EFP_AFTER_NEWID_ADDED:       // received the EFP txed after a new ID was added
					// doing nothing
                                        break;
				case EFP_TESTING_CASE13:
                                        DEBUG("Process the EFP_ACC, the testing code is: %d", fi->sm.testing);
                                       	payload_size = ntohl(*(buff_addr + 8)) & 0x0000FFFF; // EFP payload size
					sw_name[0] = ntohl(*(buff_addr + 10));
                        		sw_name[1] = ntohl(*(buff_addr + 11));
					if(payload_size >= 0x20) {
					    	while (fi->domain_ids) {
                                			struct domain_id *temp = fi->domain_ids;
                                			fi->domain_ids = fi->domain_ids->next;
                                 			kfree(temp);
					    	}
                        			fcsw->g.psw_name[0] = sw_name[0];
                        			fcsw->g.psw_name[1] = sw_name[1] + 10;
                        			temp_name[0] = sw_name[0];
                        			temp_name[1] = sw_name[1] + 10;
                       		 		add_to_domain_ids(5, temp_name);
                        			tx_efp(1, fi, 0xFFFFFD, received_ox_id);
					}
					break;
				case EFP_TESTING_CASE14:
                                        DEBUG("Process the EFP_ACC, the testing code is: %d", fi->sm.testing);
                                        payload_size = ntohl(*(buff_addr + 8)) & 0x0000FFFF; // EFP payload size
                                        sw_name[0] = ntohl(*(buff_addr + 10));
                                        sw_name[1] = ntohl(*(buff_addr + 11));
                                        if(payload_size >= 0x20) {   
                                                tx_efp(1, fi, 0xFFFFFD, received_ox_id);
                                        }
                                        break;

				default:
					DEBUG("Warning: EFP_ACC received from card: %x", from_card);
			}
                        break;
                case SW_AAI:
		       DEBUG("AAI_ACC received---------------------------");
                       break;
                case SW_RDI:
			if(cmd == SW_ACC) {
				DEBUG("RDI ACC received");
				DEBUG("From Card: %x", from_card);
				if(from_card == FROM_MYSELF) {   // this rdi was txed by myself
					temp = ntohl(*(buff_addr + 11)) & 0x000000FF;  // get the assigned domain ID
					fcsw->sm.got_domainID = 1;
					add_lsr(1, fcsw->g.my_domainID, INCARNATION + 1, 28, NULL, fi->g.card_order);
                                                // Add myself to the LSR database
					if(temp != fi->g.my_domainID) {  // should re-initialize all other F/FL_Ports
						for(i=0; i<MAX_FC_CARDS; i++) {
							fc[i]->g.my_domainID = (u_char)temp;
							if(fc[i]->g.i_am_E_Port == FALSE) {
								if(fc[i]->g.loop_up == TRUE) 
									fl_port_init(fc[i]);
								else 	f_port_init(fc[i]);
							}
						}
 					}
					tx_hello(fi, 0xFFFFFFFF);  // after I got my domain_ID, I should transmit a Hello
                                                                   // the first hello doesn't know to whom it is transmitted
                                        fi->rt.hello_txed = 1;
                                        set_hello_timer(fi);
                                        set_dead_timer(fi);

				} else {
					//forward the RDI_ACC to other card
				}
			} else {
				T_MSG("SW_RJT to RDI received");
			}
                        break;
                case SW_BF:
                case SW_RCF:
                        break;
                default:
                } // end of the switch(cmd)

                break;

	default:		
	} // end of outside switch
     
        /* provide Tachyon will another set of buffers */
        if (offset == (BUFF_ADDRS - 1))
                update_SFSBQ_indx(fi);
        reset_latch(fi); /* don't forget to do this! */
	LEAVE("handle_Unknown_Frame_interrupt");
}




static void handle_Busied_Frame_interrupt(struct fc_info *fi)
{
u_int *ptr_imq_entry;
int queue_indx, offset;
	ENTER("handle_Busied_Frame_interrupt");
	ptr_imq_entry = fi->q.ptr_imqe[fi->q.imq_cons_indx];
	offset = ntohl(*(ptr_imq_entry + 1)) & 0x00000007;
	queue_indx = ntohl(*(ptr_imq_entry + 1)) & 0xFFFF0000;
	queue_indx = queue_indx >> 16;
	/* We discard the "busied" frame */
	/* provide Tachyon will another set of buffers */
	if (offset == (BUFF_ADDRS - 1))
		update_SFSBQ_indx(fi);
	reset_latch(fi); /* don't forget to do this! */
	LEAVE("handle_Busied_Frame_interrupt");
}

static void handle_SFS_interrupt(struct fc_info *fi)
{
u_int *ptr_imq_entry, *buff_addr;
u_int class_of_frame, type_of_frame, s_id, d_id, els_type = 0;
u_int wwn_high, wwn_low, node_high, node_low;
int queue_indx, offset, payload_size;
u_short received_ox_id, received_rx_id, fs_cmnd_code;
u_int inputArray[64], id, i, class_valid, class_service;
u_int name_length, node_name[2];
struct fc_info *fi_dest; //destination card
u_char alpa; 
unsigned char seq_id;
u_int r_ctl;
u_int rscn_needed = 0;   //if rscn is needed, set rscn_needed to the value of the S_ID of the frame which causes the rscn
		     // otherwise set rscn_needed to zero
u_char expln;	// all FCS_RJT reason codes use 09: unable to perform the request
		// but use different explanation code
u_int d_domain;
	
	ENTER("handle_SFS_interrupt");
	ptr_imq_entry = fi->q.ptr_imqe[fi->q.imq_cons_indx];
	offset = ntohl(*(ptr_imq_entry + 1)) & 0x00000007;
	queue_indx = ntohl(*(ptr_imq_entry + 1)) & 0xFFFF0000;
	queue_indx = queue_indx >> 16;
	DPRINTK("queue_indx = %d, offset  = %d\n", queue_indx, offset);
	payload_size = ntohl(*(ptr_imq_entry + 2));
	DPRINTK("payload_size = %d&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&&", payload_size);

	buff_addr = bus_to_virt(ntohl(*(fi->q.ptr_sfsbq_base + queue_indx*BUFF_ADDRS + offset)));

	#if DEBUG_5526
	/*DPRINTK("buff_addr = %x", (u_int)buff_addr);
	{
	int i; u_int *temp_addr = buff_addr;
		for (i=0; i<payload_size/4; i++) {
			DPRINTK("PL[%d] = %x", i, (u_int)ntohl(*temp_addr));
			temp_addr++;
		}
	}*/
	#endif

	/* extract Type of Frame */
        r_ctl = ntohl(*(buff_addr + 2)) & 0xFF000000;
	type_of_frame = ntohl(*(buff_addr + 4)) & 0xFF000000;
	d_id = ntohl(*(buff_addr + 2)) & 0x00FFFFFF;
	s_id = ntohl(*(buff_addr + 3)) & 0x00FFFFFF;
	alpa = d_id & 0x0000FF;  // used for tachyon_header pg156
	seq_id = ntohl(*(buff_addr + 5)) >> 24;
	received_ox_id = ntohl(*(buff_addr + 6)) >> 16;
	received_rx_id = ntohl(*(buff_addr + 6)) & 0x0000FFFF;
       

	DPRINTK("S_AREA: %x", s_id & 0xFFFF00);	
	DPRINTK("D_AREA: %x", d_id & 0xFFFF00); 



	switch(d_id)
	{
	case 0xFFFFFE:   // only Flogi should be transmitted to FFFFFE.
		fi->g.my_id = F_PORT;
        	tx_logi_acc(fi, ELS_ACC, fi->g.my_ddaa | s_id, received_ox_id);     //tx FLOGI_ACC
		break;
	case 0xFFFFFD:	// E_Port should accept frames to FFFFFD
		if(type_of_frame == TYPE_ELS)
		{
			class_of_frame = ntohl(*(buff_addr + 8));
			switch(class_of_frame & 0xFF000000)
                         {
 				case ELS_SCR:
                                       fi->g.my_id = FABRIC_CONTROLLER;
                                       tx_acc(fi, s_id, received_ox_id);
                                       fi->g.my_id = F_PORT;
                                       break;
				default:

			 }
		} // end of if: TYPE_ELS

                // maybe Class F service
		break;
	case 0xFFFFFC:	// all name server request
		fi->g.my_id = DIRECTORY_SERVER;
	        switch(type_of_frame) 
		  {
		  case TYPE_ELS:
			class_of_frame = ntohl(*(buff_addr + 8));
			switch(class_of_frame & 0xFF000000) 
			  {
				case ELS_PLOGI:
					if (s_id != fi->g.my_id) 
                                        {
						u_int ret_code;
						DEBUG("PLOGI received from D_ID 0x%x with 0X_ID = %x", s_id, received_ox_id);
						if ((ret_code = plogi_ok(fi, buff_addr, payload_size)) == 0)
						  {
								tx_logi_acc(fi, ELS_ACC, s_id, received_ox_id);
 								
                                         			wwn_high = ntohl(*(buff_addr + 13));
                                         			wwn_low = ntohl(*(buff_addr + 14)) ;    //get world wide name in the  payload
                                                                node_high = ntohl(*(buff_addr + 15));
                                                                node_low = ntohl(*(buff_addr + 16)) ;    //get node name in the  payload
								DEBUG("Name server rx: %x %x  node name %x %x", wwn_high, wwn_low, node_high, node_low);
								class_valid = ntohl(*(buff_addr + 17)) & 0x80000000; // get class 1 validity
								class_valid = class_valid >> 31;
                                                                class_service = class_valid * 2;
                                                                class_valid = ntohl(*(buff_addr + 21)) & 0x80000000; // get class 2 validity
								class_valid = class_valid >> 31;
                                                                class_service += class_valid * 4;
                                                                class_valid = ntohl(*(buff_addr + 25)) & 0x80000000; // get class 3 validity
								class_valid = class_valid >> 31;
								class_service += class_valid * 8;
 								add_to_name_server(fi, s_id, wwn_high, wwn_low, node_high, node_low, class_service);
								show_names();
								rscn_needed = s_id;
						                add_to_address_cache(fi, buff_addr);
						}
						else {
							u_short cmnd_code = ret_code >> 16;
							u_short expln_code =  ret_code;
							tx_ls_rjt(fi, s_id, received_ox_id, cmnd_code, expln_code);
						     }
					} // end of if (s_id != fi->g.my_id)
			                break;
			            default:

			  } // end of switch (class_of_frame..)
			  break;
			case TYPE_FC_SERVICES:     
			fs_cmnd_code = (ntohl(*(buff_addr + 10)) & 0xFFFF0000) >>16;
			switch(fs_cmnd_code) {
			     // respond to Get request
                               case GA_NXT:
                                         DEBUG("FCS GA_NXT command received from D_ID 0x%x in response to %x", s_id, els_type);
	 				 inputArray[0] = ntohl(*(buff_addr + 12)) & 0x00FFFFFF;	
                                         fcs_query(fi, s_id, received_ox_id, inputArray, GA_NXT);
                                         break;  


                                case GI_A:
                                         inputArray[0] = (ntohl(*(buff_addr + 12)) & 0x00FF0000 );   
					 fcs_query(fi, s_id, received_ox_id, inputArray, GI_A);
                                        break;

                                case GPN_ID:
                                         inputArray[0] = ntohl(*(buff_addr + 12)) & 0x00FFFFFF;  // get the ID 
					 fcs_query(fi, s_id, received_ox_id, inputArray, GPN_ID);
                                        break;
                                case GNN_ID:
                                         inputArray[0] = ntohl(*(buff_addr + 12)) & 0x00FFFFFF;  // get the ID
					fcs_query(fi, s_id, received_ox_id, inputArray, GNN_ID);
                                        break;
                                case GCS_ID:
                                         inputArray[0] = ntohl(*(buff_addr + 12)) & 0x00FFFFFF;  // get the ID
					 fcs_query(fi, s_id, received_ox_id, inputArray, GCS_ID);
                                        break;
                                case GFT_ID:
                                         inputArray[0] = ntohl(*(buff_addr + 12)) & 0x00FFFFFF;  // get the ID 
					 fcs_query(fi, s_id, received_ox_id, inputArray, GFT_ID);
                                        break;
                                case GSPN_ID:
                                        inputArray[0] = ntohl(*(buff_addr + 12)) & 0x00FFFFFF;  // get the ID
			 	 	fcs_query(fi, s_id, received_ox_id, inputArray, GSPN_ID);
                                        break;
                                case GPT_ID:
                                         inputArray[0] = ntohl(*(buff_addr + 12)) & 0x00FFFFFF;  // get the ID
					 fcs_query(fi, s_id, received_ox_id, inputArray, GPT_ID);
                                        break;
                                case GIPP_ID:
                                         inputArray[0] = ntohl(*(buff_addr + 12)) & 0x00FFFFFF;  // get the ID 
					 fcs_query(fi, s_id, received_ox_id, inputArray, GIPP_ID);
                                        break;
                                case GFPN_ID:
                                         inputArray[0] = ntohl(*(buff_addr + 12)) & 0x00FFFFFF;  // get the ID 
					 fcs_query(fi, s_id, received_ox_id, inputArray, GFPN_ID);
                                        break;
                                case GHA_ID:
                                         inputArray[0] = ntohl(*(buff_addr + 12)) & 0x00FFFFFF;  // get the ID 
					 fcs_query(fi, s_id, received_ox_id, inputArray, GHA_ID);
                                        break;
						// by Port Name				
                                case GID_PN:
                                         inputArray[0] = ntohl(*(buff_addr + 12));  // get the Port Name 
                                         inputArray[1] = ntohl(*(buff_addr + 13));  // get the Port Name 
					fcs_query(fi, s_id, received_ox_id, inputArray, GID_PN);
                                        break;
                                case GIPP_PN:
                                         inputArray[0] = ntohl(*(buff_addr + 12));  // get the Port Name 
                                         inputArray[1] = ntohl(*(buff_addr + 13));  // get the Port Name 
					 fcs_query(fi, s_id, received_ox_id, inputArray, GIPP_PN);
                                        break;
						
						// by Node Name
                                case GID_NN:
                                         inputArray[0] = ntohl(*(buff_addr + 12));  // get the Node Name 
                                         inputArray[1] = ntohl(*(buff_addr + 13));  // get the Node Name 
					 fcs_query(fi, s_id, received_ox_id, inputArray, GID_NN);
                                        break;
                                case GIP_NN:
                                         inputArray[0] = ntohl(*(buff_addr + 12));  // get the Node Name 
                                         inputArray[1] = ntohl(*(buff_addr + 13));  // get the Node Name 
					 fcs_query(fi, s_id, received_ox_id, inputArray, GIP_NN);
                                        break;
                                case GIPA_NN:
                                         inputArray[0] = ntohl(*(buff_addr + 12));  // get the Node Name 
                                         inputArray[1] = ntohl(*(buff_addr + 13));  // get the Node Name 
					 fcs_query(fi, s_id, received_ox_id, inputArray, GIPA_NN);
                                        break;
                                case GSNN_NN:
                                         inputArray[0] = ntohl(*(buff_addr + 12));  // get the Node Name 
                                         inputArray[1] = ntohl(*(buff_addr + 13));  // get the Node Name 
					 fcs_query(fi, s_id, received_ox_id, inputArray, GSNN_NN);
                                        break;
                     	
						// by IP          
				case GNN_IP:
					 for (i=0; i<4; i++)
                                         inputArray[i] = ntohl(*(buff_addr + 12 + i));  // get the Node Name 
					 fcs_query(fi, s_id, received_ox_id, inputArray, GNN_IP);
                                        break;
				case GIPA_IP:
					 for (i=0; i<4; i++)
                                         inputArray[i] = ntohl(*(buff_addr + 12 + i));  // get the Node Name 
					 fcs_query(fi, s_id, received_ox_id, inputArray, GIPA_IP);
                                        break;

						// by FC-4 Types
                                case GID_FT:
                                         DEBUG("FCS GID_FT command received from D_ID 0x%x in response to %x", s_id, els_type);
                                         inputArray[0] = ntohl(*(buff_addr + 12)) & 0x00FFFFFF;  // get the ID scope and FC-4 type
					 fcs_query(fi, s_id, received_ox_id, inputArray, GID_FT);
                                        break;
                                case GNN_FT:
                                         inputArray[0] = ntohl(*(buff_addr + 12)) & 0x00FFFFFF;  // get the ID scope and FC-4 type
					 fcs_query(fi, s_id, received_ox_id, inputArray, GNN_FT);
                                        break;

						// by Port Type          
				case GID_PT:
                                         inputArray[0] = ntohl(*(buff_addr + 12)) & 0xFF000000;  
					 fcs_query(fi, s_id, received_ox_id, inputArray, GID_PT);
                                        break;

						// by IPP          
				case GID_IPP:
					 for (i=0; i<4; i++)
                                         inputArray[i] = ntohl(*(buff_addr + 12 + i));  
					 fcs_query(fi, s_id, received_ox_id, inputArray, GID_IPP);
                                        break;
				case GPN_IPP:
					 for (i=0; i<4; i++)
                                         inputArray[i] = ntohl(*(buff_addr + 12 + i));  
					 fcs_query(fi, s_id, received_ox_id, inputArray, GPN_IPP);
                                        break;



                               // respond to Register request
                                case FCS_RPN_ID:
                                        DEBUG("FCS_RPN_ID: %X received from D_ID 0x%x in response to %x",
                                                    fs_cmnd_code, s_id, els_type);
                                         id = ntohl(*(buff_addr + 12)) & 0x00FFFFFF;
                                         inputArray[0] = ntohl(*(buff_addr + 13));
                                         inputArray[1] = ntohl(*(buff_addr + 14));    //get world wide name in the  payload
                                         expln = fcs_register_id(fi, id, inputArray, FCS_RPN_ID);
                                         tx_fcs_resp(fi, s_id, received_ox_id, (expln ? 9 : 0), expln);
                                        break;
				case	FCS_RNN_ID:
					DEBUG("FCS_RNN_ID: %X received from D_ID 0x%x in response to %x",
                                                    fs_cmnd_code, s_id, els_type);
					 id = ntohl(*(buff_addr + 12)) & 0x00FFFFFF;
                                         inputArray[0] = ntohl(*(buff_addr + 13));
                                         inputArray[1] = ntohl(*(buff_addr + 14));    //get node name in the  payload
                                         expln = fcs_register_id(fi, id, inputArray, FCS_RNN_ID);
                                         tx_fcs_resp(fi, s_id, received_ox_id, (expln ? 9 : 0), expln);
                                        break;
                                case    FCS_RCS_ID:
                                        DEBUG("FCS_RCS_ID: %X received from D_ID 0x%x in response to %x",
                                                    fs_cmnd_code, s_id, els_type);
                                         id = ntohl(*(buff_addr + 12)) & 0x00FFFFFF;
                                         inputArray[0] = ntohl(*(buff_addr + 13));
                                         expln = fcs_register_id(fi, id, inputArray, FCS_RCS_ID);
                                         tx_fcs_resp(fi, s_id, received_ox_id, (expln ? 9 : 0), expln);
                                        break;
                                case FCS_RFT_ID:
                                         id = ntohl(*(buff_addr + 12)) & 0x00FFFFFF;
                                         for (i=0; i<8; i++)
                                            inputArray[i] = ntohl(*(buff_addr + 13 +i )) ;
                                                       //get the FC-4 types in the  payload
                                         expln = fcs_register_id(fi, id, inputArray, FCS_RFT_ID);   
                                         tx_fcs_resp(fi, s_id, received_ox_id, (expln ? 9 : 0), expln);   
					break;
                                 case    FCS_RSPN_ID:
                                         id = ntohl(*(buff_addr + 12)) & 0x00FFFFFF;
					 name_length = (ntohl(*(buff_addr + 13)) & 0xFF000000) >> 24;
					 name_length = name_length / 4 + 1; // convert to word count 
					 for (i=0; i<name_length; i++)
                                            inputArray[i] = ntohl(*(buff_addr + 13 +i )) ;
                                                       //get the symbolic port name in the  payload
                                         expln = fcs_register_id(fi, id, inputArray, FCS_RSPN_ID);
                                         tx_fcs_resp(fi, s_id, received_ox_id, (expln ? 9 : 0), expln);
                                        break;
                                 case    FCS_RPT_ID:
                                         id = ntohl(*(buff_addr + 12)) & 0x00FFFFFF;
                                         inputArray[0] = ntohl(*(buff_addr + 13)) & 0xFF000000;
                                         expln = fcs_register_id(fi, id, inputArray, FCS_RPT_ID);
                                         tx_fcs_resp(fi, s_id, received_ox_id, (expln ? 9 : 0), expln);
                                        break;

                                 case    FCS_RIPP_ID:
                                         id = ntohl(*(buff_addr + 12)) & 0x00FFFFFF;
                                         for (i=0; i<4; i++)
                                            inputArray[i] = ntohl(*(buff_addr + 13 +i ));
                                         expln = fcs_register_id(fi, id, inputArray, FCS_RIPP_ID);
                                         tx_fcs_resp(fi, s_id, received_ox_id, (expln ? 9 : 0), expln);
                                        break;

                                 case    FCS_RFPN_ID:
                                         id = ntohl(*(buff_addr + 12)) & 0x00FFFFFF;
                                         for (i=0; i<2; i++)
                                            inputArray[i] = ntohl(*(buff_addr + 13 +i ));
                                         expln = fcs_register_id(fi, id, inputArray, FCS_RFPN_ID);
                                         tx_fcs_resp(fi, s_id, received_ox_id, (expln ? 9 : 0), expln);
                                        break;

                                case    FCS_RHA_ID:
                                         id = ntohl(*(buff_addr + 12)) & 0x00FFFFFF;
                                         inputArray[0] = ntohl(*(buff_addr + 13)) & 0x00FFFFFF;
                                         expln = fcs_register_id(fi, id, inputArray, FCS_RHA_ID);
                                         tx_fcs_resp(fi, s_id, received_ox_id, (expln ? 9 : 0), expln);
                                        break;

				case FCS_RIP_NN:   
                                         node_name[0] = ntohl(*(buff_addr + 12)) ;
                                         node_name[1] = ntohl(*(buff_addr + 13)) ;
					 for (i=0; i<4; i++)
                                            inputArray[i] = ntohl(*(buff_addr + 14 +i )) ;  					 
                                         expln = fcs_register_nn(fi, node_name, inputArray, FCS_RIP_NN);
                                         tx_fcs_resp(fi, s_id, received_ox_id, (expln ? 9 : 0), expln);
                                        break;
                                 case    FCS_RIPA_NN:
                                         node_name[0] = ntohl(*(buff_addr + 12)) ;
                                         node_name[1] = ntohl(*(buff_addr + 13)) ;
                                         for (i=0; i<2; i++)
                                            inputArray[i] = ntohl(*(buff_addr + 14 +i )); 
                                         expln = fcs_register_nn(fi, node_name, inputArray, FCS_RIPA_NN);
                                         tx_fcs_resp(fi, s_id, received_ox_id, (expln ? 9 : 0), expln);
                                        break;
                                case    FCS_RSNN_NN:
                                         node_name[0] = ntohl(*(buff_addr + 12)) ;
                                         node_name[1] = ntohl(*(buff_addr + 13)) ;
					                     name_length = (ntohl(*(buff_addr + 14)) & 0xFF000000) >> 24;
                                         name_length = name_length / 4 + 1; // convert to word count
                                         for (i=0; i<name_length; i++)
                                            inputArray[i] = ntohl(*(buff_addr + 14 +i )) ;
                                                       //get the symbolic node name in the  payload
                                         expln = fcs_register_nn(fi, node_name, inputArray, FCS_RSNN_NN);
                                         tx_fcs_resp(fi, s_id, received_ox_id, (expln ? 9 : 0), expln);
                                        break;


                                 case DA_ID:
                                         inputArray[0] = ntohl(*(buff_addr + 12)) & 0x00FFFFFF;  // get the ID 
					 fcs_query(fi, s_id, received_ox_id, inputArray, DA_ID);
                                        break; 

				default:
                                         DEBUG("Unknown FCS command:%X received from D_ID 0x%x in response to %x", 
					           fs_cmnd_code, s_id, els_type);
			} // end of switch(fs_cmnd_code)	 
		  default:
		  }  // end of switch(type_of_frame)
		break;
	default:
		
        if( (d_id & 0xFFFF00) != (s_id & 0xFFFF00) )
	{	// two different cases: 1: same domain, different ports    2: different domain
	  	if( (d_id & 0xFF0000) != (s_id & 0xFF0000) )
	  	{
			d_domain = (d_id & 0xFF0000) >> 16;    // destination domain_id	
			fi_dest = find_card( get_src_port(d_domain) );  // get the port number from the routing table		
	  	}
	  	else      //   if ( (d_id & 0x00FF00) != (s_id & 0x00FF00) )
            	{    
			//same domain ID, but different Area IDs
             		fi_dest = find_card((d_id & 0x00FF00) >> 8);   // find the card using area ID
		}

              	if(!fi_dest)
			{
		  	DPRINTK("Find a area ID without corresponding card: %d", d_id);
		 	 return; // null, return
			}
		//xiaoyi
                fi_dest->g.tach_header.resv = 0x00000000;
  		fi_dest->g.tach_header.sof_and_eof = SOFI3 | EOFN;
  		fi_dest->g.tach_header.dest_alpa = alpa;
  		fi_dest->g.tach_header.lcr_and_time_stamp = 0;       
		              // set the first two words
                
		if(get_free_EDB(fi_dest) )
		  return;
		memcpy(fi_dest->q.ptr_tachyon_header[fi_dest->q.tachyon_header_indx], &(fi_dest->g.tach_header), 8);
		//copy first two words
  		memcpy(fi_dest->q.ptr_tachyon_header[fi_dest->q.tachyon_header_indx] + 2, buff_addr+2, 24);
  		// move real 6 words header from buff_addr + 2
		// header was build ***************************************
		build_EDB(fi_dest, (char *)(buff_addr+8), EDB_END_BIT, payload_size - 32);  // real payload data length = -4*8

		memcpy(fi_dest->q.ptr_edb[fi_dest->q.edb_buffer_indx], &(fi_dest->g.edb), sizeof(EDB));

		build_ODB(fi_dest, seq_id, d_id, payload_size-32, r_ctl, FRAME_SIZE, received_ox_id, received_rx_id, 0, 1, 1);
		// mtu: 2048    NW_flag=0     int_required =1     frame_class=1

		memcpy(fi_dest->q.ptr_odb[fi_dest->q.ocq_prod_indx], &(fi_dest->g.odb), sizeof(ODB));
		if ((fi_dest->g.loop_up != TRUE) && (fi_dest->g.ptp_up != TRUE)) 
		  {
		    T_MSG("SFS: Fibre Channel Link not up. Dropping Exchange! fi_dest.card_order: %x", fi_dest->g.card_order);
       		  }
		else 
		  {
		    /* To be on the safe side, a check should be included
		     * at this point to check if we are overrunning 
		     * Tachyon.
		     */
		    update_OCQ_indx(fi_dest);
		  }
		update_EDB_indx(fi_dest);
		update_tachyon_header_indx(fi_dest);
	} 
	else   // local port : same domain and area IDs
	     // printk(KERN_INFO "@@@@@@@@@@@@@@@@@@@@  panic: received a frame from %x to %x, this frame should not be seen!!!!!", s_id, d_id);
	{
		// discard this frame!
	}
  }// end of the main switch(d_id)


       if(rscn_needed != 0)
                  tx_rscn(rscn_needed);

	/* provide Tachyon will another set of buffers */
	if (offset == (BUFF_ADDRS - 1))
		update_SFSBQ_indx(fi);
	reset_latch(fi); /* don't forget to do this! */
	LEAVE("handle_SFS_interrupt");
}




/*
This is added by hxd for the name server.
*/
static void tx_rscn(u_int rscn_id)
{
struct port_info *walker;
u_int i, d_id;
u_int buf[2];
u_int r_ctl = RCTL_ELS_UCTL;
u_int type  = TYPE_ELS | SEQUENCE_INITIATIVE | FIRST_SEQUENCE;
u_int my_mtu = FRAME_SIZE;

buf[0] = htonl(0x61040008);
buf[1] = 0x02000000 | (rscn_id & 0xFF0000);    // add domain_id
buf[1] = htonl(buf[1]);

for(i=0; i<MAX_FPORT_CARDS; i++)
  {
   walker = fc[i]->name_list;
   while(walker)
	{
	 d_id = walker->portID;
	 if(d_id != rscn_id)
	   {
	     fc[i]->g.my_id = FABRIC_CONTROLLER;
	     memcpy(fc[i]->g.els_buffer[fc[i]->g.e_i], buf, 8);   // 8 bytes
	     tx_exchange(fc[i], (char *)(fc[i]->g.els_buffer[fc[i]->g.e_i]), 8, r_ctl, type, d_id, my_mtu, 1, 0x0001, 0x61000000);	
	   }
	 walker = walker->next;
	}
  }


}


static void add_to_name_server(struct fc_info *fi, u_int new_id, u_int wwn_high, u_int wwn_low, u_int node_high, u_int node_low, u_int cs)
{
struct port_info *walker = fi->name_list;
struct port_info *finder = NULL;
struct port_info *new_port;
u_int i;

DEBUG("go into add_to_name_server");

while(walker && walker->portID <= new_id)
  	{ finder = walker;
	  walker = walker->next;
	}
if (finder && finder->portID == new_id)  //get same id
  {	DEBUG("get the same fabric id: %x", new_id);
        return; 
  }
new_port = (struct port_info *)kmalloc(sizeof(struct port_info), GFP_KERNEL);
if(new_port == NULL) {
			T_MSG("kmalloc failed in add_to_name_server()");
			return;
		  }
 else  {            // set or default the value of the struct
        new_port->portID = new_id;
	new_port->port_name_high = wwn_high;
        new_port->port_name_low  = wwn_low;
	for(i=0; i<64; i++)
	  new_port->s_port_name[i] = 0;
        new_port->node_name_high = node_high;
        new_port->node_name_low  = node_low;
        for(i=0; i<64; i++)
	  new_port->s_node_name[i] = 0;
	for(i=0; i<2; i++)
	  new_port->initial_pa[i] = 0;
	for(i=0; i<4; i++)
	  new_port->ip[i] = 0;
        new_port->class_service  = cs;
        for(i=0; i<8; i++)
	  new_port->FC_4_types[i] = 0;
        new_port->portType = 0;
        for(i=0; i<4; i++)
	  new_port->ip_port[i] = 0;
        new_port->fabric_name_high = 0;
        new_port->fabric_name_low = 0;
        new_port->hard_address = 0; 
	new_port->SET_FLAG = 0;
}

if(finder) 
{ finder->next = new_port;
  new_port->next = walker;
}
else {
 new_port->next = fi->name_list;
 fi->name_list = new_port;

}

} // end of add_to_name_server

static void show_names()
{
struct port_info *walker; 
int flag;
int i;

//DEBUG("go into show_names");
  
printk("=========================== Devices Registered =============================\n");

for(i=0; i<MAX_FPORT_CARDS; i++) { 
  walker = fc[i]->name_list;
  while(walker) {
    printk("Port: %x  WWN: %x   %x NodeName: %x  %x CS: %x\n", walker->portID, walker->port_name_high,
    walker->port_name_low, walker->node_name_high, walker->node_name_low, walker->class_service);

    flag = walker->SET_FLAG & FCS_FC4_SET;

    if (flag)    // if the fc4_types was set, print it out
    printk("FC_4_types: %x %x %x %x %x %x %x %x \n", walker->FC_4_types[0], walker->FC_4_types[1], walker->FC_4_types[2],
    walker->FC_4_types[3],
    walker->FC_4_types[4], walker->FC_4_types[5], walker->FC_4_types[6], walker->FC_4_types[7]);

    flag = walker->SET_FLAG & FCS_PT_SET;
    if (flag)    // if the port types was set, print it out
        printk("port type: %x \n", walker->portType); 

    printk("-----------------------------------------------------------------------------\n");

    walker = walker->next;
  }
}


printk("=============================================================================\n");
}



//-------------------------------  fcs_register_id() ---------------------------------------------------------
static int fcs_register_id(struct fc_info *fi, u_int id, u_int inputArray[], u_int flag)
{
struct port_info *walker = fi->name_list;
int i, name_length;

while(walker && walker->portID < id)
   walker = walker->next;      // after this, walker has a ID either equal or larger than the ID
if(!walker || walker->portID != id)
  { printk("%x: could not find this id in the name_server!\n", id);
    return NO_PORT_ID;
  }
// if could not find this ID in the name_server, should return 0 and the upper function should send FC_RJT, but this should not happen

switch(flag)
{
case FCS_RPN_ID:
                walker->port_name_high = inputArray[0];
		walker->port_name_low  = inputArray[1];
		break;
case FCS_RNN_ID:
                walker->node_name_high = inputArray[0];
                walker->node_name_low  = inputArray[1];
                break;
case FCS_RCS_ID:                         
                walker->class_service = inputArray[0];
                break;
case FCS_RFT_ID:
                for(i=0; i<8; i++)
                        walker->FC_4_types[i] = inputArray[i];
               	walker->SET_FLAG |= FCS_FC4_SET;   // indicate that FC4_types was registered.
                break;
case FCS_RSPN_ID:
		 name_length = (inputArray[0] & 0xFF000000) >> 24;
                 name_length = name_length / 4 + 1; // convert to word count
                 for (i=0; i<name_length; i++)
                        walker->s_port_name[i] = inputArray[i];
	        	walker->SET_FLAG |= FCS_SPN_SET;   // indicate that FC4_types was registered.
                break;
case FCS_RPT_ID:
                walker->portType = inputArray[0];   // the highest byte is the port typw such as 03 00 00 00, magbe >> 24
	            walker->SET_FLAG |= FCS_PT_SET;   // indicate that port type was set.
                break;
case FCS_RIPP_ID:
                for(i=0; i<4; i++)
                        walker->ip_port[i] = inputArray[i];
                walker->SET_FLAG |= FCS_IPP_SET;   // indicate that IP (port) was registered.
                break;
case FCS_RFPN_ID:
                walker->fabric_name_high = inputArray[0];
                walker->fabric_name_low  = inputArray[1];
                walker->SET_FLAG |= FCS_FPN_SET;   // indicate that Fabric Port Name was registered.
                break;

case FCS_RHA_ID:
                walker->hard_address = inputArray[0];
                walker->SET_FLAG |= FCS_HA_SET;   // indicate that Fabric Port Name was registered.
                break;
default:
   DEBUG("wrong flag %X, receiver in fcs_register_id", flag);  // it shouldn't happen

}


return 0;   // 0 means OK
}


//-------------------------------  fcs_register_nn() ---------------------------------------------------------
static int fcs_register_nn(struct fc_info *fi, u_int nn[], u_int inputArray[], u_int flag)
{
struct port_info *walker = fi->name_list;
int i, name_length;
   
while(walker && (walker->node_name_high != nn[0] || walker->node_name_low != nn[1]))
   walker = walker->next;     
if(!walker)
  { printk("%x  %x: could not find this node name in the name_server!\n", nn[0], nn[1]);
    return NO_NODE_NAME;
  }
// if could not find this ID in the name_server, should return 0 and the upper function should send FC_RJT, but this should not happen

switch(flag)
{
case FCS_RIP_NN:
                for(i=0; i<4; i++)
                        walker->ip[i] = inputArray[i];
                walker->SET_FLAG |= FCS_IP_SET;   // indicate that IP was registered.
                break;
case FCS_RIPA_NN:
                for(i=0; i<2; i++)
                        walker->initial_pa[i] = inputArray[i];
                walker->SET_FLAG |= FCS_IPA_SET;   // indicate that IP was registered.
                break;
case FCS_RSNN_NN:
                 name_length = (inputArray[0] & 0xFF000000) >> 24;
                 name_length = name_length / 4 + 1; // convert to word count
				 for (i=0; i<name_length; i++)
                        walker->s_node_name[i] = inputArray[i];
                walker->SET_FLAG |= FCS_SNN_SET;   // indicate that FC4_types was registered.
                break;
default:
   DEBUG("wrong flag %X, receiver in fcs_register_id", flag);  // it shouldn't happen

}
                
                
return 0;  // 0 means OK
}


//-------------------------------  fcs_query() ---------------------------------------------------------
static int fcs_query(struct fc_info *fi, u_int s_id, u_short ox_id, u_int inputArray[], u_int cmd)
{                                       
u_int buf[159];   // the maximum buf: GA_NXT_AXX 
struct port_info *walker = NULL, *finder = NULL, *temp = NULL;
u_int r_ctl = RCTL_RFC_SCTL;
u_int type  = TYPE_FC_SERVICES | EXCHANGE_RESPONDER | LAST_SEQUENCE;
u_int i, k = 4, found;
u_int id, cardNumber;

buf[0] = htonl(0x01000000);
buf[1] = htonl(0xFC020000);
buf[2] = htonl(0x80020000);
buf[3] = htonl(0x00000000);   // the CT header is fixed

next_card(0);

if(cmd == GA_NXT) {   //GA_NXT should never return a RJT, because at least the sw has a ID from the GA_NXT S_ID
	id = inputArray[0];
	//DEBUG("id: Wrong cardNumber in fcs_query : %d", id)
        cardNumber = (id & 0x0000FF00) >> 8;
	//DEBUG("after & Wrong cardNumber in fcs_query : %d", cardNumber)
	if (cardNumber >= 1)
		cardNumber -= 1;    // like 180101 is from card 0 : fc[0]
		    // but the id may be set to any value, so be careful about the cardNumber =0, or <0, like Qlogic id=0;

	if(cardNumber >= MAX_FPORT_CARDS) {   // This should never happen.
		DEBUG("Wrong cardNumber in fcs_query : %d", cardNumber)
		return 0;
	}  // this check saved me at least 2 hours and a bad feeling, Qlogic2200 set the ID in GA_NXT as 000000 !!
	   // very good to check this. Make sure to check very possible NULL in the code.

	walker = fi->name_list;
	if(!walker)  // because DA_ID may remove all nodes
		{
		tx_fcs_resp(fi, s_id, ox_id, 9, NO_PORT_ID);
                return 0;
	}	
	while(walker && walker->portID <= id)
		walker = walker->next;
	if(!walker){
		 for(i=0; i<MAX_FPORT_CARDS; i++)    {
			  walker = next_card(cardNumber)->name_list;
			  if(walker)
			      break;
			  else 
                              cardNumber++;
		 } // if can't find the next in current card, use the first id in the next_card
	}
		
	buf[k++] = htonl(walker->portType | walker->portID);
	buf[k++] = htonl(walker->port_name_high);
	buf[k++] = htonl(walker->port_name_low);
	for(i=0; i<64; i++)
		buf[k++] = htonl(walker->s_port_name[i]);
	buf[k++] = htonl(walker->node_name_high);
	buf[k++] = htonl(walker->node_name_low);
	for(i=0; i<64; i++)
		buf[k++] = htonl(walker->s_node_name[i]);
	for(i=0; i<2; i++)
		buf[k++] = htonl(walker->initial_pa[i]);
	for(i=0; i<4; i++)
		buf[k++] = htonl(walker->ip[i]);
        buf[k++] = htonl(walker->class_service);
	for(i=0; i<8; i++)
		buf[k++] = htonl(walker->FC_4_types[i]);
	for(i=0; i<4; i++)
		buf[k++] = htonl(walker->ip_port[i]);
        buf[k++] = htonl(walker->fabric_name_high);
	buf[k++] = htonl(walker->fabric_name_low);
	buf[k++] = htonl(walker->hard_address);
}

 if(cmd == GI_A) {   // I only report all the info regarding myself
  	id = s_id & 0x00FF0000;
  	if(inputArray[0] == 0)    // Domain_ID scope is zero
    		buf[k++] = htonl(0x80000000 | id);   //add the Domain ID
  	else if (inputArray[0] == id) 
       	     {
	 	for(i=0; i<MAX_FPORT_CARDS; i++)
	   		if (fc[i]->name_list)
	     			buf[k++] = htonl(id | ( (i+1) << 8));   // i left move to be a area_id
         	buf[k-1] = ntohl(buf[k-1]) | 0x80000000; // set the last record flag
	     	buf[k-1] = htonl(buf[k-1]);
       	     }
             else 
		tx_fcs_resp(fi, s_id, ox_id, 9, ACCESS_DENIED);
}


if(cmd >= GPN_ID && cmd <= GHA_ID) {
	found = 0;
	for(i=0; i<MAX_FPORT_CARDS; i++) {
		if(!found) walker = fc[i]->name_list;
		while(walker && !found) {
			id = walker->portID;
			if(id == inputArray[0]) {  
				found = 1;
				break;	// if found this id, jump out the while loop
			}
			walker = walker->next;
		}
	} 	
	if( !found ) {   // if no device found, return
		tx_fcs_resp(fi, s_id, ox_id, 9, NO_PORT_ID);
        return 0;
        }

	switch(cmd) {
		case GPN_ID:
			buf[k++] = htonl(walker->port_name_high);
			buf[k++] = htonl(walker->port_name_low);
			break;
                case GNN_ID:
			buf[k++] = htonl(walker->node_name_high);
			buf[k++] = htonl(walker->node_name_low);
                        break;
                case GCS_ID:
			buf[k++] = htonl(walker->class_service);
                        break;
                case GFT_ID:
			if(walker->SET_FLAG & FCS_FC4_SET) { 
				 for(i=0; i<8; i++)
				 	buf[k++] = htonl(walker->FC_4_types[i]);
			}
			else {
				  tx_fcs_resp(fi, s_id, ox_id, 9, NO_FC4);
			 	  return 0;
			}
                        break;
                case GSPN_ID:
			if(walker->SET_FLAG & FCS_SPN_SET) { 
				for(i=0; i<64; i++)
				 	buf[k++] = htonl(walker->s_port_name[i]);
			}
			else {
				tx_fcs_resp(fi, s_id, ox_id, 9, NO_SPN);
				return 0;
			}
                        break;
                case GPT_ID:
			if(walker->SET_FLAG & FCS_PT_SET) { 
				buf[k++] = htonl(walker->portType);
			}
			else {
				tx_fcs_resp(fi, s_id, ox_id, 9, NO_PT);
				return 0;
			}
                        break;
                case GIPP_ID:
			if(walker->SET_FLAG & FCS_IPP_SET) {
				for(i=0; i<4; i++)
					buf[k++] = htonl(walker->ip_port[i]);
			}
			else {
				tx_fcs_resp(fi, s_id, ox_id, 9, NO_IPP);
				return 0;
			}
                        break;
                case GFPN_ID:
			if(walker->SET_FLAG & FCS_FPN_SET) {
			 	buf[k++] = htonl(walker->fabric_name_high);
			 	buf[k++] = htonl(walker->fabric_name_low);
			}
			else {
				tx_fcs_resp(fi, s_id, ox_id, 9, NO_FPN);
				return 0;
			}
                        break;
                case GHA_ID:
			if(walker->SET_FLAG & FCS_HA_SET) {
				buf[k++] = htonl(walker->hard_address);
			}
			else {
				tx_fcs_resp(fi, s_id, ox_id, 9, NO_HA);
				return 0;
			}
                        break;
	} //end of the switch(cmd)
 
}


if(cmd == GID_PN || cmd == GIPP_PN) {
	found = 0;
	for(i=0; i<MAX_FPORT_CARDS; i++) {
		if(!found) walker = fc[i]->name_list;
		while(walker && !found) {
			if(walker->port_name_high == inputArray[0] && walker->port_name_low == inputArray[1]) 
			{  
				found = 1;
				break;	// if found this id, jump out the while loop
			}
			walker = walker->next;
		}
	} 	
	if( !found ) {   // if no device found, return
		tx_fcs_resp(fi, s_id, ox_id, 9, NO_PORT_NAME);
        return 0;
        }
        
        switch(cmd) {
                case GID_PN:
			buf[k++] = htonl(walker->portID);
                        break;
                case GIPP_PN:  
			if(walker->SET_FLAG & FCS_IPP_SET) { 
				for(i=0; i<4; i++)
					buf[k++] = htonl(walker->ip_port[i]);
			} else {
				tx_fcs_resp(fi, s_id, ox_id, 9, NO_IPP);
				return 0;
				}
			break;
        } //end of the switch(cmd)
}

if(cmd >= GID_NN && cmd <= GSNN_NN) {
	found = 0;
	for(i=0; i<MAX_FPORT_CARDS; i++) {
		if(!found) walker = fc[i]->name_list;
		while(walker && !found) {
			if(walker->node_name_high == inputArray[0] && walker->node_name_low == inputArray[1]) 
			{  
				found = 1;
				break;	// if found this id, jump out the while loop
			}
			walker = walker->next;
		}
	} 	
	if( !found ) {   // if no device found, return
		tx_fcs_resp(fi, s_id, ox_id, 9, NO_NODE_NAME);
        return 0;
        }
                
        switch(cmd) {
                case GID_NN:
			buf[k++] = htonl(walker->portID);
                        break;
                case GIP_NN:
			if(walker->SET_FLAG & FCS_IP_SET) {
				 for(i=0; i<4; i++)
			 		buf[k++] = htonl(walker->ip[i]);
			} else {
				tx_fcs_resp(fi, s_id, ox_id, 9, NO_IP);
				return 0;
			}
                        break;
                case GIPA_NN:
			if(walker->SET_FLAG & FCS_IPA_SET) {
				for(i=0; i<2; i++)
				buf[k++] = htonl(walker->initial_pa[i]);
			} else {
				tx_fcs_resp(fi, s_id, ox_id, 9, NO_IPA);
				return 0;
			}
                        break;
                case GSNN_NN:
			if(walker->SET_FLAG & FCS_SNN_SET) {
				for(i=0; i<64; i++)
					buf[k++] = htonl(walker->s_node_name[i]);
			} else {
				tx_fcs_resp(fi, s_id, ox_id, 9, NO_SNN);
				return 0;
			} 
                        break;
        } //end of the switch(cmd)
}


if(cmd == GNN_IP || cmd == GIPA_IP) {
	found = 0;
	for(i=0; i<MAX_FPORT_CARDS; i++) {
		if(!found) walker = fc[i]->name_list;
		while(walker && !found) {
			if (walker->ip[0] == inputArray[0] && walker->ip[1] == inputArray[1] && 
				walker->ip[2] == inputArray[2] && walker->ip[3] == inputArray[3] ) 
			{  
				found = 1;
				break;	// if found this id, jump out the while loop
			}
			walker = walker->next;
		}
	} 	
	if( !found ) {   // if no device found, return
		tx_fcs_resp(fi, s_id, ox_id, 9, NO_IP);
        return 0;
        }
        switch(cmd) {
                case GNN_IP:
			buf[k++] = htonl(walker->node_name_high);
			buf[k++] = htonl(walker->node_name_low);
                        break;
                case GIPA_IP:
			if(walker->SET_FLAG & FCS_IPA_SET) {
				buf[k++] = htonl(walker->initial_pa[0]);
			 	buf[k++] = htonl(walker->initial_pa[1]);
			} else {
				tx_fcs_resp(fi, s_id, ox_id, 9, NO_IPA);
				return 0;
			}
                        break;
        } //end of the switch(cmd)
}


if(cmd == GID_FT || cmd == GNN_FT) {
	if( (inputArray[0] & 0x000000FF) != 0x00000008) {  // I only report SCSI_FCP
		tx_fcs_resp(fi, s_id, ox_id, 0x09, NO_FC4);
		return 0;
	}
	for(i=0; i<MAX_FPORT_CARDS; i++) {
		walker = fc[i]->name_list;
		while(walker) {
			id = walker->portID;
			if(cmd == GID_FT) {  
				buf[k++] = htonl(id);	// we assume any device support FCP (should)	
			} else { 
				   buf[k++] = htonl(id);
				   buf[k++] = 0;
				   buf[k++] = htonl(walker->node_name_high);
				   buf[k++] = htonl(walker->node_name_low);
			}
			walker = walker->next;
		}
	} 	
	if( k == 4) {   // if no device found, return
		tx_fcs_resp(fi, s_id, ox_id, 9, NO_FC4);
                return 0;
        }

        switch(cmd) {
                case GID_FT:
			buf[k-1] = ntohl(buf[k-1]) | 0x80000000; // set the last record flag
			buf[k-1] = htonl(buf[k-1]);
                        break;
                case GNN_FT:
			buf[k-4] = ntohl(buf[k-4]) | 0x80000000; // set the last record flag
			buf[k-4] = htonl(buf[k-4]);
                        break;
        } //end of the switch(cmd)
}


if(cmd == GID_PT) {
	for(i=0; i<MAX_FPORT_CARDS; i++) {
		walker = fc[i]->name_list;
		while(walker) {
			id = walker->portID;
		 	if(walker->portType == inputArray[0])
				buf[k++] = htonl(id);	// we assume any device support FCP (should)	
			walker = walker->next;
		}
	} 	
	if( k == 4) {   // if no device found, return
		tx_fcs_resp(fi, s_id, ox_id, 9, NO_PT);
                return 0;
        }
	buf[k-1] = ntohl(buf[k-1]) | 0x80000000; // set the last record flag
	buf[k-1] = htonl(buf[k-1]);

}


if(cmd == GID_IPP || cmd == GPN_IPP) {
	found = 0;
	for(i=0; i<MAX_FPORT_CARDS; i++) {
		if(!found) walker = fc[i]->name_list;
		while(walker && !found) {
			if (walker->ip_port[0] == inputArray[0] && walker->ip_port[1] == inputArray[1] && 
				walker->ip_port[2] == inputArray[2] && walker->ip_port[3] == inputArray[3] ) 
			{  
			    if(cmd == GID_IPP)
					{
					  buf[k++] = htonl(walker->portID);	// we assume any device support FCP (should)	
					}
					else
					{
					  found = 1;
					  buf[k++] = htonl(walker->port_name_high);
					  buf[k++] = htonl(walker->port_name_low);
					  break;	// if found this id, jump out the while loop
					}
			}
			walker = walker->next;
		}
	} 	
	if(k == 4) {   // if no device found, return
		tx_fcs_resp(fi, s_id, ox_id, 9, NO_IPP);
                return 0;
        }
	if(cmd == GID_IPP)	
		buf[k-1] = ntohl(buf[k-1]) | 0x80000000; // set the last record flag
		buf[k-1] = htonl(buf[k-1]);
                
}


if(cmd == DA_ID)
{
    found = 0;
	for(i=0; i<MAX_FPORT_CARDS; i++) {
		finder = NULL;  
        if(!found) walker = fc[i]->name_list;
		while(walker && !found) 
		{
		    id = walker->portID;
			if(id == inputArray[0]) 
			{  
				found = 1;
				if (finder==NULL)
				 fc[i]->name_list = NULL;  // change the name_list to zero (don't change the walker, it's no use :)= )
	      	     else { temp = walker;  // temp need to be freed
                        finder->next = walker->next;
	                  } 
				break;	// if found this id, jump out the while loop
			}
            finder = walker;
			walker = walker->next;
		}  // end of while
	} // end of for
	if( !found ) {   // if no device found, return
		tx_fcs_resp(fi, s_id, ox_id, 9, NO_PORT_ID);
        return 0;
    } else  
		{ tx_fcs_resp(fi, s_id, ox_id, 0, 0);
          return 0;
		}
    }  

	// k is the valus of buffer length
	memcpy(fi->g.els_buffer[fi->g.e_i], buf, k*4);   // buffer length in byte
	tx_exchange(fi, (char *)(fi->g.els_buffer[fi->g.e_i]), k*4, r_ctl, type, s_id, FRAME_SIZE, 1, ox_id, 0x01000000);

        fi->g.e_i++;
        if (fi->g.e_i == MAX_PENDING_FRAMES / 2)
                fi->g.e_i = 0; 
	return 0;
}  // end of function fcs_query




static struct fc_info *next_card(u_int number)
{
int i;

	for(i = 0; i < MAX_FC_CARDS; ++i) { 
		printk("FC card names: %s\n", fc[i]->name);
	  	printk("WWN: %x    %x\n", fc[i]->g.my_port_name_high, fc[i]->g.my_port_name_low);
        }
	i = (number + 1) % MAX_FPORT_CARDS;

	return fc[i];
}

static struct fc_info *find_card(u_int area_id)
{
	return fc[area_id-1];
}


static void efp_process(struct fc_info *fi, u_int *buff_addr, u_char cmd)
{		// cmd == 1: rxed an EFP command, cmd==0: rxed an EFP_ACC
u_int   priority, payload_size;   
u_int   i, base, size_left, status, null_domain_ids;
u_int   sw_name[2];    
u_int	same_id, s_id;
u_short received_ox_id;

			s_id = ntohl(*(buff_addr + 3)) & 0x00FFFFFF;
        		received_ox_id = ntohl(*(buff_addr + 6)) >> 16;
			priority = ntohl(*(buff_addr + 9)) & 0x000000FF; // priority
                        payload_size = ntohl(*(buff_addr + 8)) & 0x0000FFFF; // EFP payload size
			sw_name[0] = ntohl(*(buff_addr + 10));
                        sw_name[1] = ntohl(*(buff_addr + 11));
			if(payload_size % 0x10 != 0 ) {
                                T_MSG("Fatal error: EFP received has wrong payload size!");
                                return;
                        }

                        if(priority == 2 )  // if this is from a principal switch
                        {
                                if(fcsw->sm.psw_selected == 0) {	
                                        T_MSG("First time: Other Principal SW selected!");
                                        fcsw->sm.psw_selected = 1;
                                        fcsw->sm.psw_card = fi->g.card_order;    // if myself is psw, set to 0xFF
					if(cmd == 1)
						tx_efp(0, fi, s_id, received_ox_id); //flag ==0 : it is a ACC to EFP
                                } else {   //pending
					/* If the psw has been selected, I should have sent RDI and got the RDI_ACC, then
					 * the psw should tranmit an EFP containing my domain ID record
					 */
					DEBUG("Psw already selected, rxed EFP with priority equals 2!");
					fcsw->g.psw_priority = 2;
					fcsw->g.psw_name[0]  =  sw_name[0];
                                       	fcsw->g.psw_name[1]  =  sw_name[1];      // principal switch priority and name
					same_id = 0;
					if(fcsw->domain_ids == NULL)     
                                        	null_domain_ids = TRUE; 
                                       	else    null_domain_ids = FALSE;
					size_left = payload_size - 0x10;
                                	base = 12;   // count from the 12th word of the tachyon buffer
					while(size_left > 0) {
                                        	i = ntohl(*(buff_addr + base )) & 0x00FF0000;
                                        	i = i >> 16;
                                        	sw_name[0] = ntohl(*(buff_addr + base + 2));
                                        	sw_name[1] = ntohl(*(buff_addr + base + 3));
						status = efp_domainID_check(i, sw_name);
						switch(status) {
							case DOMAINID_NOT_IN:	// this sw_name is not in my domain id database
								add_to_domain_ids(i, sw_name);
								break;
							case DOMAINID_SAME: 	// this sw_name was alread in and got the same ID
								same_id = 1;
								break;
							case DOMAINID_CONFLICT: // this sw_name has got an ID different from this one
								T_MSG("efp_process: This really should not happen: A switch assigned different ids!");
								break;
							default:	
								T_MSG("Something shouldn't happen in efp_domainID_check");
						}		
                                        	size_left = size_left - 0x10;
                                        	base = base + 4;
                                	}        
					if(cmd == 1)
                                                tx_efp(0, fi, s_id, received_ox_id); //flag ==0 : it is a ACC to EFP
 
					if(null_domain_ids == FALSE  && same_id == 0) 
						// if my domain_ids is not null and nothing in the coming ELP is same as the ids in my domain_ids
					{
						tx_bf_rcf(SW_BF, fi, 0xfffffd);  //pending should tx to every card
					}	
                                }
			} else {	// the received priority is not 02
                                if(fcsw->sm.psw_selected == 0) {
                                        T_MSG("Principal SW not selected yet!");
					if(fcsw->g.psw_priority < priority)  	// less value has higher priority
					{					// this also handles the prioriy "FF"
						if(fi->sm.efp_timer_set == 0)
							set_efp_timer(fi);
					} else  if (fcsw->g.psw_priority  == priority) {
							if(  sw_name[0]  > fcsw->g.psw_name[0] ||
                                            		    (sw_name[0] == fcsw->g.psw_name[0] && sw_name[1] > fcsw->g.psw_name[1])
                                          		  )   // this psw is higher than current psw
							{
								if(fi->sm.efp_timer_set == 0)
									set_efp_timer(fi);
							}
						} else 	{
							T_MSG("I will not become psw: the coming priority: %x, sw_name: %x    %x", 
								priority, sw_name[0], sw_name[1]);
							//pending I have less priority, waiting for others	
							}
					if(cmd == 1)
                                                tx_efp(0, fi, s_id, received_ox_id); //flag ==0 : it is a ACC to EFP

				} else {    // if the principal switch already decided
					if(fcsw->sm.psw_card == I_AM_PSW) {
						if(cmd == 1)
                                                	tx_efp(0, fi, s_id, received_ox_id); //flag ==0 : it is a ACC to EFP
					  	T_MSG("EFP_Processing: I am psw and transmitting AAI."); 
						tx_aai(1, fi, 0xfffffd, 0);
                                        	add_to_ox_id_list(fi, fi->g.ox_id - 1, SW_AAI, FROM_MYSELF);
					} else {
						if(cmd == 1)
                                                        tx_efp(0, fi, s_id, received_ox_id); //flag ==0 : it is a ACC to EFP
						T_MSG("EFP_Processing: I am not psw and transmitting AAI.");
                                                tx_aai(1, fi, 0xfffffd, 0);
                                                add_to_ox_id_list(fi, fi->g.ox_id - 1, SW_AAI, FROM_MYSELF);
					}
				}
			}   // end of the if (priority != 02
} // end of this function


static u_int check_domainID(u_int id, u_int sw_name[])
{
struct domain_id *walker = fcsw->domain_ids;

	if(id == 0)	return DOMAINID_ZERO;
	if(id == fcsw->g.my_domainID && sw_name[0] == fcsw->g.mysw_name[0] && sw_name[1] == fcsw->g.mysw_name[1])
		return DOMAINID_MINE;    // this case happen only during my testing: using same switch name and id 

	// first round, check whether this switch has requested an ID before
	while(walker) {
		if( (walker->sw_name[0] == sw_name[0]) && (walker->sw_name[1] == sw_name[1]) ) {
			if(walker->id == id)
				return DOMAINID_SAME;
			else 	return DOMAINID_CONFLICT;
		}
		walker = walker->next;
	}
	// after the loop, this switch was proved to be a new one
	walker = fcsw->domain_ids;
	while(walker) {
		if(walker->id == id)
			return DOMAINID_NOT_OK;
		walker = walker->next;
	}
	// after this loop, there is no such id
	return DOMAINID_OK;
}


static u_int efp_domainID_check(u_int id, u_int sw_name[])
{
struct domain_id *walker = fcsw->domain_ids;
 
        // first round, check whether this switch has requested an ID before
        while(walker) {
                if( (walker->sw_name[0] == sw_name[0]) && (walker->sw_name[1] == sw_name[1]) ) {
                        if(walker->id == id)
                                return DOMAINID_SAME;
                        else    return DOMAINID_CONFLICT;
                }
                walker = walker->next;
        }

        // after the loop, this switch was proved to be a new one
        return DOMAINID_NOT_IN;
}

static void show_domain_ids(void)
{
struct domain_id *p = fcsw->domain_ids;
 
	printk("================================ Domain IDs =================================\n");
	while(p) {
		printk("Domain_ID: %x   Switch name: %x    %x\n", p->id, p->sw_name[0], p->sw_name[1]);
		p = p->next;
		printk("-----------------------------------------------------------------------------\n");
	}

	printk("=============================================================================\n");

}

static u_int grab_a_domainID()
{
u_int  id, status;
u_int  sw_name[2];

	sw_name[0] = 0;
	sw_name[1] = 0; 	// this switch has never been granted a domain ID, so just use a fake sw name	
	printk("Current domain ID: %x\n", fcsw->sm.cur_domainID);
	id = fcsw->sm.cur_domainID++;
	while(id != 0) {
		status = check_domainID(id, sw_name);
		if(status == DOMAINID_OK) {
			return id;
		} else   	// urh, this id was already granted
			id = fcsw->sm.cur_domainID++; 
        }	
	// all avalable domainIDs were used out
       	return DOMAINID_USED_OUT; 
}

static void set_efp_timer(struct fc_info *fi)
{
	DEBUG("%lu  Set EFP timer", jiffies);
	fi->sm.efp_timer.function = efp_timer;
	fi->sm.efp_timer.data = (unsigned long)fi;
	fi->sm.efp_timer.expires = RUN_AT(2 * F_S_TOV * HZ);        // 2*F_S_TOV
	init_timer(&fi->sm.efp_timer);
	add_timer(&fi->sm.efp_timer);
	fi->sm.efp_timer_set = 1;
}

static void elp_timer(unsigned long data)
{
struct fc_info *fi = (struct fc_info*)data;
	tx_elp(1, fi, FABRIC_CONTROLLER, 0); //flag ==0 : it is a ACC to ELP
	add_to_ox_id_list(fi, fi->g.ox_id - 1, SW_ELP, FROM_MYSELF);
	fi->sm.elp_timer_set = FALSE;
}


static void efp_timer(unsigned long data)
{
struct fc_info *fi = (struct fc_info*)data;
u_int   sw_name[2];

	fcsw->sm.psw_selected = 1;
	fcsw->g.psw_priority = 2;
	sw_name[0] = fcsw->g.psw_name[0];
	sw_name[1] = fcsw->g.psw_name[1];

	add_to_domain_ids(fcsw->g.my_domainID, sw_name);

	tx_efp(1, fi, FABRIC_CONTROLLER, 0); //flag ==0 : it is a ACC to ELP
	add_to_ox_id_list(fi, fi->g.ox_id - 1, SW_EFP, I_AM_PSW);	// from_card is 0xFF means this EFP is the EFP after 2*F_S_TOV
	fi->sm.efp_timer_set = 0;
}


static void bf_timer(unsigned long data)
{
struct fc_info *fi = (struct fc_info*)data;

	tx_efp(1, fi, 0xFFFFFD, 0);
	fi->sm.bf_timer_set = 0;

}


static void set_hello_timer(struct fc_info *fi)
{
	
    if(fi->rt.hello_timer_set == 0) {
        DEBUG("%lu  Set HELLO timer.", jiffies);
        fi->rt.hello_timer.function = hello_timer;
        fi->rt.hello_timer.data = (unsigned long)fi;
        fi->rt.hello_timer.expires = RUN_AT(HELLO_INTV * HZ);        // HELLO_INTV = 20 seconds
        init_timer(&fi->rt.hello_timer);
        add_timer(&fi->rt.hello_timer);
        fi->rt.hello_timer_set = 1;
    }
}


static void hello_timer(unsigned long data)
{
struct fc_info *fi = (struct fc_info*)data;

	tx_hello(fi, fi->rt.adj_domain);
	fi->rt.hello_txed = 1;
        DEBUG("%lu  HELLO timer handler.", jiffies);
	fi->rt.hello_timer_set = 0;
	set_hello_timer(fi);
}


static void set_dead_timer(struct fc_info *fi)
{
            	if(fi->rt.dead_timer_set == 0) {
			DEBUG("%lu  Set Hello dead timer.", jiffies);
                        fi->rt.dead_timer.function = dead_timer;   
                        fi->rt.dead_timer.data = (unsigned long)fi;
                        fi->rt.dead_timer.expires = RUN_AT(DEAD_INTV * HZ);        // DEAD_INTV = 80 seconds
                        init_timer(&fi->rt.dead_timer);
                        add_timer(&fi->rt.dead_timer);
                        fi->rt.dead_timer_set = 1;
		}
}

static void dead_timer(unsigned long data)
{
struct fc_info *fi = (struct fc_info*)data;
                        
	DEBUG("Hello dead timer handler!");
	if(fi->rt.hello_rxed == 0) {
        	DEBUG("%lu  Has not received HELLO for 80 seconds. Link down.", jiffies);
		fi->rt.adj_domain = NONE;
		// the connection maybe down
	}
	fi->rt.hello_rxed = 0;
        fi->rt.dead_timer_set = 0;
	set_dead_timer(fi);	// restart the timer.
}


static void nos_ols_timer(unsigned long data)
{
struct fc_info *fi = (struct fc_info*)data;
u_int fm_status;
	fm_status = readl(fi->t_r.ptr_fm_status_reg);
	DEBUG("FM_status in timer= %x", fm_status);
	fi->g.nport_timer_set = FALSE;
	del_timer(&fi->nport_timer);
	if ((fi->g.ptp_up == TRUE) || (fi->g.loop_up == TRUE))
		return;
	if (((fm_status & 0xF0) == FM_STATUS_OLD_PORT) && (((fm_status & 0x0F) == PORT_STATE_ACTIVE) || ((fm_status & 0x0F) == PORT_STATE_OFFLINE))) {
		DEBUG("In OLD-PORT after E_D_TOV.");
		take_tachyon_offline(fi);
		/* write R_T_TOV & E_D_TOV into the registers */
		writel(PTP_TOV_VALUES, fi->t_r.ptr_fm_tov_reg);
		writel(FM_PT2PT_BB_CR | FM_NPORT_CONNECTION, fi->t_r.ptr_fm_config_reg);
		fi->g.n_port_try = TRUE;
		DEBUG("In timer, TACHYON initializing as N_Port...\n");
		writel(FMCNTL_CMD_INITIALIZE, fi->t_r.ptr_fm_control_reg);
	}
	else
	if ((fi->g.lport_timer_set == FALSE) && ((fm_status & 0xF0) == FM_STATUS_LOOP_FAIL)) {
		DEBUG("Loop Fail after E_D_TOV.");
		fi->lport_timer.function = loop_timer;
		fi->lport_timer.data = (unsigned long)fi;
		fi->lport_timer.expires = RUN_AT((8*HZ)/100); /* 40 msec */
		init_timer(&fi->lport_timer);
		add_timer(&fi->lport_timer);
		fi->g.lport_timer_set = TRUE;
		take_tachyon_offline(fi);
		/* write AL_TIME & E_D_TOV into the registers */
		writel(TOV_VALUES, fi->t_r.ptr_fm_tov_reg);
		writel(FM_SOFT_ADDRESS, fi->t_r.ptr_fm_config_reg);
		DEBUG("In timer, TACHYON initializing as L_Port...\n");
		writel(FMCNTL_CMD_INITIALIZE, fi->t_r.ptr_fm_control_reg);
	}
	else
	if (((fm_status & 0xF0) == FM_STATUS_OLD_PORT) && (((fm_status & 0x0F) == PORT_STATE_LF1) || ((fm_status & 0x0F) == PORT_STATE_LF2))) {
		DEBUG("Link Fail in OLD-PORT.");
		take_tachyon_offline(fi);
		writel(FMCNTL_CMD_CLEAR_LF, fi->t_r.ptr_fm_control_reg);
		/* write R_T_TOV & E_D_TOV into the registers */
		writel(PTP_TOV_VALUES, fi->t_r.ptr_fm_tov_reg);
		writel(FM_PT2PT_BB_CR | FM_NPORT_CONNECTION, fi->t_r.ptr_fm_config_reg);
		fi->g.n_port_try = TRUE;
		DEBUG("In timer, TACHYON initializing as N_Port...\n");
		writel(FMCNTL_CMD_INITIALIZE, fi->t_r.ptr_fm_control_reg);
	}
}

static void loop_timer(unsigned long data)
{
struct fc_info *fi = (struct fc_info*)data;
	fi->g.lport_timer_set = FALSE;
	del_timer(&fi->lport_timer);
	if ((fi->g.ptp_up == TRUE) || (fi->g.loop_up == TRUE))
		return;
	/* Else try to come up as an N_Port */
	//take_tachyon_offline(fi);
	/* write R_T_TOV & E_D_TOV into the registers */
	//writel(PTP_TOV_VALUES, fi->t_r.ptr_fm_tov_reg);
	//writel(FM_PT2PT_BB_CR | FM_NPORT_CONNECTION, fi->t_r.ptr_fm_config_reg);
	//fi->g.n_port_try = TRUE;
	//DEBUG("In timer, TACHYON initializing as N_Port...\n");
	//writel(FMCNTL_CMD_INITIALIZE, fi->t_r.ptr_fm_control_reg);
}


static void fl_port_init(struct fc_info *fi)
{
                        DEBUG("Initializing Loop as FL_Port...");
			 take_tachyon_offline(fi);
                        /* write AL_TIME & E_D_TOV into the registers */
                        writel(TOV_VALUES, fi->t_r.ptr_fm_tov_reg);
                        /* tell Tachyon to Intialize as an FL_Port */
                        writel(FM_FABRIC_ADDRESS | FM_INIT_FABRIC | FM_FABRIC_LOGIN | FM_RESPOND_FABRIC, fi->t_r.ptr_fm_config_reg);
                        fi->g.i_am_fabric = TRUE;
                        writel(FMCNTL_CMD_INITIALIZE, fi->t_r.ptr_fm_control_reg);
}

static void f_port_init(struct fc_info *fi)
{
			DEBUG("Initializing Loop as F_Port...");
                        take_tachyon_offline(fi);
                        /* write R_T_TOV & E_D_TOV into the registers */
                        writel(PTP_TOV_VALUES, fi->t_r.ptr_fm_tov_reg);
                        writel(FM_PT2PT_BB_CR | FM_NPORT_CONNECTION | FM_FABRIC_ADDRESS | FM_INIT_FABRIC | FM_FABRIC_LOGIN |
                                FM_RESPOND_FABRIC, fi->t_r.ptr_fm_config_reg);
                        fi->g.n_port_try = TRUE;
                        fi->g.i_am_fabric = TRUE;
                        writel(FMCNTL_CMD_INITIALIZE, fi->t_r.ptr_fm_control_reg);
}

static void eport_flush(struct fc_info *fi) 
{ 
			DEBUG("Flushing sitch parameters...");
                        fi->g.i_am_fabric = TRUE;
                        fi->g.i_am_E_Port = TRUE;
                        fi->sm.psw_selected = 0;        // principal switch not selected yet
                        fi->sm.elp_done = 0;
                        fi->sm.elp_timer_set = 0;
                        fi->sm.efp_timer_set = 0;
                        fi->sm.cur_domainID  = FIRST_DOMAINID;
                        writel(FMCNTL_CMD_INITIALIZE, fi->t_r.ptr_fm_control_reg);
}

static void reset_tachyon(struct fc_info *fi, u_int value)
{
u_int tachyon_status, reset_done = OCQ_RESET_STAT | HI_PRI_RESET_STAT;
int not_done = 1, i = 0;
	ENTER("tachyon_reset");
	writel(value, fi->t_r.ptr_tach_control_reg);
	if (value == OCQ_RESET) 
		fi->q.ocq_prod_indx = 0;
	tachyon_status = readl(fi->t_r.ptr_tach_status_reg);

	/* Software resets are immediately done, whereas other aren't. It 
	about 30 clocks to do the reset */
	if (value != SOFTWARE_RESET) {
		while(not_done) {
			if (i++ > 100000) {
				T_MSG("Reset was unsuccessful! Tachyon Status = %x", tachyon_status);
				break;
			}
			tachyon_status = readl(fi->t_r.ptr_tach_status_reg);
			if ((tachyon_status & reset_done) == 0)
				not_done = 0;
		}
	}
	else {
		write_to_tachyon_registers(fi);
	}
	LEAVE("tachyon_reset");
}

static void take_tachyon_offline(struct fc_info *fi)
{
u_int fm_status = readl(fi->t_r.ptr_fm_status_reg);
	/* Take Tachyon Offline. The first 2 conditions will never be
	 * TRUE. The Errata says to do it a bit differently but it
	 * doesn't seem to work very well. This implementation seems to be
	 * work decently.
	 */
	if (fm_status == FM_STATUS_LOOP_FAIL) {
		// workaround as in P. 89 
		writel(FMCNTL_CMD_HOST_CTRL, fi->t_r.ptr_fm_control_reg);
		writel(FMCNTL_CMD_OFFLINE, fi->t_r.ptr_fm_control_reg);
		writel(FMCNTL_CMD_EXIT_HOST, fi->t_r.ptr_fm_control_reg);
	}
	else
	if (fm_status == FM_STATUS_LP) {
		// as defined in P. 89 
		writel(0x0, fi->t_r.ptr_fm_config_reg);
		writel(FMCNTL_CMD_INITIALIZE, fi->t_r.ptr_fm_control_reg);
		writel(FMCNTL_CMD_OFFLINE, fi->t_r.ptr_fm_control_reg);
	}
	else
		writel(FMCNTL_CMD_OFFLINE, fi->t_r.ptr_fm_control_reg);
}


static void read_novram(struct fc_info *fi)
{
int off = 0;
	fi->n_r.ptr_novram_hw_control_reg = fi->i_r.ptr_ichip_hw_control_reg; 
	fi->n_r.ptr_novram_hw_status_reg = fi->i_r.ptr_ichip_hw_status_reg; 
	iph5526_nr_do_init(fi);
	if (fi->clone_id == 0x107e)
		off = 32;
	
	/* This might be wrong. Never mind... It will work anyways.
	 */
	if (fi->clone_id == 0x133d)
		off = 52;

	fi->g.my_node_name_high = (fi->n_r.data[off] << 16) | fi->n_r.data[off+1];
	fi->g.my_node_name_low = (fi->n_r.data[off+2] << 16) | fi->n_r.data[off+3];
	fi->g.my_port_name_high = (fi->n_r.data[off+4] << 16) | fi->n_r.data[off+5];
	fi->g.my_port_name_low = (fi->n_r.data[off+6] << 16) | fi->n_r.data[off+7];
	if (fi->clone_id == 0x133d) {
		fi->g.my_port_name_low = fi->g.my_node_name_low;
		fi->g.my_port_name_high = fi->g.my_node_name_high;
	}
	DPRINTK1("node_name = %x %x", fi->g.my_node_name_high, fi->g.my_node_name_low);
	DPRINTK1("port_name = %x %x", fi->g.my_port_name_high, fi->g.my_port_name_low);
}

static void reset_ichip(struct fc_info *fi)
{
	/* (i)chip reset */
	writel(ICHIP_HCR_RESET, fi->i_r.ptr_ichip_hw_control_reg);
	/*wait for chip to get reset */
	udelay(10000);
	/*de-assert reset */
	writel(ICHIP_HCR_DERESET, fi->i_r.ptr_ichip_hw_control_reg);
	
	/* enable INT lines on the (i)chip */
	writel(ICHIP_HCR_ENABLE_INTA , fi->i_r.ptr_ichip_hw_control_reg);
	/* enable byte swap */
	writel(ICHIP_HAMR_BYTE_SWAP_ADDR_TR, fi->i_r.ptr_ichip_hw_addr_mask_reg);
}

static void tx_logi(struct fc_info *fi, u_int logi, u_int d_id)
{
int int_required = 1;
u_short ox_id = OX_ID_FIRST_SEQUENCE;
u_int r_ctl = RCTL_ELS_UCTL;
u_int type  = TYPE_ELS | SEQUENCE_INITIATIVE | FIRST_SEQUENCE;
u_int my_mtu = fi->g.my_mtu;
	ENTER("tx_logi");
	/* We dont want interrupted for our own logi. 
	 * It screws up the port discovery process. 
	 */
	if (d_id == fi->g.my_id)
		int_required = 0;
	fill_login_frame(fi, logi);	
	fi->g.type_of_frame = FC_ELS;
	memcpy(fi->g.els_buffer[fi->g.e_i], &fi->g.login, sizeof(LOGIN));
	tx_exchange(fi, (char *)(fi->g.els_buffer[fi->g.e_i]),sizeof(LOGIN), r_ctl, 
type, d_id, my_mtu, int_required, ox_id, logi);
	//tx_exchange(fi, (char *)(&fi->g.login),sizeof(LOGIN), r_ctl, type, d_id, my_mtu, int_required, ox_id, logi);
	fi->g.e_i++;
	if (fi->g.e_i == MAX_PENDING_FRAMES/2)
		fi->g.e_i = 0;
	LEAVE("tx_logi");
	return;
}

static void tx_prli(struct fc_info *fi, u_int d_id)             
{
int int_required = 1;
u_short ox_id = OX_ID_FIRST_SEQUENCE;
u_int r_ctl = RCTL_ELS_UCTL;
u_int type  = TYPE_ELS | SEQUENCE_INITIATIVE | FIRST_SEQUENCE;
u_int my_mtu = fi->g.my_mtu;
        ENTER("tx_prli");
        fi->g.prli.cmnd_code = htons(ELS_PRLI >> 16);
        fi->g.prli.payload_length = htons(PRLI_LEN);
        fi->g.prli.type_code = htons(FCP_TYPE_CODE);
        fi->g.prli.est_image_pair = htons(IMAGE_PAIR);
        fi->g.prli.responder_pa = 0;
        fi->g.prli.originator_pa = 0;
        fi->g.prli.service_params = htonl(INITIATOR_FUNC | READ_XFER_RDY_DISABLED);
        fi->g.type_of_frame = FC_ELS;
        memcpy(fi->g.els_buffer[fi->g.e_i], &fi->g.prli, sizeof(PRLI));
        tx_exchange(fi, (char *)(fi->g.els_buffer[fi->g.e_i]), sizeof(PRLI), r_ctl, type, d_id, my_mtu,
                int_required, ox_id, ELS_PRLI);
        fi->g.e_i++;
        if (fi->g.e_i == MAX_PENDING_FRAMES)
                fi->g.e_i = 0;
        LEAVE("tx_prli");
        return;
}



static void tx_generic(struct fc_info *fi, u_int data[], u_int r_ctl, u_int type)
{
u_int d_id;
int k, logi;
u_int buf[50];
int buf_int;
int int_required = 1;
u_short ox_id = 0x1234;

u_int my_mtu = fi->g.my_mtu;


        ENTER("tx_els");

        fi->g.my_id = data[ data[0]+1 ];
        d_id = data[ data[0] +2 ];
        if (d_id == fi->g.my_id)
                int_required = 0;
        fi->g.type_of_frame = FC_ELS;
        buf_int = data[0];
        logi = data[1];
        for(k=0; k<buf_int; k++)
        {       buf[k] = data[k+1];
                buf[k] = htonl( buf[k] );
        }
 
        memcpy(fi->g.els_buffer[fi->g.e_i], buf, buf_int*4);
        tx_exchange(fi, (char *)(fi->g.els_buffer[fi->g.e_i]), buf_int*4,
         r_ctl, type, d_id, my_mtu, int_required, ox_id, logi);

        fi->g.e_i++;
        if (fi->g.e_i == MAX_PENDING_FRAMES)
                fi->g.e_i = 0;
        LEAVE("tx_els");
        return;
}


static void tx_ack1(struct fc_info *fi, u_int d_id, u_short ox_id, u_short rx_id)
{
fi->g.tach_header.resv = 0x00000000;
       
fi->g.tach_header.sof_and_eof = SOFF | EOFN;            // a char
fi->g.tach_header.dest_alpa = d_id & 0x000000FF;        // a char
fi->g.tach_header.lcr_and_time_stamp = 0;

fi->g.tach_header.r_ctl_and_d_id = htonl(0xC0FFFFFD);   // header word 0
fi->g.tach_header.vc_id_and_s_id = htonl(0x00FFFFFD);   // header word 1   
fi->g.tach_header.type_and_f_cntl = htonl(0x00580000);  // header word 2
        
fi->g.tach_header.seq_id = 0;
fi->g.tach_header.df_cntl= 0;
fi->g.tach_header.seq_cnt = 0;                          // header word 3
        
fi->g.tach_header.ox_id = htons(ox_id);
fi->g.tach_header.rx_id = htons(rx_id);                // header word 4

fi->g.tach_header.ro     = htonl(0x00000001);           // header word 5
        
       memcpy(fi->q.ptr_tachyon_header[fi->q.tachyon_header_indx], &(fi->g.tach_header), HEADER_LEN);

       build_ODB(fi, 0, 0xFFFFFD, 0, 0, FRAME_SIZE, ox_id, rx_id, 0, 0, 0);

        memcpy(fi->q.ptr_odb[fi->q.ocq_prod_indx], &(fi->g.odb), sizeof(ODB));

       if(fi->g.i_am_E_Port == TRUE)   // righ now only E_Port using ACK_1
        {
 		// DEBUG("E_Port: g.ptp_up: %x", fi->g.ptp_up);
                if(fi->g.ptp_up != TRUE) {
                        T_MSG("tx_ack1: Fibre Channel Link not up. Dropping Exchange!");
                        return;
                }   
                else {
                        update_OCQ_indx(fi);
			// don't need to increase rx_id here, the following SW_ACC will use the same rx_id
                }
        }
        else 
	    T_MSG("Something wrong, this port should be an E_Port!");

          //update_EDB_indx(fi);
          update_tachyon_header_indx(fi);

}


static void tx_elp(u_int flag, struct fc_info *fi, u_int d_id, u_short received_ox_id)
{
int int_required = 0;
u_int r_ctl, type;
u_int my_mtu = fi->g.my_mtu;
ELP   elp;
u_int e_port = 0;
u_int i, ox_id;

udelay(40);

 fi->g.my_id = FABRIC_CONTROLLER;

e_port = fi->g.card_order * 0x10000;
                 // change the byte 2 of the wwn to e_port_name (WWN_high + XX0000)


	if(flag == 1)	// transmit an ELP, otherwise an ELP_ACC 
 	{
  	r_ctl = RCTL_SW;
  	type = TYPE_SW | FIRST_SEQUENCE | END_SEQUENCE | SEQUENCE_INITIATIVE;
	elp.cmnd_code  = htonl(0x10000000);
	ox_id = fi->g.ox_id;
	fi->g.type_of_frame = FC_SW;
	}  else {   
	     	r_ctl = RCTL_SW_RESP;   
    		type  = TYPE_SW | EXCHANGE_RESPONDER | LAST_SEQUENCE  | END_SEQUENCE;
		elp.cmnd_code  = htonl(0x02000000);
		ox_id = received_ox_id;    
		fi->g.type_of_frame = FC_SW_ACC;
		}

        ENTER("tx_elp");  
	
        elp.revision   = htonl(0x01000000);
        elp.ra_tov     = htonl(R_A_TOV);
        elp.ed_tov     = htonl(E_D_TOV); 
        elp.E_Port_Name[0] = htonl( fi->g.mysw_name[0] | e_port) ;
        elp.E_Port_Name[1] = htonl( fi->g.mysw_name[1] );

        elp.Switch_Name[0] = htonl(fi->g.mysw_name[0]);      
        elp.Switch_Name[1] = htonl(fi->g.mysw_name[1]);

        elp.class_f_valid  = htonl(0x80000000);
        elp.class_f_XII	   = 0;     
        elp.class_f_frame_size = htons(0x0840);         // receive data field size
        elp.class_f_con_seq    = htons(0x0001);         // concurrent sequences
        elp.class_f_credit     = htons(0x0001);         // end-to-end credit
        elp.class_f_seqs       = htonl(0x00010000);     // open sequences per exchange, only the fire short useful

        elp.class_1_params     = 0;         // class 1 parameters
        elp.class_2_params     = htonl(0x88000840);         // class 2 parameters
        elp.class_3_params     = htonl(0x88000840);         // class 3 parameters
        
        for(i=0; i<5; i++)        
	  	elp.reserved[i] = 0;            // reserved 20 bytes
    
        elp.ISL_mode = 0;               // ISL flow control mode
        elp.flow_cntl_len = 0;          // flow control parameter length

        //	elp.ISL_mode = htons(0x1000);               // ISL flow control mode
        //	elp.flow_cntl_len = htons(0x0004);          // flow control parameter length
        
        memcpy(fi->g.els_buffer[fi->g.e_i], &elp, sizeof(ELP));
        tx_exchange(fi, (char *)(fi->g.els_buffer[fi->g.e_i]),sizeof(ELP),
                 r_ctl, type, d_id, my_mtu, int_required, ox_id, SW_ELP);
        fi->g.e_i++;
        if (fi->g.e_i == MAX_PENDING_FRAMES/2)
                fi->g.e_i = 0;  
        LEAVE("tx_elp");
        return;
}

static void tx_esc(u_int flag, struct fc_info *fi, u_int d_id, u_short received_ox_id)
{
int int_required = 0;
u_int r_ctl, type;
u_int my_mtu = fi->g.my_mtu;
ESC   esc;
u_int i, ox_id;
                
 fi->g.my_id = FABRIC_CONTROLLER;
        
        if(flag == 1)   // transmit an ESC, otherwise an ESC_ACC
        {
        r_ctl = RCTL_SW;
        type = TYPE_SW | FIRST_SEQUENCE | END_SEQUENCE | SEQUENCE_INITIATIVE;
        esc.cmnd_code  = htons(0x3000);
	ox_id = fi->g.ox_id;
	fi->g.type_of_frame = FC_SW;
        }  else {
                r_ctl = RCTL_SW_RESP;
                type  = TYPE_SW | EXCHANGE_RESPONDER | LAST_SEQUENCE  | END_SEQUENCE;
                esc.cmnd_code  = htons(0x0200);
		ox_id = received_ox_id;
		fi->g.type_of_frame = FC_SW_ACC;
                }
        
        esc.verderID[0] = htonl(0x58440048);	// XD H
	esc.verderID[1] = htonl(0x75616E67);	// uang
        for(i=0; i<2; i++)
                esc.protocol[i] = 0;            
        esc.protocol[2] = htonl(2);
   	esc.payload_len = htons(0x18);        

        memcpy(fi->g.els_buffer[fi->g.e_i], &esc, sizeof(ESC));
        tx_exchange(fi, (char *)(fi->g.els_buffer[fi->g.e_i]),sizeof(ESC),
                 r_ctl, type, d_id, my_mtu, int_required, ox_id, SW_ESC);
        fi->g.e_i++;
        if (fi->g.e_i == MAX_PENDING_FRAMES/2)
                fi->g.e_i = 0;
        return;
}


static void tx_efp(u_int flag, struct fc_info *fi, u_int d_id, u_short received_ox_id)
{
int int_required = 0;
u_int r_ctl, type;
u_int my_mtu = fi->g.my_mtu;
EFP   efp;
u_int len = 0x10;
u_int buf[4];
u_int *base;
u_int i = 0;
u_int ox_id;
struct domain_id *walker = fcsw->domain_ids; 
        
 fi->g.my_id = FABRIC_CONTROLLER;
        
        if(flag == 1)   // transmit an EFP, otherwise an EFP_ACC
        {
        r_ctl = RCTL_SW;
        type = TYPE_SW | FIRST_SEQUENCE | END_SEQUENCE | SEQUENCE_INITIATIVE;
        efp.cmnd_code  = 0x11;
	ox_id = fi->g.ox_id;
	fi->g.type_of_frame = FC_SW;
        }  else {
                r_ctl = RCTL_SW_RESP;
                type  = TYPE_SW | EXCHANGE_RESPONDER | LAST_SEQUENCE  | END_SEQUENCE;
                efp.cmnd_code  = 0x02;
		ox_id = received_ox_id;
		fi->g.type_of_frame = FC_SW_ACC;
                }

        efp.rcd_lengh  = 0x10;   // = 0x10
        efp.priority   = htonl(fcsw->g.psw_priority);
	efp.sw_name[0] = htonl(fcsw->g.psw_name[0]);
	efp.sw_name[1] = htonl(fcsw->g.psw_name[1]);

	// if any Domain ID record added, the payload length should be increased
	base = fi->g.els_buffer[fi->g.e_i] + sizeof(EFP)/4;    	// record the start address where the domain ID can be added
							  	// added value is the word aligned address 
	while(walker)
	{
		buf[0] = (walker->type << 24) + (walker->id << 16);
		buf[0] = htonl(buf[0]);
		buf[1] = 0;
		buf[2] = htonl(walker->sw_name[0]);
		buf[3] = htonl(walker->sw_name[1]);	// create a temp domain id record 
		memcpy(base, buf, 4*4); // copy the buffer into the ELS temp buff
   		base = base + 4;	// increase the base address
		len  = len + 16;	// increase the pay_load length by 16 bytes 
        	i++;
		if(i > MAX_DOMAIN_IDS) {
			T_MSG("Error: tx_efp: domain ids more than MAX_DOMAIN_IDS: %d", i);
			break; 	//stop here
		}
		walker = walker->next;		
	}

	efp.payload_len = htons(len);

        memcpy(fi->g.els_buffer[fi->g.e_i], &efp, sizeof(EFP));
        tx_exchange(fi, (char *)(fi->g.els_buffer[fi->g.e_i]), len,
                 r_ctl, type, d_id, my_mtu, int_required, ox_id, SW_EFP);
        fi->g.e_i++;
        if (fi->g.e_i == MAX_PENDING_FRAMES/2)
                fi->g.e_i = 0;
        return;
}

static void tx_aai(u_int flag, struct fc_info *fi, u_int d_id, u_short received_ox_id)
{
int int_required = 0;
u_int r_ctl, type;
u_int my_mtu = fi->g.my_mtu;
AAI   aai;
u_int id, ox_id;
        
 fi->g.my_id = FABRIC_CONTROLLER;
        
        if(flag == 1)   // transmit an AAI, otherwise an AAI_ACC
        {
        r_ctl = RCTL_SW;
        type = TYPE_SW | FIRST_SEQUENCE | END_SEQUENCE | SEQUENCE_INITIATIVE;
        aai.cmnd_code  = htonl(0x12000000);
	ox_id = fi->g.ox_id;
	fi->g.type_of_frame = FC_SW;
        }  else {
                r_ctl = RCTL_SW_RESP;
                type  = TYPE_SW | EXCHANGE_RESPONDER | LAST_SEQUENCE  | END_SEQUENCE;
                aai.cmnd_code  = htonl(0x02000000);
		ox_id = received_ox_id;
		fi->g.type_of_frame = FC_SW_ACC;
                }

        aai.sw_name[0] = htonl(fcsw->g.mysw_name[0]);
        aai.sw_name[1] = htonl(fcsw->g.mysw_name[1]);

	id = fi->g.my_domainID * 0x10000 + (fi->g.card_order+1) * 0x100;  // card 0 has area_id 1      
	aai.orig_id = htonl(id);
        
        memcpy(fi->g.els_buffer[fi->g.e_i], &aai, sizeof(AAI));
        tx_exchange(fi, (char *)(fi->g.els_buffer[fi->g.e_i]),sizeof(AAI),
                 r_ctl, type, d_id, my_mtu, int_required, ox_id, SW_AAI);
        fi->g.e_i++;
        if (fi->g.e_i == MAX_PENDING_FRAMES/2)
                fi->g.e_i = 0;
        return;
}               

static void tx_rdi(u_int flag, struct fc_info *fi, u_int d_id, u_short received_ox_id, u_int id, u_int sw_name[])
{
int int_required = 0;
u_int r_ctl, type;
u_int my_mtu = fi->g.my_mtu;
RDI   rdi;
u_int ox_id;
        
 fi->g.my_id = FABRIC_CONTROLLER;
        
        if(flag == 1)   // transmit an RDI, otherwise an RDI_ACC
        {
        r_ctl = RCTL_SW;
        type = TYPE_SW | FIRST_SEQUENCE | END_SEQUENCE | SEQUENCE_INITIATIVE;
        rdi.cmnd_code  = htons(0x1300);
	ox_id = fi->g.ox_id;
	fi->g.type_of_frame = FC_SW;
        }  else {
                r_ctl = RCTL_SW_RESP;
                type  = TYPE_SW | EXCHANGE_RESPONDER | LAST_SEQUENCE  | END_SEQUENCE;
                rdi.cmnd_code  = htons(0x0200);
		ox_id = received_ox_id;
		fi->g.type_of_frame = FC_SW_ACC;
                }
        
        rdi.sw_name[0] = htonl(sw_name[0]);
        rdi.sw_name[1] = htonl(sw_name[1]);

        rdi.requested_id = htonl(id);

        //normally we only ask for one domainID, if more than one, the payload_len shoud increase
	rdi.payload_len = htons(0x10);
        memcpy(fi->g.els_buffer[fi->g.e_i], &rdi, sizeof(RDI));
        tx_exchange(fi, (char *)(fi->g.els_buffer[fi->g.e_i]),sizeof(RDI),
                 r_ctl, type, d_id, my_mtu, int_required, ox_id, SW_RDI);
        fi->g.e_i++;
        if (fi->g.e_i == MAX_PENDING_FRAMES/2)
                fi->g.e_i = 0;
        return;
}

static void tx_sw_acc(struct fc_info *fi, u_int d_id, u_short received_ox_id)
{
int int_required = 0;
u_int r_ctl, type;
u_int my_mtu = fi->g.my_mtu;
u_int buf[1];
u_int ox_id;
 
 fi->g.my_id = FABRIC_CONTROLLER;
 fi->g.type_of_frame = FC_SW_ACC;
        
        r_ctl = RCTL_SW_RESP;
        type  = TYPE_SW | EXCHANGE_RESPONDER | LAST_SEQUENCE  | END_SEQUENCE;
        ox_id = received_ox_id;
        buf[0]  = htonl(SW_ACC);        // comand code

        memcpy(fi->g.els_buffer[fi->g.e_i], buf, 4);    // totally the payload length is 4 bytes
        tx_exchange(fi, (char *)(fi->g.els_buffer[fi->g.e_i]), 4,
                 r_ctl, type, d_id, my_mtu, int_required, ox_id, SW_ACC);
        fi->g.e_i++;
        if (fi->g.e_i == MAX_PENDING_FRAMES/2)
               fi->g.e_i = 0;
        return;
}


static void tx_sw_rjt(struct fc_info *fi, u_int d_id, u_short received_ox_id, u_char reason, u_char reason_expl)
{
int int_required = 0;
u_int r_ctl, type;
u_int my_mtu = fi->g.my_mtu;
u_int buf[2];
u_int ox_id;
                
 fi->g.my_id = FABRIC_CONTROLLER;
 fi->g.type_of_frame = FC_SW_ACC;
       
       	r_ctl = RCTL_SW_RESP;
        type  = TYPE_SW | EXCHANGE_RESPONDER | LAST_SEQUENCE  | END_SEQUENCE;
	ox_id = received_ox_id;
        buf[0]  = htonl(SW_RJT);	// comand code
	buf[1]  = (reason << 16) + (reason_expl << 8);  // +- has higher priority than << >>
	buf[1]  = htonl(buf[1]);
                                
        memcpy(fi->g.els_buffer[fi->g.e_i], buf, 8);	// totally the payload length is 8 bytes
        tx_exchange(fi, (char *)(fi->g.els_buffer[fi->g.e_i]), 8,
                 r_ctl, type, d_id, my_mtu, int_required, ox_id, SW_RJT);
        fi->g.e_i++;
        if (fi->g.e_i == MAX_PENDING_FRAMES/2)
               fi->g.e_i = 0;
        return;
}

static void tx_bf_rcf(u_int cmd, struct fc_info *fi, u_int d_id)
{
int int_required = 0;
u_int r_ctl, type;
u_int my_mtu = fi->g.my_mtu;
u_int buf;
u_int ox_id;
        
  fi->g.my_id = FABRIC_CONTROLLER;
  fi->g.type_of_frame = FC_SW;

        r_ctl = RCTL_SW;
        type = TYPE_SW | FIRST_SEQUENCE | END_SEQUENCE | SEQUENCE_INITIATIVE;
        ox_id = fi->g.ox_id;

      	buf = htonl(cmd);        // comand code

        memcpy(fi->g.els_buffer[fi->g.e_i], &buf, 4);    // totally the payload length is 4 bytes
        tx_exchange(fi, (char *)(fi->g.els_buffer[fi->g.e_i]), 4,
                 r_ctl, type, d_id, my_mtu, int_required, ox_id, cmd);
        fi->g.e_i++;
        if (fi->g.e_i == MAX_PENDING_FRAMES/2)
                fi->g.e_i = 0;
        return;
}


static void tx_hello(struct fc_info *fi, u_int recv_id)
{
int int_required = 0;
u_int r_ctl, type, ox_id;
u_int my_mtu = fi->g.my_mtu;
HELLO   hello;
        
 fi->g.my_id = FABRIC_CONTROLLER;
 
// tx Hello only if I have already got my own Domain ID       
if(fcsw->sm.got_domainID == 1) {
        r_ctl = RCTL_SW;
        type = TYPE_SW | FIRST_SEQUENCE | END_SEQUENCE | SEQUENCE_INITIATIVE;
        hello.cmnd_code  = htonl(SW_HELLO);
        ox_id = fi->g.ox_id;
        fi->g.type_of_frame = FC_SW;
        
	hello.version = htonl(0x00000002);
	hello.src_id = htonl(fi->g.my_domainID);
 	hello.auth[0] = 0;
	hello.auth[1] = 0;

	hello.options = 0;
	hello.hello_intv = htonl(HELLO_INTV);
	hello.dead_intv  = htonl(DEAD_INTV);
	hello.recv_id = htonl(recv_id);
	hello.port_id = htonl(fi->g.card_order);

        memcpy(fi->g.els_buffer[fi->g.e_i], &hello, sizeof(HELLO));
        tx_exchange(fi, (char *)(fi->g.els_buffer[fi->g.e_i]),sizeof(HELLO),
                 r_ctl, type, 0xFFFFFD, my_mtu, int_required, ox_id, SW_HELLO);
        fi->g.e_i++;
        if (fi->g.e_i == MAX_PENDING_FRAMES/2)
                fi->g.e_i = 0;
 }
        return;
}


static void tx_lsa(struct fc_info *fi, u_int flags, struct domain_id *ids)
{
int int_required = 0;
u_int r_ctl, type;
u_int my_mtu = fi->g.my_mtu;
u_int len = 4*7;  // the LSU/LSA field before the LSR is 7 words
u_int *base;
u_int buf[6];
u_int i = 0;
u_int ox_id;
struct domain_id *walker = ids; 
struct lsr_info  *lsr;
LS ls;   

        
 fi->g.my_id = FABRIC_CONTROLLER;
        
        r_ctl = RCTL_SW;
        type = TYPE_SW | FIRST_SEQUENCE | END_SEQUENCE | SEQUENCE_INITIATIVE;
	ox_id = fi->g.ox_id;
	fi->g.type_of_frame = FC_SW;

	ls.cmnd_code = htonl(SW_LSA);
	ls.version = htonl(0x02000000);
	ls.src_id  = htonl(fcsw->g.my_domainID);
	ls.auth[0] = 0;
	ls.auth[1] = 0;
	ls.flags = htonl(flags);
		// finished setting the FSPF header and flags, the record number will be set later

	base = fi->g.els_buffer[fi->g.e_i] + sizeof(LS)/4;    	// record the start address where the LSR can be added
							  	// added value is the word aligned address 
	while(walker)
	{
		lsr = get_lsr(walker->id);
		if(!lsr) 	
			return;
		buf[0] = (lsr->type << 16) + lsr->age;
		buf[0] = htonl(buf[0]);
		buf[1] = htonl(lsr->options);
		buf[2] = htonl(lsr->id);
		buf[3] = htonl(lsr->adv_id);
		buf[4] = htonl(lsr->incarnation);
		buf[5] = (lsr->checksum << 16) + lsr->length;
                buf[5] = htonl(buf[5]);
		memcpy(base, buf, 4*6); 	// copy the LSR header buffer into the ELS temp buff: 6 words
   		base = base + 6;	// increase the base address
		len  = len + 6*4;	// increase the pay_load length by 16 bytes 
		i++;
		walker = walker->next;		
	}

	ls.num = htonl(i);	// records number

        memcpy(fi->g.els_buffer[fi->g.e_i], &ls, sizeof(LS));
        tx_exchange(fi, (char *)(fi->g.els_buffer[fi->g.e_i]), len,
                 r_ctl, type, 0xFFFFFD, my_mtu, int_required, ox_id, SW_LSA);
        fi->g.e_i++;
        if (fi->g.e_i == MAX_PENDING_FRAMES/2)
                fi->g.e_i = 0;
        return;
}


static void try_lsu(struct fc_info *fi, u_int flags)
{
int int_required = 0;
u_int r_ctl, type;
u_int my_mtu = fi->g.my_mtu;
u_int len = 4*7;  // the LSU/LSA field before the LSR is 7 words
u_int link_len = 0;
u_int *base, *link_base;
u_int buf[7], link_buf[4];
u_int i = 0, link_i = 0;
u_int ox_id;
struct lsr_info *walker = fcsw->rt.lsr_list; 
struct link_info *link;
LS ls;   

        
 fi->g.my_id = FABRIC_CONTROLLER;
        
        r_ctl = RCTL_SW;
        type = TYPE_SW | FIRST_SEQUENCE | END_SEQUENCE | SEQUENCE_INITIATIVE;
	ox_id = fi->g.ox_id;
	fi->g.type_of_frame = FC_SW;

	ls.cmnd_code = htonl(SW_LSU);
	ls.version = htonl(0x02000000);
	ls.src_id  = htonl(fcsw->g.my_domainID);
	ls.auth[0] = 0;
	ls.auth[1] = 0;
	ls.flags = htonl(flags);
		// finished setting the FSPF header and flags, the record number will be set later

	base = fi->g.els_buffer[fi->g.e_i] + sizeof(LS)/4;    	// record the start address where the LSR can be added
							  	// added value is the word aligned address 
	while(walker)
	{
	    if(walker->is_new) {
		buf[0] = (walker->type << 16) + walker->age;
		buf[0] = htonl(buf[0]);
		buf[1] = htonl(walker->options);
		buf[2] = htonl(walker->id);
		buf[3] = htonl(walker->adv_id);
		buf[4] = htonl(walker->incarnation);
			// the buf[5] and buf[6] will be filled by the link data numbers later
		link = walker->link_list;
		link_base = base + 7;  	// the LSR header and the Link Descriptor number occupies 7 words
		link_len = 0;
		while(link) {
			link_buf[0] = htonl(link->neighbor);
			link_buf[1] = htonl(link->outPort);
			link_buf[2] = htonl(link->neighborPort);
			link_buf[3] = (link->type << 16) + link->cost;
			link_buf[3] = htonl(link_buf[3]);
			memcpy(link_base, link_buf, 4*4);
 			link_base = link_base + 4;
			link_len  = link_len  + 4*4;
			link_i++; 		
			link = link->next;
		}
		buf[5] = (walker->checksum << 16) + walker->length;
                buf[5] = htonl(buf[5]);
		buf[6] = htonl(link_i);
		memcpy(base, buf, 4*7); 		// copy the LSR header buffer into the ELS temp buff: 7 words
   		base = base + 7 + link_len/4;		// increase the base address
		len  = len + 7*4 + link_len;		// increase the pay_load length 
		i++;
		walker->is_new = 0;  	// indicate that this record is already sent
	    }
		walker = walker->next;		
	}

	ls.num = htonl(i);	// records number

        memcpy(fi->g.els_buffer[fi->g.e_i], &ls, sizeof(LS));
        tx_exchange(fi, (char *)(fi->g.els_buffer[fi->g.e_i]), len,
                 r_ctl, type, 0xFFFFFD, my_mtu, int_required, ox_id, SW_LSU);
        fi->g.e_i++;
        if (fi->g.e_i == MAX_PENDING_FRAMES/2)
                fi->g.e_i = 0;
        return;
}



static void add_to_domain_ids(u_int new_id, u_int sw_name[])
{
struct domain_id *walker = fcsw->domain_ids;
struct domain_id *finder = NULL;
struct domain_id *new_rcd;

	while(walker && walker->id <= new_id) { 
		finder = walker;
	  	walker = walker->next;
	}
	if (finder && finder->id == new_id)  //get same id
  	{	printk("Error: add_to_domain_ids: get the same fabric id: %x\n", new_id);
        	return; 
  	}
	new_rcd = (struct domain_id *)kmalloc(sizeof(struct domain_id), GFP_KERNEL);
	if(new_rcd == NULL) {
		printk("kmalloc failed in add_to_domain_ids() \n");
		return;
	}  else  {            // set or default the value of the struct
        	new_rcd->id   = new_id;
		new_rcd->type = TYPE_DOMAIN_ID;        
		new_rcd->sw_name[0] = sw_name[0];
		new_rcd->sw_name[1] = sw_name[1];
	}

	if(finder) { finder->next = new_rcd;
  		new_rcd->next = walker;
	} else {
 		new_rcd->next = fcsw->domain_ids;
 		fcsw->domain_ids = new_rcd;
	}

} // end of add_to_domain_ids


// show the LSR database
static void show_lsrs(void)
{
struct lsr_info  *p = fcsw->rt.lsr_list;
struct link_info *q;

        printk("================================ Link State Record Database =================================\n");
        while(p) {  
		q = p->link_list;                                    
                printk("Domain_ID: %x  age: %x   incarnation: %x  size: %d \n", p->id, p->age, p->incarnation, p->length);
		while(q) {
			printk("	Neighbor: %x    outPort: %x    neighborPort: %x   cost: %d \n", 
			q->neighbor, q->outPort, q->neighborPort, q->cost);
			q = q->next;
		}                
		p = p->next;     
                printk("-----------------------------------------------------------------------------\n");
        }                               

        printk("=============================================================================================\n");
	
  	show_cost();
	calc_routing_table();
}

static void show_cost(void)
{
int i,j;
int n = fcsw->rt.node_num;


        printk("================================ Link Cost Matrix =================================\n");
	printk("There are %d nodes in the database:\n", n);
	for(i=0; i<n; i++)
		printk("Node number: %d   Domain ID: %d \n", i, fcsw->rt.nodes[i].id);
	for(i=0; i<n; i++) {
		for(j=0; j<n; j++) {
			printk("%d	", fcsw->rt.AdjMatrix[i][j]);
		}
		printk("\n");                
	}
        printk("====================================================================================\n");

}


static struct lsr_info *get_lsr(u_int id)
{
struct lsr_info *walker = fcsw->rt.lsr_list;
struct lsr_info *finder = NULL;

        while(walker && walker->id <= id) {
                finder = walker;
                walker = walker->next;
        }
        if (finder && finder->id == id)  //get same id
		return finder;
        else {
		printk("get_lsr: Can not the LSR for domain: %X\n", id);
		return NULL;
	}

}


static void add_lsr(u_int age, u_int new_id, u_int incarnation, u_int length, struct link_info *link_data, u_int card_order)
{
// for type 1 LSR, the link start ID is as same as the advertising switch ID: the switch Domain ID
struct lsr_info *walker = fcsw->rt.lsr_list;
struct lsr_info *finder = NULL;
struct lsr_info *new_rcd;
            
	printk(KERN_INFO "Adding new LSR: age: %x, id: %X, incarnation: %X!\n", age, new_id, incarnation);    
        while(walker && walker->id <= new_id) {
                finder = walker;
                walker = walker->next;
        }
        if (finder && finder->id == new_id)  //get same id
        {       printk("add_lsr: same record already existed. %x\n", new_id);
		if(age >= finder->age) {
			printk("add_lsr: this incoming LSR is older than the current record: new id: %x, dropped!\n", new_id);
		} else {
			printk("add_lsr: this incoming LSR is younger than the current record: new id: %x, age changed!\n", new_id);
			finder->age = age;
		} 
		return;
        }
        new_rcd = (struct lsr_info *)kmalloc(sizeof(struct lsr_info), GFP_KERNEL);
        if(new_rcd == NULL) {
                printk("kmalloc failed in add_lsr() \n");
                return;
        }  else  {            // set or default the value of the struct
		new_rcd->type = 0x0100;
		new_rcd->age  = age;
		new_rcd->options = 0;
		new_rcd->id = new_id;
		new_rcd->adv_id = new_id;
		new_rcd->incarnation = incarnation;
		new_rcd->checksum = 0;
		new_rcd->length = length;
	
		new_rcd->is_new = 1;
		new_rcd->link_list = link_data;
        }

        if(finder) { finder->next = new_rcd;
                new_rcd->next = walker;
        } else {
                new_rcd->next = fcsw->rt.lsr_list;  
                fcsw->rt.lsr_list = new_rcd;
        }

	add_node(new_id);
	add_link_data(get_lsr(fcsw->g.my_domainID), new_id, card_order, LINK_COST);
	show_lsrs();
}

static void add_link_data(struct lsr_info *lsr, u_int new_id, u_int outPort, u_int cost)
{
struct link_info *finder = NULL, *walker = lsr->link_list;
struct link_info *new_rcd;

	if(lsr->id == new_id)   // if the new_id is myself, for sure it is not my neighbor
		return;
	if(!lsr) {
		printk("add_link_data: lsr null pointer!\n");
		return;
	}
	
        while(walker && walker->neighbor <= new_id) {
                finder = walker;
                walker = walker->next;
        }
        if (finder && finder->neighbor == new_id)  //get same id
        {       printk("add_link_data: same link data already existed. %x\n", new_id);
                return;
        }
	printk(KERN_INFO "Adding new link data: neighbor: %x, outPort: %X\n", new_id, outPort); 
        new_rcd = (struct link_info *)kmalloc(sizeof(struct link_info), GFP_KERNEL);
        if(new_rcd == NULL) {
                printk("kmalloc failed in add_link_data() \n");
                return;
        }  else  {            // set or default the value of the struct
                new_rcd->neighbor = new_id;
                new_rcd->outPort = outPort;
                new_rcd->neighborPort = fc[outPort]->rt.adj_port; // the Port ID was got from the Hello frame
                new_rcd->type  = 0x0100;
                new_rcd->cost = cost;
        }
        
	lsr->length += 4*4; 	// adjust the LSR length
        if(finder) { finder->next = new_rcd;
                new_rcd->next = walker;
        } else {
                new_rcd->next = lsr->link_list;
                lsr->link_list = new_rcd;
        }
	link_cost(lsr->id, new_id, cost);
 
}


// for routing
static void add_to_vlist(int order)
{
struct vertex *newV = (struct vertex *)kmalloc(sizeof(struct vertex), GFP_KERNEL);
struct vertex *lastV = fcsw->rt.vhead;

        if (!newV) {
                printk(KERN_INFO "Out of memory.\n");
                return;
    	}
        newV->order = order;
        newV->next = NULL;
        if(!lastV) fcsw->rt.vhead = newV;
        else {
                        while(lastV->next)
                                lastV = lastV->next;
                        lastV->next = newV;   // put the new item at the end of the list
        }
}


static void remove_from_vlist(int order)
{
struct vertex *finder, *walker, *temp;
int found = 0;

        finder = NULL;
        walker = fcsw->rt.vhead;
        while(walker){
                if(order == walker->order) {
                        found = 1;
                        if(finder == NULL)
                                fcsw->rt.vhead = walker->next;
                        else {  
                                temp = walker;
                                finder->next = walker->next;
                                kfree(temp);
                        }
                        break;
                }
                finder = walker;
                walker = walker->next;
        } // end of while
        if(found == 0){
                printk(KERN_INFO "The order is not in the vertex list: %d.", order);
                return;
        }
}

static void print_path (struct NODE nodes[], int chNode)
{               
        if (nodes[chNode].iPrev != NONE)
        {
        print_path(nodes, nodes[chNode].iPrev);
        }
        printk(KERN_INFO " %X", nodes[chNode].id);
}
                                
static int get_first_hop(int k)  
{
int p = fcsw->rt.nodes[k].iPrev;
                
        if(p != NONE) {
                if(fcsw->rt.nodes[p].iPrev == NONE)
                        return fcsw->rt.nodes[k].id;
                else return get_first_hop(p);
        }  else
           return fcsw->rt.nodes[k].id; // this is the sourse node, the next hop is itself (assumely)
}       


static int add_node(int id)            
{
int i;

        for(i=0; i<fcsw->rt.node_num; i++)
                if(id == fcsw->rt.nodes[i].id) {
                        printk(KERN_INFO "This id was already in: %X.\n", id);
                        return i;
                }
        // if there is no such id in nodes[]
 
        fcsw->rt.nodes[fcsw->rt.node_num].id = id;
           
        fcsw->rt.node_num++;
	
	printk("New id: %x added as number: %d\n", id, fcsw->rt.node_num - 1);
        return fcsw->rt.node_num - 1;  // return the order of this id in the nodes array
}

static int get_node(int id)
{
int i;
                        
        for(i=0; i<fcsw->rt.node_num; i++)
                if(id == fcsw->rt.nodes[i].id)
                        return i;
        // otherwise
        printk(KERN_INFO "Can not find this id: %X\n", id);
        return 0;
}
           
static u_int get_src_port(u_int id)
{
u_int node_order;
	
	node_order = get_node(id);
	return fcsw->rt.nodes[node_order].src_port;

}

     
static void link_cost(int from, int to, int cost)
{
   printk("From: %d to: %d Link cost is %d==================================\n", from, to, cost);
   fcsw->rt.AdjMatrix[get_node(from)][get_node(to)] = cost;
}


static void calc_routing_table()
{
int i, j, start;
struct vertex *walker;
int order, minCost, minOrder;
int preNode, oldCost, newCost;
int node_num = fcsw->rt.node_num;        
struct NODE *nodes = fcsw->rt.nodes;

	start = 0; 	// start will always be the first element in the nodes array 
	for(i=0; i<node_num; i++) {    //for testing, now has 6 nodes
                if(i!= start)
                        add_to_vlist(i);
        }  // initialize the un-processed vertices list
        
 
        for(i=0; i<node_num; i++) {
                nodes[i].iDist = NONE;
                nodes[i].iPrev = NONE;
        }

        nodes[start].iDist = 0;
        nodes[start].iPrev = NONE;

        for(i=0; i<node_num; i++) {
                if(fcsw->rt.AdjMatrix[start][i] != NONE) {
                                nodes[i].iDist = fcsw->rt.AdjMatrix[start][i];
                                nodes[i].iPrev = start;
                }
        }  // initially, the neighboring vertex cost is the link cost

	while (fcsw->rt.vhead != NULL)  // while the vlist is not empty
	{
		minCost = NONE;
		minOrder = 0;
		walker = fcsw->rt.vhead;
		while(walker)
		{
			order = walker->order;
			if(nodes[order].iDist < minCost) {
				minCost = nodes[order].iDist;
				minOrder = order;
			}
			walker = walker->next;
		}
		remove_from_vlist(minOrder);
	
		preNode = minOrder;
		walker = fcsw->rt.vhead;
		while(walker) // update least-cost paths
		{	
			order = walker->order;
			oldCost = nodes[order].iDist;
			newCost = minCost + fcsw->rt.AdjMatrix[minOrder][order];
			if(oldCost > newCost) {
				nodes[order].iDist = newCost;
				nodes[order].iPrev = preNode;
			}
			walker = walker->next;
		}
	
	}
	
	printk("=================================================\n");
	printk("All pathes from: %d\n", nodes[0].id);
    	for(i = 0; i<node_num; i++) {
		print_path(nodes, i);
		printk(KERN_INFO "\n");
	}
	printk("-------------------------------------------------\n");
    	for(i = 0; i<node_num; i++) {
		nodes[i].first_hop = get_first_hop(i);
		printk("To: %d, first hop: %d \n", nodes[i].id, nodes[i].first_hop);
	}
	printk("-------------------------------------------------\n");
        for(i = 0; i<node_num; i++) {
		for(j=0; j<MAX_FC_CARDS; j++) {
			if(fc[j]->rt.adj_domain == nodes[i].id)
				nodes[i].src_port = j;
		}
                printk("Domain: %d, source port: %d \n", nodes[i].id, nodes[i].src_port);
        } 
	printk("=================================================\n");
}

/* This is a ospf calculation sample. The description is on page 394 "High-Speed Networks" by William Stallings */
static void ospf_sample()
{
/* the result should be:
	A: A
	B: A B
	C: A D E C
	D: A D
	E: A D E
	F: A D E F
*/ 

	/* first, add all possible nodes on the list */
	add_node(0x100a);
	add_node(0x100b);
	add_node(0x100c);
	add_node(0x100d);
	add_node(0x100e);
	add_node(0x100f);
	 
	/* then set the link cost */
	link_cost(0x100a, 0x100b, 2);
	link_cost(0x100a, 0x100c, 5);
	link_cost(0x100a, 0x100d, 1);
	
	link_cost(0x100b, 0x100a, 3);
	link_cost(0x100b, 0x100c, 3);
	link_cost(0x100b, 0x100d, 2);

	link_cost(0x100c, 0x100a, 8);
	link_cost(0x100c, 0x100b, 6);
	link_cost(0x100c, 0x100d, 3);
	link_cost(0x100c, 0x100e, 1);
	link_cost(0x100c, 0x100f, 5);

	link_cost(0x100d, 0x100a, 7);
	link_cost(0x100d, 0x100b, 2);
	link_cost(0x100d, 0x100c, 3);
	link_cost(0x100d, 0x100e, 1);


	link_cost(0x100e, 0x100c, 1);
	link_cost(0x100e, 0x100d, 1);
	link_cost(0x100e, 0x100f, 2);

	link_cost(0x100f, 0x100c, 8);
	link_cost(0x100f, 0x100e, 4);

	
	calc_routing_table();
}





static void tx_logi_acc(struct fc_info *fi, u_int logi, u_int d_id, u_short received_ox_id)
{
int int_required = 0;
u_int r_ctl = RCTL_ELS_SCTL;
u_int type  = TYPE_ELS | EXCHANGE_RESPONDER | LAST_SEQUENCE;
u_int my_mtu = fi->g.my_mtu;
	ENTER("tx_logi_acc");
	fill_login_frame(fi, logi);	
	fi->g.type_of_frame = FC_ELS;
	memcpy(fi->g.els_buffer[fi->g.e_i], &fi->g.login, sizeof(LOGIN));
	tx_exchange(fi, (char *)(fi->g.els_buffer[fi->g.e_i]),sizeof(LOGIN), 
                 r_ctl, type, d_id, my_mtu, int_required, received_ox_id, logi);
	//tx_exchange(fi, (char *)(&fi->g.login),sizeof(LOGIN), r_ctl, type, d_id, my_mtu, int_required, received_ox_id, logi);
	fi->g.e_i++;
	if (fi->g.e_i == MAX_PENDING_FRAMES/2)
		fi->g.e_i = 0;
	LEAVE("tx_logi_acc");
	return;
}


//xyz
static void tx_fcs_resp(struct fc_info *fi, u_int d_id, u_short received_ox_id, u_char reason, u_char expln)
{
int int_required = 1;                           
u_int r_ctl = RCTL_RFC_SCTL;
u_int type  = TYPE_FC_SERVICES | EXCHANGE_RESPONDER | LAST_SEQUENCE;
u_int my_mtu = fi->g.my_mtu;            
CT_HDR ct_hdr;

        ENTER("tx_fcs_resp");
        /* Fill up CT_Header */
        ct_hdr.rev_in_id = htonl(FC_CT_REV);

        ct_hdr.fs_type = DIRECTORY_SERVER_APP;
        ct_hdr.fs_subtype = NAME_SERVICE;
        ct_hdr.options = 0;
        ct_hdr.resv1 = 0;

	if(reason == 0)  // reason code = 0 means ACC should be sent
        	{ ct_hdr.cmnd_resp_code = htons(FCS_ACC);
		  ct_hdr.reason_code = 0;
		  ct_hdr.expln_code = 0;
		}
        else    {
	          ct_hdr.cmnd_resp_code = htons(FCS_REJECT);
		  ct_hdr.reason_code = reason;
                  ct_hdr.expln_code = expln;
	        }
	ct_hdr.max_res_size = 0;

        ct_hdr.resv2 = 0;
        ct_hdr.vendor_unique = 0;

        fi->g.type_of_frame = FC_FCS;

 fi->g.my_id = DIRECTORY_SERVER;



        memcpy(fi->g.els_buffer[fi->g.e_i], &ct_hdr, sizeof(CT_HDR));  
        tx_exchange(fi, (char *)(fi->g.els_buffer[fi->g.e_i]), sizeof(CT_HDR), r_ctl, type, d_id, my_mtu, 
                int_required, received_ox_id, 0x01000000);
        fi->g.e_i++;
        if (fi->g.e_i == MAX_PENDING_FRAMES/2)
                fi->g.e_i = 0;
        LEAVE("tx_fcs_resp");
        return;
}


static void tx_ls_rjt(struct fc_info *fi, u_int d_id, u_short received_ox_id, u_short reason_code, u_short expln_code)
{
int int_required = 0;
u_int r_ctl = RCTL_ELS_SCTL;
u_int type  = TYPE_ELS | EXCHANGE_RESPONDER | LAST_SEQUENCE;
int size = sizeof(LS_RJT);
u_int my_mtu = fi->g.my_mtu;
	ENTER("tx_ls_rjt");
	fi->g.ls_rjt.cmnd_code = htonl(ELS_LS_RJT);
	/* reason -> command not supported */
	fi->g.ls_rjt.reason_code = htonl((reason_code << 16) | expln_code); 
	fi->g.type_of_frame = FC_ELS;
	memcpy(fi->g.els_buffer[fi->g.e_i], &fi->g.ls_rjt, size);
	tx_exchange(fi, (char *)(fi->g.els_buffer[fi->g.e_i]),size, r_ctl, type, d_id, my_mtu, int_required, received_ox_id, ELS_LS_RJT);
	fi->g.e_i++;
	if (fi->g.e_i == MAX_PENDING_FRAMES/2)
		fi->g.e_i = 0;
	LEAVE("tx_ls_rjt");
}

static u_int plogi_ok(struct fc_info *fi, u_int *buff_addr, int size)
{
int ret_code = 0;
u_short mtu = ntohl(*(buff_addr + 10)) & 0x00000FFF;
u_short class3 = ntohl(*(buff_addr + 25)) >> 16;
u_short class3_conc_seq = ntohl(*(buff_addr + 27)) >> 16;
u_short open_seq = ntohl(*(buff_addr + 28)) >> 16;
	DEBUG("mtu = %x class3 = %x conc_seq = %x open_seq = %x", mtu, class3, class3_conc_seq, open_seq);	
	size -= HEADER_LEN;
	if (!(class3 & 0x8000)) {
		DEBUG("Received PLOGI with class3 = %x", class3);
		ret_code = (LOGICAL_ERR << 16) | NO_EXPLN;
		return ret_code;
	}
	if (mtu < 256) {
		DEBUG("Received PLOGI with MTU set to %x", mtu);
		ret_code = (LOGICAL_ERR << 16) | RECV_FIELD_SIZE;
		return ret_code;
	}
	if (size != PLOGI_LEN) {	
		DEBUG("Received PLOGI of size %x", size);
		ret_code = (LOGICAL_ERR << 16) | INV_PAYLOAD_LEN;
		return ret_code;
	}
	if (class3_conc_seq == 0) {	
		DEBUG("Received PLOGI with conc_seq == 0");
		ret_code = (LOGICAL_ERR << 16) | CONC_SEQ;
		return ret_code;
	}
	if (open_seq == 0) {	
		DEBUG("Received PLOGI with open_seq == 0");
		ret_code = (LOGICAL_ERR << 16) | NO_EXPLN;
		return ret_code;
	}

	/* Could potentially check for more fields, but might end up
	   not talking to most of the devices. ;-) */
	/* Things that could get checked are:
	   common_features = 0x8800
	   total_concurrent_seq = at least 1
	*/
	return ret_code;
}

static void tx_acc(struct fc_info *fi, u_int d_id, u_short received_ox_id)
{
int int_required = 0;
u_int r_ctl = RCTL_ELS_SCTL;
u_int type  = TYPE_ELS | EXCHANGE_RESPONDER | LAST_SEQUENCE;
int size = sizeof(ACC);
u_int my_mtu = fi->g.my_mtu;
	ENTER("tx_acc");
	fi->g.acc.cmnd_code = htonl(ELS_ACC);
	fi->g.type_of_frame = FC_ELS;
	memcpy(fi->g.els_buffer[fi->g.e_i], &fi->g.acc, size);
	tx_exchange(fi, (char *)(fi->g.els_buffer[fi->g.e_i]),size, r_ctl, type, d_id, my_mtu, int_required, received_ox_id, ELS_ACC);
	//tx_exchange(fi, (char *)(&fi->g.acc),size, r_ctl, type, d_id, my_mtu, int_required, received_ox_id, ELS_ACC);
	fi->g.e_i++;
	if (fi->g.e_i == MAX_PENDING_FRAMES/2)
		fi->g.e_i = 0;
	LEAVE("tx_acc");
}



static void add_to_ox_id_list(struct fc_info *fi, u_int transaction_id, u_int cmnd_code, u_char from_card)
{
struct ox_id_els_map *p, *q = fi->ox_id_list, *r = NULL;
int size = sizeof(struct ox_id_els_map);

// if transaction_id < 0, it will become FF because it is unsigned integer
	
	while (q != NULL) {
		r = q;
		q = q->next;
	}
	p = (struct ox_id_els_map *)kmalloc(size, GFP_DMA | GFP_ATOMIC);
	if (p == NULL) {
		T_MSG("kmalloc failed in add_to_ox_id_list()");
		return;
	}
	p->ox_id = transaction_id;
	p->els = cmnd_code;
	p->from_card = from_card;
	p->next = NULL;
	if (fi->ox_id_list == NULL)
		fi->ox_id_list = p;
	else
		r->next = p;
	return;
}

static u_int remove_from_ox_id_list(struct fc_info *fi, u_short received_ox_id)
{
struct ox_id_els_map *p = fi->ox_id_list, *q = fi->ox_id_list;
u_int els_type;
        while (q != NULL) {
                if (q->ox_id == received_ox_id) {
                
                        if (q == fi->ox_id_list)
                                fi->ox_id_list = fi->ox_id_list->next;
                        else
                                p->next = q->next;

                        els_type = q->els;
             		fi->g.from_card = q->from_card;
		        kfree(q);  
                        return els_type;
                }
                p = q;
                q = q->next;
        }
        if (q == NULL)
                T_MSG("Could not find ox_id %x in ox_id_els_map", received_ox_id);
        return 0;
}

static void increase_oxid(struct fc_info *fi)
{
    fi->g.ox_id++;
    if (fi->g.ox_id == 0xFFFF)
       fi->g.ox_id = NOT_SCSI_XID;
}


static void build_tachyon_header(struct fc_info *fi, u_int my_id, u_int r_ctl, u_int d_id, u_int type, u_char seq_id, u_char df_ctl, u_short ox_id, u_short rx_id, char *data)
{
u_char alpa = d_id & 0x0000FF;
u_int dest_ddaa = d_id &0xFFFF00;
u_int tmp_type;

	ENTER("build_tachyon_header");
	DPRINTK("d_id = %x, my_ddaa = %x", d_id, fi->g.my_ddaa);
	/* Does it have to go to/thru a Fabric? */
	if ((dest_ddaa != 0) && ((d_id == F_PORT) || (fi->g.fabric_present && (dest_ddaa != fi->g.my_ddaa))))
		alpa = 0x00;
	fi->g.tach_header.resv = 0x00000000;

       tmp_type  = TYPE_FCP | SEQUENCE_INITIATIVE | FIRST_SEQUENCE;
    
          if(fi->g.i_am_E_Port == TRUE)
                fi->g.tach_header.sof_and_eof = SOFF | EOFN;
           else fi->g.tach_header.sof_and_eof = SOFI3 | EOFN;  

	fi->g.tach_header.dest_alpa = alpa;
	/* Set LCr properly to have enuff credit */
	if (alpa == REPLICATE)
		fi->g.tach_header.lcr_and_time_stamp = htons(0xC00);/* LCr=3 */
	else
		fi->g.tach_header.lcr_and_time_stamp = 0;

	fi->g.tach_header.r_ctl_and_d_id = htonl(r_ctl | d_id);
	fi->g.tach_header.vc_id_and_s_id = htonl(my_id);
	fi->g.tach_header.type_and_f_cntl = htonl(type);
	fi->g.tach_header.seq_id = seq_id;
	fi->g.tach_header.df_cntl = df_ctl;
	fi->g.tach_header.seq_cnt = 0;
	fi->g.tach_header.ox_id = htons(ox_id);
	fi->g.tach_header.rx_id = htons(rx_id);
	switch(r_ctl) {
		case F_RJT:
			fi->g.tach_header.ro = htonl(0x01020000);
			break;
                case P_BSY:
                        fi->g.tach_header.ro = htonl(0x01030000);
                        break;
                case P_RJT:
                        fi->g.tach_header.ro = htonl(0x01020000);
                        break;
 		default:
			fi->g.tach_header.ro = 0;
	}
	if (data) {
		/* We use the Seq_Count to keep track of IP frames in the
		 * OCI_interrupt handler. Initial Seq_Count of IP frames is 1.
		 */
		fi->g.tach_header.seq_cnt = htons(0x1);
		fi->g.tach_header.nw_header.d_naa = htons(0x1000);
		fi->g.tach_header.nw_header.s_naa = htons(0x1000);
		memcpy(&(fi->g.tach_header.nw_header.dest_high), data, 2);
		memcpy(&(fi->g.tach_header.nw_header.dest_low), data + 2, 4);
		memcpy(&(fi->g.tach_header.nw_header.source_high), data + 6, 2);
		memcpy(&(fi->g.tach_header.nw_header.source_low), data + 8, 4);
	}
	LEAVE("build_tachyon_header");
}

static void build_EDB(struct fc_info *fi, char *data, u_short flags, u_short len)
{
	fi->g.edb.buf_addr = ntohl((u_int)virt_to_bus(data));
	fi->g.edb.ehf = ntohs(flags);
	if (len % 4)
		len += (4 - (len % 4));
	fi->g.edb.buf_len = ntohs(len);
}

static void build_ODB(struct fc_info *fi, u_char seq_id, u_int d_id, u_int len, u_int cntl, u_short mtu, u_short ox_id, u_short rx_id, int NW_header, int int_required, u_int frame_class)
{
// cntl is not used;


	fi->g.odb.seq_d_id = htonl(seq_id << 24 | d_id);
	fi->g.odb.tot_len = len;
	if (NW_header)
		fi->g.odb.tot_len += NW_HEADER_LEN;
	if (fi->g.odb.tot_len % 4)
		fi->g.odb.tot_len += (4 - (fi->g.odb.tot_len % 4));
	fi->g.odb.tot_len = htonl(fi->g.odb.tot_len);

        fi->g.odb.cntl = htons(ODB_CLASS_THREE | ODB_EE_CR_2 | ODB_NO_INT | ODB_NO_COMP );

	/*
        switch(int_required) {
		case NO_COMP_AND_INT:
			fi->g.odb.cntl = htons(ODB_CLASS_THREE | ODB_EE_CR_2 | ODB_NO_INT | ODB_NO_COMP | cntl);
			break;
		case INT_AND_COMP_REQ:
			fi->g.odb.cntl = htons(ODB_CLASS_THREE | ODB_EE_CR_2 | cntl);
			break;
		case NO_INT_COMP_REQ:
			fi->g.odb.cntl = htons(ODB_CLASS_THREE | ODB_EE_CR_2 | ODB_NO_INT | cntl);
			break;
	}
        */

	fi->g.odb.rx_id = htons(rx_id);
	fi->g.odb.cs_enable = 0;
	fi->g.odb.cs_seed = htons(1);

	fi->g.odb.hdr_addr = htonl(virt_to_bus(fi->q.ptr_tachyon_header[fi->q.tachyon_header_indx]));
	fi->g.odb.frame_len = htons(mtu);

	/* helps in tracking transmitted OX_IDs */
	/* track the trans_id in OCI added by xh 05/14/2000 */
	fi->g.odb.trans_id = htonl((frame_class & 0xFFFF0000) | ox_id);
	fi->g.odb.hdr_len = HEADER_LEN;

	fi->g.odb.hdr_len = htons(fi->g.odb.hdr_len);
		
	fi->g.odb.edb_addr = htonl(virt_to_bus(fi->q.ptr_edb[fi->q.edb_buffer_indx]));
}



static void fill_login_frame(struct fc_info *fi, u_int logi)
{
int i;
	fi->g.login.ls_cmnd_code= htonl(logi);
	fi->g.login.fc_ph_version = htons(FC_PH_VERSION);
	if (fi->g.loop_up)
		fi->g.login.buff_to_buff_credit = htons(BB_CREDIT);
	else
	if (fi->g.ptp_up)
		fi->g.login.buff_to_buff_credit = htons(PT2PT_BB_CREDIT);
	if ((logi == ELS_PLOGI) || (logi == ELS_ACC))
		fi->g.login.common_features = htons(PLOGI_C_F);
	else
	if (logi == ELS_FLOGI)
		fi->g.login.common_features = htons(FLOGI_C_F);

       if (fi->g.i_am_fabric == TRUE) // modified by xiangdong huang for Flogi_acc
             fi->g.login.common_features = htons(FLOGI_C_F);





	fi->g.login.recv_data_field_size = htons(FRAME_SIZE);
	fi->g.login.n_port_total_conc_seq = htons(CONCURRENT_SEQ);
	fi->g.login.rel_off_by_info_cat = htons(RO_INFO_CAT);
	fi->g.login.ED_TOV = htonl(E_D_TOV);
	fi->g.login.n_port_name_high = htonl(N_PORT_NAME_HIGH);
	fi->g.login.n_port_name_low = htonl(N_PORT_NAME_LOW);
	fi->g.login.node_name_high = htonl(NODE_NAME_HIGH);
	fi->g.login.node_name_low = htonl(NODE_NAME_LOW);
	
	/* Fill Class 1 parameters */
	fi->g.login.c_of_s[0].service_options = htons(0);
	fi->g.login.c_of_s[0].initiator_ctl = htons(0);
	fi->g.login.c_of_s[0].recipient_ctl = htons(0);
	fi->g.login.c_of_s[0].recv_data_field_size = htons(0);
	fi->g.login.c_of_s[0].concurrent_sequences = htons(0);
	fi->g.login.c_of_s[0].n_port_end_to_end_credit = htons(0);
	fi->g.login.c_of_s[0].open_seq_per_exchange = htons(0);
	fi->g.login.c_of_s[0].resv = htons(0);

	/* Fill Class 2 parameters */
	fi->g.login.c_of_s[1].service_options = htons(0);
	fi->g.login.c_of_s[1].initiator_ctl = htons(0);
	fi->g.login.c_of_s[1].recipient_ctl = htons(0);
	fi->g.login.c_of_s[1].recv_data_field_size = htons(0);
	fi->g.login.c_of_s[1].concurrent_sequences = htons(0);
	fi->g.login.c_of_s[1].n_port_end_to_end_credit = htons(0);
	fi->g.login.c_of_s[1].open_seq_per_exchange = htons(0);
	fi->g.login.c_of_s[1].resv = htons(0);

	/* Fill Class 3 parameters */
        /*  This is wrong
	if (logi == ELS_FLOGI)
		fi->g.login.c_of_s[2].service_options  = htons(SERVICE_VALID | SEQUENCE_DELIVERY);
	else
		fi->g.login.c_of_s[2].service_options  = htons(SERVICE_VALID);
        */ 
        /*
        FLOGI ACC from the FL port is also required to set the sequence delivery bit to 1.
        Learned from the FLA test for Hitachi disk on Oct. 19, 1999 
         */

        fi->g.login.c_of_s[2].service_options  = htons(SERVICE_VALID | SEQUENCE_DELIVERY);

	
 	fi->g.login.c_of_s[2].initiator_ctl = htons(0);
	fi->g.login.c_of_s[2].recipient_ctl = htons(0);
	fi->g.login.c_of_s[2].recv_data_field_size = htons(FRAME_SIZE);


        fi->g.login.c_of_s[2].concurrent_sequences = htons(C3_CONCURRENT_SEQ);
	fi->g.login.c_of_s[2].n_port_end_to_end_credit = htons(0);
	fi->g.login.c_of_s[2].open_seq_per_exchange = htons(C3_OPEN_SEQ);
	fi->g.login.c_of_s[2].resv = htons(0);
	
	for(i = 0; i < 4; i++) {
		fi->g.login.resv[i] = 0;
		fi->g.login.vendor_version_level[i] = 0;
	}
}


/* clear the Interrupt Latch on the (i)chip, so that you can receive 
 * Interrupts from Tachyon in future 
 */
static void reset_latch(struct fc_info *fi)
{
	writel(readl(fi->i_r.ptr_ichip_hw_status_reg) | ICHIP_HSR_INT_LATCH, fi->i_r.ptr_ichip_hw_status_reg);
}

static void update_OCQ_indx(struct fc_info *fi)
{
	fi->q.ocq_prod_indx++;
	if (fi->q.ocq_prod_indx == OCQ_LEN)
		fi->q.ocq_prod_indx = 0;
	writel(fi->q.ocq_prod_indx, fi->t_r.ptr_ocq_prod_indx_reg);
}

static void update_IMQ_indx(struct fc_info *fi, int count)
{
	fi->q.imq_cons_indx += count;
	if (fi->q.imq_cons_indx >= IMQ_LEN)
		fi->q.imq_cons_indx -= IMQ_LEN;
	writel(fi->q.imq_cons_indx, fi->t_r.ptr_imq_cons_indx_reg);
}

static void update_SFSBQ_indx(struct fc_info *fi)
{
	fi->q.sfsbq_prod_indx++;
	if (fi->q.sfsbq_prod_indx == SFSBQ_LEN)
		fi->q.sfsbq_prod_indx = 0;
	writel(fi->q.sfsbq_prod_indx, fi->t_r.ptr_sfsbq_prod_reg);
}

static void update_MFSBQ_indx(struct fc_info *fi, int count)
{
	fi->q.mfsbq_prod_indx += count;
	if (fi->q.mfsbq_prod_indx >= MFSBQ_LEN)
		fi->q.mfsbq_prod_indx -= MFSBQ_LEN;
	writel(fi->q.mfsbq_prod_indx, fi->t_r.ptr_mfsbq_prod_reg);
}


static void update_tachyon_header_indx(struct fc_info *fi)
{
	fi->q.tachyon_header_indx++;
	if (fi->q.tachyon_header_indx == NO_OF_TACH_HEADERS)
		fi->q.tachyon_header_indx = 0;
}

static void update_EDB_indx(struct fc_info *fi)
{
	fi->q.edb_buffer_indx++;
	if (fi->q.edb_buffer_indx == EDB_LEN)
		fi->q.edb_buffer_indx = 0;
}

static int iph5526_open(struct device *dev)
{
	/*do we need to do this here? */
	/*tachyon_init(fi);*/
	dev->tbusy = 0;
	dev->interrupt = 0;
	dev->start = 1;
	MOD_INC_USE_COUNT;
	return 0;
}

static int iph5526_close(struct device *dev)
{
	dev->tbusy = 1;
	dev->start = 0;
	MOD_DEC_USE_COUNT;
	return 0;
}

//hxd
static int iph5526_ioctl(struct device *dev, struct ifreq *rq, int cmd)
{
struct fc_info *fi = (struct fc_info*)dev->priv;
u_int *data = (u_int *)rq->ifr_data;
u_int r_ctl, type;

	DEBUG("entering iph5526_ioctl");
	switch(cmd) {
		case SIOCDEVPRIVATE + 1: 
			DEBUG("Initializing Loop as FL_Port...");
			take_tachyon_offline(fi);
			/* write AL_TIME & E_D_TOV into the registers */
			writel(TOV_VALUES, fi->t_r.ptr_fm_tov_reg);
			/* tell Tachyon to Intialize as an FL_Port */
			writel(FM_FABRIC_ADDRESS | FM_INIT_FABRIC | FM_FABRIC_LOGIN | FM_RESPOND_FABRIC, fi->t_r.ptr_fm_config_reg);

			fi->g.i_am_fabric = TRUE;
			writel(FMCNTL_CMD_INITIALIZE, fi->t_r.ptr_fm_control_reg);
			break;
		case SIOCDEVPRIVATE + 2: 
			DEBUG("Initializing Loop as F_Port...");
			take_tachyon_offline(fi);
			/* write R_T_TOV & E_D_TOV into the registers */
			writel(PTP_TOV_VALUES, fi->t_r.ptr_fm_tov_reg);
			writel(FM_PT2PT_BB_CR | FM_NPORT_CONNECTION, fi->t_r.ptr_fm_config_reg);
			fi->g.n_port_try = TRUE;
			fi->g.i_am_fabric = TRUE;
			writel(FMCNTL_CMD_INITIALIZE, fi->t_r.ptr_fm_control_reg);
			break;
                case SIOCDEVPRIVATE + 3:
                        DEBUG("Initializing as E_Port...");
                        take_tachyon_offline(fi); 
                        /* write R_T_TOV & E_D_TOV into the registers */
                        writel(PTP_TOV_VALUES, fi->t_r.ptr_fm_tov_reg);
                        writel(FM_PT2PT_BB_CR | FM_NPORT_CONNECTION | FM_FABRIC_ADDRESS | FM_INIT_FABRIC | FM_FABRIC_LOGIN | 
				FM_RESPOND_FABRIC, fi->t_r.ptr_fm_config_reg);
                        // writel(FM_PT2PT_BB_CR | FM_NPORT_CONNECTION, fi->t_r.ptr_fm_config_reg);
                        fi->g.n_port_try = TRUE;
                        fi->g.i_am_fabric = TRUE;
			fi->g.i_am_E_Port = TRUE;
                        fi->sm.psw_selected = 0;        // principal switch not selected yet
                        fi->sm.elp_done = 0;
                        fi->sm.elp_timer_set = 0;  
                        fi->sm.efp_timer_set = 0;
                        fi->sm.cur_domainID  = FIRST_DOMAINID;
                        writel(FMCNTL_CMD_INITIALIZE, fi->t_r.ptr_fm_control_reg);
                        break;

		case SIOCDEVPRIVATE + 4:
                        DEBUG("Initializing Port as E_Port for E_Port testing ......");
			take_tachyon_offline(fi);
			init_fabric_paras(fi, 1);
			fi->sm.testing = data[0];	// set the testing flag	
                        /* write R_T_TOV & E_D_TOV into the registers */
                        writel(PTP_TOV_VALUES, fi->t_r.ptr_fm_tov_reg);
			writel(FM_PT2PT_BB_CR | FM_NPORT_CONNECTION, fi->t_r.ptr_fm_config_reg);
			writel(FMCNTL_CMD_INITIALIZE, fi->t_r.ptr_fm_control_reg);
                        break;

		case SIOCDEVPRIVATE + 5:
			T_MSG("Txing SW_ILS from %x to %x", data[ data[0]+1 ], data[ data[0]+2 ] );
			fi->g.my_id = data[ data[0]+1 ];
                        r_ctl = RCTL_SW;
			type = TYPE_SW | FIRST_SEQUENCE | END_SEQUENCE | SEQUENCE_INITIATIVE;
                        tx_generic(fi, data, r_ctl, type);
                        break;

		case SIOCDEVPRIVATE + 8: 
                        DEBUG("Txing  FCS command from %x to %x", data[ data[0]+1 ], data[ data[0]+2 ]);
                        fi->g.my_id = data[ data[0]+1 ];
                        r_ctl = RCTL_RFC_UCTL;
                        type  = TYPE_FC_SERVICES | SEQUENCE_INITIATIVE | FIRST_SEQUENCE;
 
                        tx_generic(fi, data, r_ctl, type);
			break;

                case SIOCDEVPRIVATE + 9:
                        DEBUG("Txing  ELS from %x to %x", data[ data[0]+1 ], data[ data[0]+2 ]);   
                        fi->g.my_id = data[ data[0]+1 ];
			r_ctl = RCTL_ELS_UCTL;
			type  = TYPE_ELS | SEQUENCE_INITIATIVE | FIRST_SEQUENCE;
                        tx_generic(fi, data, r_ctl, type);
                        break;

                case SIOCDEVPRIVATE + 11:
                        DEBUG("Txing  FCP command from %x to %x", data[ data[0]+1 ], data[ data[0]+2 ]);
                        fi->g.my_id = data[ data[0]+1 ];
                        r_ctl = RCTL_FCP_CMD;   
                        type  = TYPE_FCP | SEQUENCE_INITIATIVE | FIRST_SEQUENCE;
                        
                        tx_generic(fi, data, r_ctl, type);
                        break;

                  case SIOCDEVPRIVATE + 13:
                        DEBUG("Txing  FCP single frame data from %x to %x", data[ data[0]+1 ], data[ data[0]+2 ]);
                        fi->g.my_id = data[ data[0]+1 ];
                        r_ctl = RCTL_FCP_DATA;
                        type = 0x08880008;  // for easy, don't bother, just for myself testing
                        tx_generic(fi, data, r_ctl, type);
                        break;

                   case SIOCDEVPRIVATE + 14:
                        DEBUG("Txing  very generate data or cmd from %x to %x", data[ data[0]+1 ], data[ data[0]+2 ]);
                        fi->g.my_id = data[ data[0]+1 ];
                        r_ctl = data[ data[0] + 3 ];
                        type  = data[ data[0] + 4 ];
                        tx_generic(fi, data, r_ctl, type);
                        break;

		default:
			T_MSG("Invalid ioctl() call with cmd = %d", cmd);
	}
	LEAVE("leaving iph5526_ioctl");
	return 0;
}


static void elp_testing(struct fc_info *fi, u_int cmd, u_int *buff_addr, u_short received_ox_id)
{
u_int eport_name[2];
	
	eport_name[0] = ntohl(*(buff_addr + 12 ));
	eport_name[1] = ntohl(*(buff_addr + 13 ));
	switch(cmd) {
		case 1: 	// rx higher eport name
			fi->g.mysw_name[0] = eport_name[0];
			fi->g.mysw_name[1] = eport_name[1] + 1;
			tx_elp(1, fi, 0xFFFFFD, received_ox_id);
			break;
                case 2:         // rx higher eport name
                        fi->g.mysw_name[0] = eport_name[0];
                        fi->g.mysw_name[1] = eport_name[1] - 1;
                        tx_elp(1, fi, 0xFFFFFD, received_ox_id);
                        break;
		case 3:		// rx F_BSY
			tx_bsy_rjt(fi, 0xFFFFFD, received_ox_id, F_BSY);
			break;
                case 4:         // rx P_BSY
                        tx_bsy_rjt(fi, 0xFFFFFD, received_ox_id, P_BSY);
                        break;
                case 5:         // rx F_RJT
                        tx_bsy_rjt(fi, 0xFFFFFD, received_ox_id, F_RJT);
                        break;
                case 6:         // rx P_RJT
                        tx_bsy_rjt(fi, 0xFFFFFD, received_ox_id, P_RJT);
                        break;
		case 7:
			tx_sw_rjt(fi, 0xFFFFFD, received_ox_id, SWRJT_LOGICAL_BUSY, 0);
			break;
		case 8:		// rx_other_frames
            		tx_logi(fi, ELS_PLOGI, 0xFFFFFD);
			tx_prli(fi, 0xFFFFFD);
		        break;
		case 9:
			// rx_no_valid response: doing nothing
                        break;

		default:
			T_MSG("Wrong testing code: %d", cmd);
	}
}


static void efp_testing(struct fc_info *fi, u_int cmd, u_int *buff_addr, u_short received_ox_id)
{
u_int sw_name[2];
u_int temp_name[2];                

        sw_name[0] = ntohl(*(buff_addr + 10 ));
        sw_name[1] = ntohl(*(buff_addr + 11 ));
        switch(cmd) {  
                case 10:         // rx_EFP with zero domain ID list
                        fcsw->g.psw_name[0] = sw_name[0];
                        fcsw->g.psw_name[1] = sw_name[1] - 1;
			tx_efp(0, fi, 0xFFFFFD, received_ox_id);
			fcsw->g.psw_name[1] = sw_name[1] + 1;
			tx_efp(1, fi, 0xFFFFFD, received_ox_id);
                        break;
                case 11:         // rx_EFP with non-zero domain ID list
			fcsw->g.psw_name[0] = sw_name[0];
                        fcsw->g.psw_name[1] = sw_name[1] + 1;
			temp_name[0] = sw_name[0];
			temp_name[1] = sw_name[1] + 1;
			add_to_domain_ids(4, temp_name);
			tx_efp(1, fi, 0xFFFFFD, received_ox_id);
                        break;
                case 12:         // rx non_zero_list_rx_EFP
			fcsw->g.psw_name[0] = sw_name[0];
                        fcsw->g.psw_name[1] = sw_name[1] + 1;
                        temp_name[0] = sw_name[0];
                        temp_name[1] = sw_name[1] + 1;         
                        add_to_domain_ids(4, temp_name);
                        tx_efp(0, fi, 0xFFFFFD, received_ox_id);
			// then clear the domain ID list
			while (fi->domain_ids) {
        			struct domain_id *temp = fi->domain_ids;
                		fi->domain_ids = fi->domain_ids->next;
               			 kfree(temp);
        		}
			// and tx efp again
			tx_efp(1, fi, 0xFFFFFD, received_ox_id);
                        break;
               	case 13:	// rx_EFP_no_overlap
			fcsw->g.psw_name[0] = sw_name[0];
                        fcsw->g.psw_name[1] = sw_name[1] - 1;
                        temp_name[0] = sw_name[0];
                        temp_name[1] = sw_name[1] - 1;
                        add_to_domain_ids(4, temp_name);
                        tx_efp(1, fi, 0xFFFFFD, received_ox_id);
			add_to_ox_id_list(fi, fi->g.ox_id - 1, SW_EFP, EFP_TESTING_CASE13);
                        break;
		case 14:	// rx_EFP_overlap
			fcsw->g.psw_name[0] = sw_name[0];
                        fcsw->g.psw_name[1] = sw_name[1] - 1;
                        temp_name[0] = sw_name[0];
                        temp_name[1] = sw_name[1] - 1;
                        add_to_domain_ids(4, temp_name);
                        tx_efp(1, fi, 0xFFFFFD, received_ox_id);
                        add_to_ox_id_list(fi, fi->g.ox_id - 1, SW_EFP, EFP_TESTING_CASE14);
                        break;  
                default:
                        T_MSG("Wrong testing code: %d", cmd);
        }
}
          

static void tx_bsy_rjt(struct fc_info *fi, u_int d_id, u_short ox_id, u_int cmd)
{
u_int r_ctl = 0;
int int_required = 1;
u_int type = 0;
u_int my_mtu = fi->g.my_mtu;

 fi->g.my_id = FABRIC_CONTROLLER;

	switch(cmd) {
		case F_BSY:
			r_ctl = F_BSY;
			type = 0x16D80000;
			break;
		case P_BSY:
                        r_ctl = P_BSY;
                        type = 0x14D80000;
                        break;
                case F_RJT:
                        r_ctl = F_RJT;
                        type = 0x15D80000;
                        break;
                case P_RJT:
                        r_ctl = P_RJT;
                        type = 0x15D80000;
                        break;

		default:
			T_MSG("Wrong BSY or RJT code: %d", cmd);
	}


        tx_exchange(fi, NULL, 0, r_ctl, type, d_id, my_mtu, int_required, ox_id, 00);
}



static int tx_exchange(struct fc_info *fi, char *data, u_int len, u_int r_ctl, u_int type, u_int d_id, u_int mtu, int int_required, u_short tx_ox_id, u_int frame_class)
{
u_char df_ctl; 
int NW_flag = 0, return_value;
u_short rx_id = RX_ID_FIRST_SEQUENCE;
u_int tachyon_status;
u_int my_id = fi->g.my_id;
	ENTER("tx_exchange");

	tachyon_status = readl(fi->t_r.ptr_tach_status_reg);
	DPRINTK("Tachyon Status = %x len = %d MTU = %d", tachyon_status, len, mtu);
	if (tachyon_status & OSM_FROZEN) {
		reset_tachyon(fi, ERROR_RELEASE);
		reset_tachyon(fi, OCQ_RESET);
		DPRINTK("Tachyon Status = %x len = %d MTU = %d", tachyon_status, len, mtu);
	}
	if (tx_ox_id == OX_ID_FIRST_SEQUENCE) {
				tx_ox_id = fi->g.ox_id;
	}
	else {
		switch(fi->g.type_of_frame) {
			case FC_ELS:
	                case FC_BLS:
                                rx_id = RX_ID_FIRST_SEQUENCE;
                                break;
			case FC_SW:
                                rx_id = RX_ID_FIRST_SEQUENCE;
                                break;

			default:
				//rx_id = 0x8005;
				rx_id = fi->g.ox_id;
				break;
		}
	}

	/* If it is a SCSI Read/Write, add the OX_ID to the active 
	 * list of OX_IDs for that Target. Needed for Error Recovery.
	 */

		df_ctl = 0;
		/* Multi Frame Sequence ? If yes, set RO bit */
		if (len > mtu)
			type |= RELATIVE_OFF_PRESENT;
		build_tachyon_header(fi, my_id, r_ctl, d_id, type, fi->g.seq_id, df_ctl, tx_ox_id, rx_id, NULL);

	/* Get free Tachyon Headers and EDBs */
	if ( get_free_EDB(fi))
		return 0;

		memcpy(fi->q.ptr_tachyon_header[fi->q.tachyon_header_indx], &(fi->g.tach_header), HEADER_LEN);

	return_value = tx_sequence(fi, data, len, mtu, d_id, tx_ox_id, rx_id, fi->g.seq_id, NW_flag, int_required, frame_class);
	DPRINTK("OX_ID = %x, alpa_list_indx = %x", fi->g.ox_id, fi->g.alpa_list_index);

			fi->g.ox_id++;
			if (fi->g.ox_id == 0xFFFF)
				fi->g.ox_id = NOT_SCSI_XID;

	if (fi->g.seq_id == MAX_SEQ_ID)
		fi->g.seq_id = 0;
	else
		fi->g.seq_id++;

	 update_tachyon_header_indx(fi);
	LEAVE("tx_exchange");
	return return_value;
}

static int tx_sequence(struct fc_info *fi, char *data, u_int len, u_int mtu, u_int d_id, u_short ox_id, u_short rx_id, u_char seq_id, int NW_flag, int int_required, u_int frame_class)
{
u_int cntl = 0;
int return_value;
	ENTER("tx_sequence");
	build_EDB(fi, data, EDB_END_BIT, len);
	memcpy(fi->q.ptr_edb[fi->q.edb_buffer_indx], &(fi->g.edb), sizeof(EDB));
	build_ODB(fi, seq_id, d_id, len, cntl, mtu, ox_id, rx_id, NW_flag, int_required, frame_class);
	memcpy(fi->q.ptr_odb[fi->q.ocq_prod_indx], &(fi->g.odb), sizeof(ODB));
	if(fi->g.i_am_E_Port == TRUE) 
	{   		
		if(fi->g.ptp_up != TRUE) {
			T_MSG("E_Port: tx_sequence: Fibre Channel Link not up. Dropping Exchange!");
	                return_value = FALSE;
		}
		else {
			update_OCQ_indx(fi);
                	return_value = TRUE;
		}
        }
	else if ((fi->g.loop_up != TRUE) && (fi->g.ptp_up != TRUE)) {
		T_MSG("tx_sequence: Fibre Channel Link not up. Dropping Exchange!");
		return_value = FALSE;
	}
	else {
		/* To be on the safe side, a check should be included
		 * at this point to check if we are overrunning 
		 * Tachyon.
		 */
		update_OCQ_indx(fi);
		return_value = TRUE;
	}
	update_EDB_indx(fi);
	update_tachyon_header_indx(fi);
	LEAVE("tx_sequence");
	return return_value;
}

static int get_free_header(struct fc_info *fi)
{
u_short temp_ox_id;
u_int *tach_header, initial_indx = fi->q.tachyon_header_indx;
	/* Check if the header is in use.
	 * We could have an outstanding command.
	 * We should find a free slot as we can queue a
	 * maximum of 32 SCSI commands only. 
	 */
	tach_header = fi->q.ptr_tachyon_header[fi->q.tachyon_header_indx];
	temp_ox_id = ntohl(*(tach_header + 6)) >> 16;
	/* We care about the SCSI writes only. Those are the wicked ones
	 * that need an additional set of buffers.
	 */
	while(temp_ox_id <= MAX_SCSI_XID) {
		update_tachyon_header_indx(fi);
		if (fi->q.tachyon_header_indx == initial_indx) {
			T_MSG("No free Tachyon headers available");
			return 1;
		}
		tach_header = fi->q.ptr_tachyon_header[fi->q.tachyon_header_indx];
		temp_ox_id = ntohl(*(tach_header + 6)) >> 16;
	}
	return 0;
}

static int get_free_EDB(struct fc_info *fi)
{
unsigned int initial_indx = fi->q.edb_buffer_indx;
	/* Check if the EDB is in use.
	 * We could have an outstanding SCSI Write command.
	 * We should find a free slot as we can queue a
	 * maximum of 32 SCSI commands only. 
	 */
	while (fi->q.free_edb_list[fi->q.edb_buffer_indx] != EDB_FREE) {
		update_EDB_indx(fi);
		if (fi->q.edb_buffer_indx == initial_indx) {
			T_MSG("No free EDB buffers avaliable")
			return 1;
		}
	}
	return 0;
}		


static void add_to_address_cache(struct fc_info *fi, u_int *base_ptr)
{
int size = sizeof(struct fc_node_info);
struct fc_node_info *p, *q = fi->node_info_list, *r = NULL;
char n_port_name[PORT_NAME_LEN];
u_int s_id;
	ENTER("add_to_address_cache");
	/*index to Port Name in the payload. We need the 8 byte Port Name */
	memcpy(n_port_name, base_ptr + 13, PORT_NAME_LEN);
	s_id = ntohl(*(base_ptr + 3)) & 0x00FFFFFF;
	
	/* check if info already exists */
	while(q != NULL) {	
		if (memcmp(n_port_name, q->hw_addr, PORT_NAME_LEN) == 0) {
			if (s_id != q->d_id) {
				memcpy(&(q->c_of_s[0]), base_ptr + 17, 3 * sizeof(CLASS_OF_SERVICE));
				q->mtu = ntohl(*(base_ptr + 10)) & 0x00000FFF;
				q->d_id = s_id;
				memcpy(q->node_name, base_ptr + 15, NODE_NAME_LEN);
			}
			q->login = LOGIN_COMPLETED;
			q->scsi = FALSE;
			display_cache(fi);
			return;
		}
		r = q;
		q = q->next;
	}
	p = (struct fc_node_info *)kmalloc(size, GFP_DMA | GFP_ATOMIC);
	if (p == NULL) {
		T_MSG("kmalloc failed in add_to_address_cache()");
		return;
	}
	memcpy(&(p->c_of_s[0]), base_ptr + 17, 3 * sizeof(CLASS_OF_SERVICE));
	p->mtu = ntohl(*(base_ptr + 10)) & 0x00000FFF;
	p->d_id = s_id;
	memcpy(p->hw_addr, base_ptr + 13, PORT_NAME_LEN);
	memcpy(p->node_name, base_ptr + 15, NODE_NAME_LEN);
	p->login = LOGIN_COMPLETED;
	p->scsi = FALSE;
	p->target_id = 0xFF;
	p->active_oxid_list = NULL;
	p->next = NULL;
	if (fi->node_info_list == NULL)
		fi->node_info_list = p;
	else
		r->next = p;
	display_cache(fi);
	LEAVE("add_to_address_cache");
	return;
}


static int display_cache(struct fc_info *fi)
{
struct fc_node_info *q = fi->node_info_list;
struct ox_id_els_map *temp_ox_id_list = fi->ox_id_list;
int count = 0, j;
	while(q != NULL) {
		printk("WWN = ");
		for (j = 0; j < PORT_NAME_LEN; j++)
			printk("%x ", q->hw_addr[j]); 
		printk("\n");
		printk("d_id = %x", q->d_id);
		printk(" target_id = %d", q->target_id);
		printk(" mtu = %x", q->mtu);
		printk(" login = %x", q->login);
		printk(" scsi = %x\n", q->scsi);
		q = q->next;
		count++;
	}
	printk("OX_ID -> ELS Map\n");
	while(temp_ox_id_list) {
			printk("ox_id = %x, ELS = %x\n", temp_ox_id_list->ox_id, temp_ox_id_list->els);
			temp_ox_id_list = temp_ox_id_list->next;
	}
	printk("My ID = %x, My WWN = %x %x, Number of entries in CACHE = %d\n", fi->g.my_id, fi->g.my_node_name_high, fi->g.my_node_name_low, count);
	if (fi->g.ptp_up == TRUE)
		DEBUG(" Port type = N_Port");
	if (fi->g.loop_up == TRUE)
		DEBUG(" Port type = L_Port");
	return 0;
}

static struct net_device_stats * iph5526_get_stats(struct device *dev)
{	
struct fc_info *fi = (struct fc_info*)dev->priv; 
	ENTER("iph5526_get_stats");
	LEAVE("iph5526_get_stats");
	return (struct net_device_stats *) &fi->fc_stats;
}

#if DEBUG_5526
static void print_out_registers(struct fc_info *fi, int from, int to)
{
int i;

	for ( i = from; i <= to; i += 4 )
	  	printk("value at %x is %x\n", i, readl(fi->g.tachyon_base + i));
}
#endif




/*************************************************************************************************************/
#ifdef MODULE

#define NAMELEN		8	/* # of chars for storing dev->name */

static struct device *dev_fc[MAX_FC_CARDS];

static int io =  0;
static int irq  = 0;
static int bad  = 0;	/* 0xbad = bad sig or no reset ack */

int init_module(void)
{
int i, present = FALSE;
    
	if(MAX_FC_CARDS < 3) {
		printk(KERN_INFO "In order to run the full functionalities of the switch driver, we suggest you have at least 3 HBAs installed.\n");
		printk(KERN_INFO "Now you defined MAX_FC_CARDS = %x \n", MAX_FC_CARDS);
		printk(KERN_INFO "MAX_FC_CARDS is defined in file include/linux/fcdevice.h\n");
	}

	for (i = 0; i <= MAX_FC_CARDS; i++) 
		fc[i] = NULL;
	for (i = 0; i < MAX_FC_CARDS; i++) {
		fc[i] = kmalloc(sizeof(struct fc_info), GFP_KERNEL);
		if (fc[i] == NULL) {
			if (i == 0)
				return -ENOMEM;
			else {
				printk("iph5526.c: Unable to register card # %d\n", i);
				break;
			}
		}
		memset(fc[i], 0, sizeof(struct fc_info));
		dev_fc[i] = NULL;
		printk("Init_module: before init_fcdev===================\n");
		dev_fc[i] = init_fcdev(dev_fc[i], 0);/* call to fn in net_init.c*/	
                printk("Init_module: after init_fcdev====================\n");
		dev_fc[i]->irq = irq;
		dev_fc[i]->mem_end = bad;
		dev_fc[i]->base_addr = io;
		dev_fc[i]->init = iph5526_probe;
		dev_fc[i]->priv = fc[i];
		fc[i]->dev = dev_fc[i];
		if (register_fcdev(dev_fc[i]) == 0) {
			if (!initialize_register_pointers(fc[i]) || !tachyon_init(fc[i])) {
				printk("iph5526.c: TACHYON initialization failed for card # %d!!!\n", i);
				if (fc[i]) 
					clean_up_memory(fc[i]);
				if (i == 0)
					return -ENOMEM;
				break;
			}
		}
		else {
			kfree_s(dev_fc[i], sizeof(struct device));
			dev_fc[i] = NULL;
			kfree_s(fc[i], sizeof(struct fc_info));
			fc[i] = NULL;
			if (i == 0) {
				printk("iph5526: IP registeration failed!!!\n");
				return -1;
			}
		}
		present = TRUE;
	}
	if (!present)
		return -1;

	return 0;
}

void cleanup_module(void)
{
int i;
	for (i = 0; i < MAX_FC_CARDS; i++) {
		writel(0x0, fc[i]->i_r.ptr_ichip_hw_control_reg);
		if (dev_fc[i]) {
		struct fc_info *fi = (struct fc_info*)dev_fc[i]->priv;
		void *priv = dev_fc[i]->priv;
			fi->g.dont_init = TRUE;
			if(fi->rt.hello_timer_set == 1)
				del_timer(&fi->rt.hello_timer);
			if(fi->rt.dead_timer_set == 1)
                                del_timer(&fi->rt.dead_timer);
			if(fi->sm.elp_timer_set == 1)
                                del_timer(&fi->sm.elp_timer);
			if(fi->sm.efp_timer_set == 1)
                                del_timer(&fi->sm.efp_timer);
			 if(fi->sm.bf_timer_set == 1)
                                del_timer(&fi->sm.bf_timer);

 			take_tachyon_offline(fi);
			unregister_fcdev(dev_fc[i]);
			free_irq(dev_fc[i]->irq, dev_fc[i]);
			iounmap(fi->g.mem_base);
			clean_up_memory(fi);
			if (dev_fc[i]->priv)
				kfree(priv);
			kfree(dev_fc[i]);
			dev_fc[i] = NULL;
		}
	}
}
#endif /* MODULE */

void clean_up_memory(struct fc_info *fi)
{
int i,j;
	ENTER("clean_up_memory");
	if (fi->q.ptr_mfsbq_base)
		free_pages((u_long)bus_to_virt(ntohl(*(fi->q.ptr_mfsbq_base))), 5);
	DPRINTK("after kfree2");
	for (i = 0; i < SFSBQ_LEN; i++)
		for (j = 0; j < BUFF_ADDRS; j++)
			if (fi->q.ptr_sfs_buffers[i*BUFF_ADDRS + j])
				kfree(fi->q.ptr_sfs_buffers[i*BUFF_ADDRS + j]);
	DPRINTK("after kfree1");
	if (fi->q.ptr_ocq_base)
		free_page((u_long)fi->q.ptr_ocq_base);
	if (fi->q.ptr_imq_base)
		free_page((u_long)fi->q.ptr_imq_base);
	if (fi->q.ptr_mfsbq_base)
		free_page((u_long)fi->q.ptr_mfsbq_base);
	if (fi->q.ptr_sfsbq_base)
		free_page((u_long)fi->q.ptr_sfsbq_base);
	if (fi->q.ptr_edb_base)
		free_pages((u_long)fi->q.ptr_edb_base, 3);
	if (fi->q.ptr_sest_base)
		free_page((u_long)fi->q.ptr_sest_base);
	if (fi->q.ptr_tachyon_header_base)
		free_page((u_long)fi->q.ptr_tachyon_header_base);
	if (fi->q.ptr_sdb_base)
		free_pages((u_long)fi->q.ptr_sdb_base, 1);
	if (fi->q.ptr_fcp_cmnd_base)
		free_page((u_long)fi->q.ptr_fcp_cmnd_base);
	DPRINTK("after free_pages"); 
	if (fi->q.ptr_host_ocq_cons_indx)
		kfree(fi->q.ptr_host_ocq_cons_indx);
	if (fi->q.ptr_host_hpcq_cons_indx)
		kfree(fi->q.ptr_host_hpcq_cons_indx);
	if (fi->q.ptr_host_imq_prod_indx)
		kfree(fi->q.ptr_host_imq_prod_indx);
	DPRINTK("after kfree3");
	while (fi->node_info_list) {
	struct fc_node_info *temp_list = fi->node_info_list;
		fi->node_info_list = fi->node_info_list->next;
		while (temp_list->active_oxid_list) {
		struct active_oxids *q = temp_list->active_oxid_list;
			temp_list->active_oxid_list = temp_list->active_oxid_list->next;
			kfree(q);
		}
		kfree(temp_list);
	}
	while (fi->ox_id_list) {
	struct ox_id_els_map *temp = fi->ox_id_list;
		fi->ox_id_list = fi->ox_id_list->next;
		kfree(temp);
	}
	while (fi->name_list) {
        struct port_info *temp = fi->name_list;
                fi->name_list = fi->name_list->next;
                kfree(temp);
        }
        while (fi->domain_ids) {
        struct domain_id *temp = fi->domain_ids; 
                fi->domain_ids = fi->domain_ids->next;
                kfree(temp);
        }
	while(fi->rt.lsr_list) {
	struct lsr_info *temp = fi->rt.lsr_list;
		while(temp->link_list) {
		struct link_info *ltemp = temp->link_list;
			temp->link_list = temp->link_list->next;
			kfree(ltemp);
		}
		fi->rt.lsr_list = fi->rt.lsr_list->next;
		kfree(temp);
	}
	LEAVE("clean_up_memory");
}

static int initialize_register_pointers(struct fc_info *fi)
{
	ENTER("initialize_register_pointers");
    if(fi->g.tachyon_base == 0)
      return -ENOMEM; 
	
//cwl	DPRINTK("TACHYON_BASE= %x",fi->g.tachyon_base);

fi->i_r.ptr_ichip_hw_control_reg	= ICHIP_HW_CONTROL_REG_OFF + fi->g.tachyon_base;
fi->i_r.ptr_ichip_hw_status_reg = ICHIP_HW_STATUS_REG_OFF + fi->g.tachyon_base;
fi->i_r.ptr_ichip_hw_addr_mask_reg = ICHIP_HW_ADDR_MASK_REG_OFF + fi->g.tachyon_base;
fi->t_r.ptr_ocq_base_reg = OCQ_BASE_OFF + fi->g.tachyon_base;
fi->t_r.ptr_ocq_len_reg = OCQ_LEN_OFF + fi->g.tachyon_base;
fi->t_r.ptr_ocq_prod_indx_reg = OCQ_PROD_INDX_OFF + fi->g.tachyon_base;
fi->t_r.ptr_ocq_cons_indx_reg = OCQ_CONS_ADDR_OFF + fi->g.tachyon_base;
fi->t_r.ptr_hpcq_base_reg = HPCQ_BASE_OFF + fi->g.tachyon_base;
fi->t_r.ptr_hpcq_len_reg = HPCQ_LEN_OFF + fi->g.tachyon_base;
fi->t_r.ptr_hpcq_prod_indx_reg = HPCQ_PROD_INDX_OFF + fi->g.tachyon_base;
fi->t_r.ptr_hpcq_cons_indx_reg = HPCQ_CONS_ADDR_OFF + fi->g.tachyon_base;
fi->t_r.ptr_imq_base_reg = IMQ_BASE_OFF + fi->g.tachyon_base;
fi->t_r.ptr_imq_len_reg = IMQ_LEN_OFF + fi->g.tachyon_base;
fi->t_r.ptr_imq_cons_indx_reg = IMQ_CONS_INDX_OFF + fi->g.tachyon_base;
fi->t_r.ptr_imq_prod_indx_reg = IMQ_PROD_ADDR_OFF + fi->g.tachyon_base;
fi->t_r.ptr_mfsbq_base_reg = MFSBQ_BASE_OFF + fi->g.tachyon_base;
fi->t_r.ptr_mfsbq_len_reg = MFSBQ_LEN_OFF + fi->g.tachyon_base;
fi->t_r.ptr_mfsbq_prod_reg = MFSBQ_PROD_INDX_OFF + fi->g.tachyon_base;
fi->t_r.ptr_mfsbq_cons_reg = MFSBQ_CONS_INDX_OFF + fi->g.tachyon_base;
fi->t_r.ptr_mfsbuff_len_reg = MFS_BUFF_LEN_OFF + fi->g.tachyon_base;
fi->t_r.ptr_sfsbq_base_reg = SFSBQ_BASE_OFF + fi->g.tachyon_base;
fi->t_r.ptr_sfsbq_len_reg = SFSBQ_LEN_OFF + fi->g.tachyon_base;
fi->t_r.ptr_sfsbq_prod_reg = SFSBQ_PROD_INDX_OFF + fi->g.tachyon_base;
fi->t_r.ptr_sfsbq_cons_reg = SFSBQ_CONS_INDX_OFF + fi->g.tachyon_base;
fi->t_r.ptr_sfsbuff_len_reg = SFS_BUFF_LEN_OFF + fi->g.tachyon_base;
fi->t_r.ptr_sest_base_reg = SEST_BASE_OFF + fi->g.tachyon_base;
fi->t_r.ptr_sest_len_reg = SEST_LEN_OFF + fi->g.tachyon_base;
fi->t_r.ptr_sestbuff_len_reg = SEST_BUFF_LEN_OFF + fi->g.tachyon_base;
fi->t_r.ptr_tach_config_reg = TACH_CONFIG_OFF + fi->g.tachyon_base;
fi->t_r.ptr_tach_control_reg = TACH_CONTROL_OFF + fi->g.tachyon_base;
fi->t_r.ptr_tach_status_reg = TACH_STATUS_OFF + fi->g.tachyon_base;
fi->t_r.ptr_tach_flush_oxid_reg = TACH_FLUSH_OXID_OFF + fi->g.tachyon_base;
fi->t_r.ptr_tach_ee_cr_0_tmr_reg = TACH_EE_CR_0_TMR_OFF + fi->g.tachyon_base;
fi->t_r.ptr_tach_bb_cr_0_tmr_reg = TACH_BB_CR_0_TMR_OFF + fi->g.tachyon_base;
fi->t_r.ptr_tach_rx_frm_err_reg = TACH_RX_FRM_ERR_OFF + fi->g.tachyon_base;
fi->t_r.ptr_fm_config_reg = FM_CONFIG_OFF + fi->g.tachyon_base;
fi->t_r.ptr_fm_control_reg = FM_CONTROL_OFF + fi->g.tachyon_base;
fi->t_r.ptr_fm_status_reg = FM_STATUS_OFF + fi->g.tachyon_base;
fi->t_r.ptr_fm_tov_reg = FM_TOV_OFF + fi->g.tachyon_base;
fi->t_r.ptr_fm_link_stat_1_reg = FM_LINK_STAT_1_OFF + fi->g.tachyon_base;
fi->t_r.ptr_fm_link_stat_2_reg = FM_LINK_STAT_2_OFF + fi->g.tachyon_base;
fi->t_r.ptr_fm_wwn_hi_reg = FM_WWN_HI_OFF + fi->g.tachyon_base;
fi->t_r.ptr_fm_wwn_low_reg = FM_WWN_LO_OFF + fi->g.tachyon_base;
fi->t_r.ptr_fm_rx_al_pa_reg = FM_RX_AL_PA_OFF + fi->g.tachyon_base;
fi->t_r.ptr_fm_prim_reg = FM_PRIM_REG_OFF + fi->g.tachyon_base;

	LEAVE("initialize_register_pointers");
	return 1;
}








/*
 * Local variables:
 *  compile-command: "gcc -DKERNEL -Wall -O6 -fomit-frame-pointer -I/usr/src/linux/net/tcp -c iph5526.c"
 *  version-control: t
 *  kept-new-versions: 5
 * End:
 */
// end of this file







// end of this file added








// end of this file again











// protection
